/* $Id: arpc.h,v 1.1 1998/12/07 20:42:32 fubobb Exp $ */

#ifndef _ARPC_H_
#define _ARPC_H_

#include <rpc/rpc.h>

#include "arpc/misc.h"
#include "arpc/queue.h"

/* Dispatch tables in .i files */
#define RPCGEN_ACTION(x) 0
struct rpcgen_table {
  char *(*proc)();
  xdrproc_t xdr_arg;
  unsigned len_arg;
  xdrproc_t xdr_res;
  unsigned len_res;
};
/* XXX should somehow use PROG_VERSION */
#define rpctbl(prog, vers)			\
	prog##_##vers##_table,			\
	prog##_##vers##_nproc
/* #define svc_req svccb */

/* Transport stuff */
struct asvc;
struct svccb;
struct axh;
struct axprt {
  char ax_reliable;          /* Set for unreliable transports */
  void (*ax_sendv) (struct axh *, struct iovec *, int, void *);
  void (*ax_free) (struct axh *);
  void (*ax_setcb) (struct axh *,
		    void (*) (void *, char *, int, void *, int),
		    void *);
  void (*ax_encrypt) (struct axh *, int, char *, int);
};
struct axh {
  const struct axprt *x;
  char connected;              /* Set for connected handles */
};
#define xh_sendv(xh, uio, cnt, sap) (xh)->x->ax_sendv(xh, uio, cnt, sap)
#define xh_send(xh, data, len, sap)		\
  do {struct iovec iov;				\
      iov.iov_base = (void *) data;		\
      iov.iov_len = len;			\
      xh_sendv(xh, &iov, 1, sap);		\
  } while (0)
#define xh_free(xh) (xh)->x->ax_free (xh)
#define xh_encrypt(xh, server, key, keylen) \
  (xh)->x->ax_encrypt (xh, server, key, keylen)
#define xh_isreliable(xh) (xh)->x->ax_reliable
#define xh_isconnected(xh) (xh)->connected
#define xh_setcb(xh, cb, arg) (xh)->x->ax_setcb(xh, cb, arg)

extern const struct axprt axprt_dx;  /* Datagram transport */
extern const struct axprt axprt_ps;  /* [Encrypting] Packet stream */
extern const struct axprt axprt_us;  /* Unix domain packet stream */
extern struct axh *dx_alloc (int);
extern struct axh *ps_alloc (int);
extern struct axh *us_alloc (int);

/* Type specific functionality */
void ps_unrecv (struct axh *, void *, int);
void us_unrecv (struct axh *, void *, int);
int ps_getfd (struct axh *);
void ps_ncfree (struct axh *);
#ifdef HAVE_FDPASS
int us_recvfd (struct axh *);
void us_sendfd (struct axh *, int);
#endif /* HAVE_FDPASS */

/* Client stuff */
typedef int (*clnt_cb) (void *, enum clnt_stat, void *);
struct aclnt;

void arpcclient_init (void);

void aclnt_free (struct aclnt *c);
struct axh *aclnt_reclaim (struct aclnt *c);
void aclnt_call (struct aclnt *c, AUTH *auth, u_int32_t procnum,
		 void *in, void *out, clnt_cb cb, void *arg);
void aclnt_call_xdrproc (struct aclnt *c, AUTH *auth, u_int32_t procnum,
			 xdrproc_t inproc, void *in,
			 xdrproc_t outproc, void *out,
			 clnt_cb cb, void *arg);
void aclnt_seteof (struct aclnt *, void (*) (void *), void *);
void aclnt_encrypt (struct aclnt *, char *, int);
struct aclnt *aclnt_rawudp_create (struct sockaddr_in *sinp,
				   u_int32_t prog, u_int32_t vers,
				   struct rpcgen_table *xdr_tbl, u_int nproc);
struct aclnt *aclnt_axh_create (struct axh *xh,
				u_int32_t prog, u_int32_t vers,
				struct rpcgen_table *xdr_tbl, u_int nproc,
				void (*eofcb) (void *), void *eofarg);
void aclntudp_create (char *host, u_short port, u_int32_t prog, u_int32_t vers,
		      struct rpcgen_table *xdr_tbl, u_int nproc,
		      void (*cb) (void *, struct aclnt *), void *arg);
void aclnttcp_create (char *host, u_short port, u_int32_t prog, u_int32_t vers,
		      struct rpcgen_table *xdr_tbl, u_int nproc,
		      void (*eofcb) (void *), void *eofarg,
		      void (*cb) (void *, struct aclnt *), void *arg);


void acallrpc (char *host, AUTH *auth,
	       u_int32_t prognum, u_int32_t versnum, u_int32_t procnum,
	       xdrproc_t inproc, void *in, xdrproc_t outproc, void *out,
	       int (*cb) (void *, enum clnt_stat, void *), void *arg);

void apmap_getport (struct in_addr ipaddr,
		    u_int32_t program, u_int32_t version, u_int protocol,
		    void (*cb) (void *, u_short), void *arg);


#define ACFREE_NONE 0
#define ACFREE_XDR 1
#define ACFREE_ALL 2

/* Internal use only */
void aclnt_reap (void *, char *, int, void *, int);

struct asvcprog;

struct svccb {
  struct rpc_msg *sb_msg;           /* Incoming message header */
  void *sb_arg;                     /* Memory for argument */
  struct authunix_parms *sb_aup;    /* RPC authunix parms (or NULL) */
  struct asvcprog *sb_svcprog;      /* Server struct */
  struct sockaddr_in sb_sin;        /* Address of request */
  char *sb_rdat;                    /* Reply (for replay cache) */
  u_int sb_rlen;                    /* Length of reply */
  LIST_ENTRY (svccb) sb_hlink;      /* Replay hash bucket */
  TAILQ_ENTRY (svccb) sb_tlink;     /* LRU hash bucket */
#define sb_xid sb_msg->rm_xid
#define sb_progno sb_msg->rm_call.cb_prog
#define sb_versno sb_msg->rm_call.cb_vers
#define sb_procno sb_msg->rm_call.cb_proc
};

struct asvc *asvc_create (struct axh *,
			  void (*) (void *, struct svccb *), void *);
void asvc_serve (struct asvc *ap,
		 u_int32_t prog, u_int32_t vers,
		 struct rpcgen_table *xdr_tbl, u_int nproc);
struct aclnt *asvc_mkclnt (struct asvc *ap, u_int32_t prog, u_int32_t vers,
			   struct rpcgen_table *xdr_tbl, u_int nproc);
void asvc_reply (struct svccb *, void *);
void asvc_reply_xdrproc (struct svccb *, xdrproc_t, void *);
void asvc_reply_raw (struct svccb *sbp, struct iovec *iov, int iovcnt);
void asvc_noreply (struct svccb *);
void asvc_free (struct asvc *);
void asvc_encrypt (struct asvc *, char *, int);
void asvc_encrypt_client (struct asvc *, char *, int);
void asvc_reply_suio (struct svccb *, struct suio *);
#ifdef HAVE_FDPASS
void asvc_clone (struct asvc *, struct svccb *, struct axh *);
#endif /* HAVE_FDPASS */

void tcp_server (int fd,
		 void (*cb) (void *, struct axh *, struct sockaddr_in *, int),
		 void *arg);
#ifdef HAVE_FDPASS
void clone_server (int fd,
		   void (*cb) (void *, struct axh *,
			       struct sockaddr_in *, int),
		   void *arg);
#endif /* HAVE_FDPASS */
void start_server (int port,
		   void (*cb) (void *, struct axh *,
			       struct sockaddr_in *, int),
		   void *arg);



/* Random auxilliary functions */
bool_t xdr_flatten (xdrproc_t, void *, char **, u_int *);
bool_t xdr_sha1 (u_char hash[20], xdrproc_t proc, void *objp);

#endif /* ! _ARPC_H_ */
