/* $Id: axprt_dx.c,v 1.1 1998/12/07 20:42:40 fubobb Exp $ */

/* Simple UDP datagram transport */

#include "sysconf.h"

#include "misc.h"
#include "arpc.h"

struct dx {
  struct axh dx_axh;
  int dx_fd;
  void (*dx_recv_fn) (void *, char *, int, void *, int);
  void *dx_recv_arg;
};

static void dx_input (void *);
static void dx_sendv (struct axh *, struct iovec *, int, void *);
static void dx_free (struct axh *);
static void dx_encrypt (struct axh *, int, char *, int);
static void dx_setcb (struct axh *,
		      void (*) (void *, char *, int, void *, int), void *);

const struct axprt axprt_dx = {
  0, dx_sendv, dx_free, dx_setcb, dx_encrypt
};

struct axh *
dx_alloc (int fd)
{
  struct dx *dxp;

  dxp = xmalloc (sizeof (*dxp));
  dxp->dx_axh.x = &axprt_dx;
  dxp->dx_axh.connected = 0;
  dxp->dx_fd = fd;
  dxp->dx_recv_fn = NULL;
  dxp->dx_recv_arg = NULL;
  cb_add (fd, 0, dx_input, dxp);
  return &dxp->dx_axh;
}

static void
dx_free (struct axh *_dxp)
{
  struct dx *dxp = (struct dx *) _dxp;

  cb_free (dxp->dx_fd, 0);
  cb_free (dxp->dx_fd, 1);
  close (dxp->dx_fd);
  xfree (dxp);
}

static void
dx_sendv (struct axh *_dxp, struct iovec *iov, int cnt, void *sinp)
{
  struct dx *dxp = (struct dx *) _dxp;
  struct msghdr mh;
  
  bzero (&mh, sizeof (mh));
  mh.msg_name = sinp;
  mh.msg_namelen = sizeof (struct sockaddr_in);
  mh.msg_iov = iov;
  mh.msg_iovlen = cnt;

  if (sendmsg (dxp->dx_fd, &mh, 0) < 0)
    warn ("dx_send: sendto: %s\n", strerror (errno));
}

static void
dx_input (void *_dxp)
{
  struct dx *dxp = _dxp;
  char buf[UDPMSGSIZE + 1];
  struct sockaddr_in sin;
  int sinlen = sizeof (sin);
  int n;

  n = recvfrom (dxp->dx_fd, buf, sizeof (buf), 0,
		(struct sockaddr *) &sin, &sinlen);
  if (n < 0)
    warn ("dx_input: %s\n", strerror (errno));
  else if (sinlen != sizeof (sin) || n > UDPMSGSIZE)
    warn ("dx_input: tossing oversized packet\n");
  else if (dxp->dx_recv_fn)
    dxp->dx_recv_fn (dxp->dx_recv_arg, buf, n, &sin, sinlen);
}

static void
dx_encrypt (struct axh *_dxp, int invert, char *key, int keylen)
{
  fatal ("dx_encrypt: encryption not supported by transport layer.\n");
}

static void
dx_setcb (struct axh *_dxp,
	  void (*fn) (void *, char *, int, void *, int), void *arg)
{
  struct dx *dxp = (struct dx *) _dxp;
  dxp->dx_recv_fn = fn;
  dxp->dx_recv_arg = arg;
}
