/* $Id: backoff.h,v 1.1 1998/12/07 20:42:44 fubobb Exp $ */

#include <time.h>
#include "misc.h"
#include "queue.h"

#define MINTO 2                 /* Time to wait before first retransmit */
#define MAXSEND 5               /* Maximum number of times to transmit */

#define TMOQ_ENTRY(type)			\
struct {					\
  u_char tm_qno;				\
  time_t tm_time;				\
  TAILQ_ENTRY (type) tm_link;			\
}

#define TMOQ_HEAD(type)				\
struct {					\
  TAILQ_HEAD (, type) tmoq[MAXSEND];		\
  char tmop[MAXSEND];				\
}

#define TMOQ_INIT(head)				\
{						\
  int n;					\
  for (n = 0; n < MAXSEND; n++) {		\
    (head).tmop[n] = 0;				\
    TAILQ_INIT (&(head).tmoq[n]);		\
  }						\
}

/* Schedule a timeout if we need one */
#define TMOQ_TMO_SCHED(head, type, field, function)		\
{								\
  int n;							\
  for (n = 0; n < MAXSEND; n++) {				\
    struct type *p;						\
    if ((head).tmop[n])						\
      break;							\
    p = (head).tmoq[n].tqh_first;				\
    if (p) {							\
      (head).tmop[n] = 1;					\
      tmo_add (p->field.tm_time, function, &(head).tmop[n]);	\
    }								\
  }								\
}

#define TMOQ_REMOVE(head, elm, field)			\
{							\
  TAILQ_REMOVE (&(head).tmoq[(elm)->field.tm_qno], elm,	\
		field.tm_link);				\
}

#define TMOQ_ADD(head, type, elm, field, function)		\
{								\
  (elm)->field.tm_qno = 0;					\
  (elm)->field.tm_time = time (NULL) + MINTO;			\
  TAILQ_INSERT_TAIL (&(head).tmoq[0], elm, field.tm_link);	\
  TMOQ_TMO_SCHED(head, type, field, function)			\
}

#define TMOQ_RUN(head, type, field, now, resend, expire,		\
		 cb_func, cb_arg)					\
{									\
  const time_t t = (now);						\
  struct type *p, *next_p;						\
  int n;								\
									\
  *(char *) (cb_arg) = 0;						\
									\
  /* Resend/expire any requests that have come due */			\
  for (n = MAXSEND - 1; n >= 0; n--)					\
    for (p = (head).tmoq[n].tqh_first;					\
	 p && p->field.tm_time <= t;					\
	 p = next_p) {							\
      next_p = p->field.tm_link.tqe_next;				\
      if (n + 1 == MAXSEND) {						\
	warn ("timeout!\n");						\
	expire (p);							\
      } else {								\
	p->field.tm_time = t + (MINTO << (n + 1));			\
	TAILQ_REMOVE (&(head).tmoq[n], p, field.tm_link);		\
	TAILQ_INSERT_TAIL (&(head).tmoq[n + 1], p, field.tm_link);	\
	p->field.tm_qno = n + 1;					\
	warn (__FUNCTION__ ": resend!\n");				\
	resend (p);							\
      }									\
    }									\
  TMOQ_TMO_SCHED(head, type, field, cb_func)				\
}
