/* $Id: daemonize.c,v 1.1 1998/12/07 20:42:45 fubobb Exp $ */

#include <fcntl.h>
#include <unistd.h>
#include <grp.h>
#include <syslog.h>
#include "misc.h"

static void
forkexit (void)
{
  switch (fork ()) {
  case -1:
    fatal ("fork: %s\n", strerror (errno));
  case 0:
    break;
  default:
    exit (0);
  }
}

static void
logger (void)
{
  int size = 4;
  int used = 0;
  int n;
  char *buf = xmalloc (size);
  char *p, *q;

  openlog (PACKAGE, LOG_CONS, LOG_DAEMON);
  for (;;) {
    n = read (0, buf + used, size - used);
    fprintf (stderr, "n = %d\n", n);
    if (n <= 0)
      exit (0);
    used += n;

    for (p = buf;
	 p < buf + size && (q = memchr (p, '\n', used - (p - buf)));
	 p = q + 1) {
      *q = '\0';
      fprintf (stderr, "TO syslog: %s\n", p);
      syslog (LOG_NOTICE, "%s\n", p);
    }
    used -= (p - buf);
    if (used && p != buf)
      memmove (buf, p, used);
    else if (used == size) {
      size *= 2;
      buf = xrealloc (buf, size);
    }
  }
}

/* Can syslog() block?  Don't risk it.  Fork off an asynchronous
 * process to do all the logging. */
static void
makelogger (void)
{
  int fds[2];

  if (pipe (fds) < 0)
    fatal ("pipe: %s\n", strerror (errno));

  switch (fork ()) {
  case -1:
    fatal ("fork: %s\n", strerror (errno));
  case 0:
    close (fds[1]);
    if (fds[0] != 0) {
      if (dup2 (fds[0], 0) < 0)
	fatal ("dup2: %s\n", strerror (errno));
      close (fds[0]);
    }
    close (1);
    setgroups (0, NULL);
    /* XXX - Should give up root privileges here, but what ID to assume? */
    logger ();
    break;
  default:
    close (fds[0]);
    if (fds[1] != 2) {
      if (dup2 (fds[1], 2) < 0)
	fatal ("dup2: %s\n", strerror (errno));
      close (fds[1]);
    }
    close (0);
    close (1);
    fcntl (2, F_SETFD, 0);
    break;
  }
}

void
daemonize (void)
{
  forkexit ();
  setsid ();
  forkexit ();
  chdir ("/");
  umask (0);
  makelogger ();
}
