/* $Id: misc.c,v 1.1 1998/12/07 20:42:53 fubobb Exp $ */

#include "sysconf.h"

#define DONT_HIDE_MALLOC
#include "misc.h"
#include "suio.h"

char *progname;
void (*fatalhook) (void);

static const int errfd = 2;
static struct suio *erruio;

/* Some prime numbers <= powers of 2 */
const u_int exp2primes[33] = {
  0x1, /* place holder */
  0x2, 0x3, 0x7, 0xd,
  0x1f, 0x3d, 0x7f, 0xfb,
  0x1fd, 0x3fd, 0x7f7, 0xffd,
  0x1fff, 0x3ffd, 0x7fed, 0xfff1,
  0x1ffff, 0x3fffb, 0x7ffff, 0xffffd,
  0x1ffff7, 0x3ffffd, 0x7ffff1, 0xfffffd,
  0x1ffffd9, 0x3fffffb, 0x7ffffd9, 0xfffffc7,
  0x1ffffffd, 0x3fffffdd, 0x7fffffff, 0xfffffffb,
};

/* Highest bit set in a byte */
static const char bytemsb[0x100] = {
  0, 1, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 5, 5,
  5, 5, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
  6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7,
  7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
  7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
  7, 7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
};

/* Find last set (most significant bit) */
inline int
fls (u_int32_t v)
{
  if (v & 0xffff0000) {
    if (v & 0xff000000) 
      return 24 + bytemsb[v>>24];
    else
      return 16 + bytemsb[v>>16];
  }

  if (v & 0x0000ff00)
    return 8 + bytemsb[v>>8];
  else
    return bytemsb[v];
}

/* Ceiling of log base 2 */
int
log2c (u_int32_t v)
{
  return v ? fls (v - 1) : -1;
}

void
setprogname (char *argv0)
{
  char *cp;
  if ((cp = strrchr (argv0, '/')))
    cp++;
  else
    cp = argv0;
  xfree (progname);
  progname = xstrdup (cp);
}

void
err_init (void)
{
  if (erruio)
    suio_free (erruio);
  erruio = suio_alloc ();
}

void
err_flush (void)
{
  int n = 1;

  if (!erruio)
    return;
  make_sync (errfd);
  while (erruio->uio_resid && n > 0) {
    n = writev (errfd, erruio->uio_iov,
		(erruio->uio_iovcnt >= UIO_MAXIOV
		 ? UIO_MAXIOV : erruio->uio_iovcnt));
    if (n > 0)
      suio_rembytes (erruio, n);
  }
}

static void
err_wcb (void *_null)
{
  int n;
  int cnt;

  if (!erruio)
    return;
  if (erruio->uio_resid == 0) {
    cb_free (errfd, 1);
    return;
  }

  /* Try to write whole lines at a time. */
  for (cnt = min (erruio->uio_iovcnt, UIO_MAXIOV);
       cnt > 0 && (erruio->uio_iov[cnt-1].iov_len == 0
		   || *((char *) erruio->uio_iov[cnt-1].iov_base
			+ erruio->uio_iov[cnt-1].iov_len - 1) != '\n');
       cnt--)
    ;
  if (!cnt) {
    if (erruio->uio_iovcnt < UIO_MAXIOV) {
      /* Wait for a carriage return */
      cb_free (errfd, 1);
      return;
    }
    else
      cnt = UIO_MAXIOV;
  }

  /* Write asynchronously, but keep stderr synchronous in case of
   * emergency (e.g. maybe assert wants to fprintf to stderr). */
  make_async (errfd);
  n = writev (errfd, erruio->uio_iov, cnt);
  make_sync (errfd);

  if (n == 0 || (n < 0 && errno != EAGAIN)) {
    erruio = NULL;
    fatal ("Write error: %s\n", strerror (errno));
  }
  if (n > 0)
    suio_rembytes (erruio, n);

  if (erruio->uio_resid == 0)
    cb_free (errfd, 1);
  else
    cb_add (errfd, 1, err_wcb, erruio);
}


void
fatal (const char *msg, ...)
{
  va_list ap;
  struct suio *tmpuio = suio_alloc ();
  char *p;
  int n;

  if (progname)
    suio_uprintf (tmpuio, "%s: ", progname);
  suio_uprintf (tmpuio, "fatal: ");
  va_start (ap, msg);
  suio_vuprintf (tmpuio, msg, ap);
  va_end (ap);

  n = tmpuio->uio_resid;
  p = suio_flatten (tmpuio);
  suio_free (tmpuio);

#ifndef UPT
  if (erruio) {
    if (n <= 32) {
      suio_copy (erruio, p, n);
      xfree (p);
    }
    else {
      suio_print (erruio, p, n);
      suio_callfree (erruio, p);
    }
    if (erruio->uio_resid)
      err_wcb (NULL);
  }
  else {
    write (errfd, p, n);
    xfree (p);
  }
#else
  p = xrealloc (p, n+ 2);
  p[n] = '\0';
  thread_warn("%s", p);
  xfree(p);
#endif

  if (fatalhook)
    fatalhook ();

  if (erruio)
    err_flush ();

  exit (1);
}

static void
err_print (struct suio *uio)
{
  char *p;
  int n;

  n = uio->uio_resid;
  p = suio_flatten (uio);

  p = xrealloc (p, n+ 2);
  p[n] = '\0';

  thread_warn("%s", p);

  xfree(p);
}

void
warn (const char *msg, ...)
{
  va_list ap;
  struct suio *tmpuio = suio_alloc ();

  if (progname)
    suio_uprintf (tmpuio, "%s: ", progname);
  va_start (ap, msg);
  suio_vuprintf (tmpuio, msg, ap);
  va_end (ap);

  err_print (tmpuio);
  suio_free (tmpuio);
}

void
ferr (const char *msg, ...)
{
  va_list ap;
  struct suio *tmpuio = suio_alloc ();

  va_start (ap, msg);
  suio_vuprintf (tmpuio, msg, ap);
  va_end (ap);

  err_print (tmpuio);
  suio_free (tmpuio);
}

void
verr (const char *fmt, va_list ap)
{
  struct suio *tmpuio = suio_alloc ();

  suio_vuprintf (tmpuio, fmt, ap);
  err_print (tmpuio);
  suio_free (tmpuio);
}

#ifndef __DMALLOC_H__
#ifndef UPT
void *
xmalloc (size_t size)
{
  void *p;
  p = malloc (size);
  if (!p) {
    fprintf (stderr, "malloc of %d bytes failed\n", size);
    abort ();
  }
  return (p);
}

void *
xrealloc (void *o, int size)
{
  void *p;
  p = realloc (o, size);
  if (!p) {
    fprintf (stderr, "realloc of %d bytes failed\n", size);
    abort ();
  }
  return (p);
}
#endif

char *
xstrdup (const char *s)
{
  char *d;
  d = xmalloc (strlen (s) + 1);
  strcpy (d, s);
  return (d);
}
#endif /* !__DMALLOC_H__ */

/* This is the memory equivalent of strpbrk().  It finds the first
 * occurance of any character from s2 in s1, but takes a length
 * argument for s1 rather than stopping at a null byte. */
char *
mempbrk (char *s1, const char *s2, int len)
{
  const char *const eom = s1 + len;
  register const char *cp;
  register int i1, i2;

  while (s1 < eom) {
    i1 = *s1++;
    cp = s2;
    while ((i2 = *cp++))
      if (i1 == i2)
	return (s1 - 1);
  }
  return (NULL);
}

/* This is a strsep function that returns a null field for adjacent
 * separators.  This is the same as the 4.4BSD strsep, but different
 * from the one in the GNU libc. */
inline char *
xstrsep(char **str, const char *delim)
{
  char *s, *e;

  if (!**str)
    return (NULL);

  s = *str;
  e = s + strcspn (s, delim);

  if (*e != '\0')
    *e++ = '\0';
  *str = e;

  return (s);
}

/* Get the next non-null token (like GNU strsep).  Strsep() will
 * return a null token for two adjacent separators, so we may have to
 * loop. */
char *
strnnsep (char **stringp, char *delim)
{
  char *tok;

  do {
    tok = xstrsep (stringp, delim);
  } while (tok && *tok == '\0');
  return (tok);
}

#if !defined (HAVE_STRCASECMP) && !defined (DMALLOC)

/*
 * This array is designed for mapping upper and lower case letter
 * together for a case independent comparison.  The mappings are
 * based upon ascii character sequences.
 */
static const u_char charmap[] =
{
  '\000', '\001', '\002', '\003', '\004', '\005', '\006', '\007',
  '\010', '\011', '\012', '\013', '\014', '\015', '\016', '\017',
  '\020', '\021', '\022', '\023', '\024', '\025', '\026', '\027',
  '\030', '\031', '\032', '\033', '\034', '\035', '\036', '\037',
  '\040', '\041', '\042', '\043', '\044', '\045', '\046', '\047',
  '\050', '\051', '\052', '\053', '\054', '\055', '\056', '\057',
  '\060', '\061', '\062', '\063', '\064', '\065', '\066', '\067',
  '\070', '\071', '\072', '\073', '\074', '\075', '\076', '\077',
  '\100', '\141', '\142', '\143', '\144', '\145', '\146', '\147',
  '\150', '\151', '\152', '\153', '\154', '\155', '\156', '\157',
  '\160', '\161', '\162', '\163', '\164', '\165', '\166', '\167',
  '\170', '\171', '\172', '\133', '\134', '\135', '\136', '\137',
  '\140', '\141', '\142', '\143', '\144', '\145', '\146', '\147',
  '\150', '\151', '\152', '\153', '\154', '\155', '\156', '\157',
  '\160', '\161', '\162', '\163', '\164', '\165', '\166', '\167',
  '\170', '\171', '\172', '\173', '\174', '\175', '\176', '\177',
  '\200', '\201', '\202', '\203', '\204', '\205', '\206', '\207',
  '\210', '\211', '\212', '\213', '\214', '\215', '\216', '\217',
  '\220', '\221', '\222', '\223', '\224', '\225', '\226', '\227',
  '\230', '\231', '\232', '\233', '\234', '\235', '\236', '\237',
  '\240', '\241', '\242', '\243', '\244', '\245', '\246', '\247',
  '\250', '\251', '\252', '\253', '\254', '\255', '\256', '\257',
  '\260', '\261', '\262', '\263', '\264', '\265', '\266', '\267',
  '\270', '\271', '\272', '\273', '\274', '\275', '\276', '\277',
  '\300', '\301', '\302', '\303', '\304', '\305', '\306', '\307',
  '\310', '\311', '\312', '\313', '\314', '\315', '\316', '\317',
  '\320', '\321', '\322', '\323', '\324', '\325', '\326', '\327',
  '\330', '\331', '\332', '\333', '\334', '\335', '\336', '\337',
  '\340', '\341', '\342', '\343', '\344', '\345', '\346', '\347',
  '\350', '\351', '\352', '\353', '\354', '\355', '\356', '\357',
  '\360', '\361', '\362', '\363', '\364', '\365', '\366', '\367',
  '\370', '\371', '\372', '\373', '\374', '\375', '\376', '\377',
};

int
strcasecmp (const char *s1, const char *s2)
{
  const u_char *cm = charmap,
    *us1 = (const u_char *) s1,
    *us2 = (const u_char *) s2;

  while (cm[*us1] == cm[*us2++])
    if (*us1++ == '\0')
      return (0);
  return (cm[*us1] - cm[*--us2]);
}

int
strncasecmp (const char *s1, const char *s2, int n)
{
  if (n != 0) {
    const u_char *cm = charmap,
      *us1 = (const u_char *) s1,
      *us2 = (const u_char *) s2;
    do {
      if (cm[*us1] != cm[*us2++])
	return (cm[*us1] - cm[*--us2]);
      if (*us1++ == '\0')
	break;
    } while (--n != 0);
  }
  return (0);
}

#endif /* !HAVE_STRCASECMP && !DMALLOC */

u_int
iovsize (struct iovec *iov, int cnt)
{
  u_int size = 0;
  while (cnt-- > 0)
    size += iov[cnt].iov_len;
  return size;
}
