/* $Id: misc.h,v 1.1 1998/12/07 20:42:54 fubobb Exp $ */

#ifndef _MISC_H_
#define _MISC_H_

#include "arpc/sysconf.h"
#include <setjmp.h>
#include <stdarg.h>

#ifdef UPT
#include "upt/upt.h"
#endif

struct suio;

#ifdef __linux__
#define DONT_HIDE_MALLOC
#endif

#if 0
#ifdef DMALLOC
#define DMALLOC_FUNC_CHECK
#include <dmalloc.h>
#define memmove(to, from, len) _dmalloc_bcopy((void *) from, (void *) to, len)
#undef memcpy
#define memcpy(to, from, len) _dmalloc_memcpy((void *) to, (void *) from, len)
#else /* !DMALLOC */
#ifndef HAVE_STRCASECMP
int strcasecmp (const char *, const char *);
int strncasecmp (const char *, const char *, int);
#endif /* HAVE_STRCASECMP */
#endif /* !DMALLOC */
#endif

#ifndef __GNUC__
#define __attribute__(x)
#endif /* !__GNUC__ */

#define min(a, b) (((a) <= (b)) ? (a) : (b))
#define max(a, b) (((a) >= (b)) ? (a) : (b))

/* amain.c */
extern int amain_jmpset;
extern jmp_buf amain_jmp;
int readytowrite (int);
void cb_free (int, int);
void cb_add (int, int, void (*)(void *), void *);
void cb_enable (int, int, int);
void tmo_add (time_t, void (*)(void *), void *);
void chld_add (pid_t, void (*) (void *, int), void *);
void acheck (void);
void amain (void) __attribute__ ((noreturn));
void ainit (void);

/* misc.c */
extern char *progname;
extern void (*fatalhook) (void);
void err_init (void);
void err_flush (void);
void fatal (const char *msg, ...)
     __attribute__ ((noreturn, format (printf, 1, 2)));

void warn (const char *, ...) __attribute__ ((format (printf, 1, 2)));
void ferr (const char *, ...) __attribute__ ((format (printf, 1, 2)));
void verr (const char *, va_list);
void setprogname (char *);
char *xstrsep (char **, const char *);
char *strnnsep (char **, char *);
char *mempbrk (char *, const char *, int);
int log2c (u_int32_t v);
int fls (u_int32_t v);
u_int iovsize (struct iovec *, int);

/* gencb.c */
void gencb_set (void *, void (*) (void *), void *);
void gencb_make (void *);
void gencb_makeone (void *);
#ifdef NDEBUG
#define gencb_assertnone(chan) ((void)0)
#else /* !NDEBUG */
void gencb_assertnone (void *chan);
#endif /* !NDEBUG */


/* socket.c */
int inetsocket (int, u_int32_t, u_short);
int inetsocket_resvport (int, u_int32_t);
void close_on_exec (int);
void make_async (int);
void make_sync (int);

#ifdef HAVE_FDPASS
/* passfd.c */
int sendfd (int, int);
int recvfd (int);
#endif /* HAVE_FDPASS */

/* daemonize.c */
void daemonize (void);

/* suio_vuprintf.c */
extern char *dsprintf (const char *, ...)
     __attribute__ ((format (printf, 1, 2)));

void sha1_suio (u_char digest[20], struct suio *uio);

#ifndef __DMALLOC_H__
#ifndef UPT
void *xmalloc (size_t);
void *xrealloc (void *, int);
#else
#include "upt/malloc.h"
#endif
char *xstrdup (const char *s);


#define xfree free

#if 0
#ifndef DONT_HIDE_MALLOC
#define strdup `wrong
#define malloc `wrong
#define strtok `wrong
#define strsep `wrong
#endif /* !DONT_HIDE_MALLOC */
#endif /* 0 */

#endif /* !__DMALLOC_H__ */

#define p8l(l)								\
     warn ("FH8: %08lx-%08lx-%08lx-%08lx-%08lx-%08lx-%08lx-%08lx\n",	\
	   (unsigned long) htonl(((u_int32_t *)(l))[0]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[1]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[2]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[3]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[4]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[5]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[6]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[7]));
#define p6l(l)								\
     warn ("FH6: %08lx-%08lx-%08lx-%08lx-%08lx-%08lx\n",		\
	   (unsigned long) htonl(((u_int32_t *)(l))[0]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[1]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[2]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[3]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[4]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[5]));
#if 0
#define p4l(l)								\
     warn ("FH4: %08lx-%08lx-%08lx-%08lx\n",				\
	   (unsigned long) htonl(((u_int32_t *)(l))[0]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[1]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[2]),		\
	   (unsigned long) htonl(((u_int32_t *)(l))[3]));
#endif

#ifndef MIN
#define MIN(x,y) ((x) < (y) ? (x) : (y))
#endif

#ifndef MAX
#define MAX(x,y) ((x) > (y) ? (x) : (y))
#endif

#endif /* !_MISC_H_ */
