/* $Id: socket.c,v 1.1 1998/12/07 20:42:59 fubobb Exp $ */

#include "sysconf.h"
#include "misc.h"

static u_short inetsocket_lastport;

int
inetsocket_resvport (int type, u_int32_t addr)
{
#ifndef NORESVPORTS
  int s;
  struct sockaddr_in sin;

  bzero (&sin, sizeof (sin));
  sin.sin_family = AF_INET;
  sin.sin_port = htons (0);
  sin.sin_addr.s_addr = htonl (addr);
  if ((s = socket (AF_INET, type, 0)) < 0)
    return (-1);

  /* Don't bother if we aren't root */
  if (geteuid ()) {
    if (bind (s, (struct sockaddr *) &sin, sizeof (sin)) >= 0)
      return s;
    close (s);
    return -1;
  }
#ifdef HAVE_BINDRESVPORT
  if (bindresvport (s, &sin) >= 0) {
    inetsocket_lastport = ntohs (sin.sin_port);
    return (s);
  }
#else /* !HAVE_BINDRESVPORT */
  for (inetsocket_lastport = IPPORT_RESERVED - 1;
       inetsocket_lastport > IPPORT_RESERVED/2;
       inetsocket_lastport--) {
    sin.sin_port = htons (inetsocket_lastport);
    if (bind (s, (struct sockaddr *) &sin, sizeof (sin)) >= 0)
      return (s);
    if (errno != EADDRINUSE)
      break;
  }
#endif /* !HAVE_BINDRESVPORT */
  close (s);
  return (-1);
#else /* NORESVPORTS */
  return inetsocket (type, addr, 0);
#endif /* NORESVPORTS */
}

int
inetsocket (int type, u_int32_t addr, u_short port)
{
  int s;
  int n, sn;
  struct sockaddr_in sin;

  bzero (&sin, sizeof (sin));
  sin.sin_family = AF_INET;
  sin.sin_port = htons (port);
  sin.sin_addr.s_addr = htonl (addr);
  if ((s = socket (AF_INET, type, 0)) < 0)
    return (-1);

  sn = sizeof (n);
  /* Avoid those annoying TIME_WAITs for TCP */
  if (port && getsockopt (s, SOL_SOCKET, SO_TYPE, (char *)&n, &sn) >= 0
      && n == SOCK_STREAM) {
    n = 1;
    if (setsockopt (s, SOL_SOCKET, SO_REUSEADDR, (char *)&n, sizeof (n)) < 0)
      fatal ("inetsocket: SO_REUSEADDR: %s\n", strerror (errno));
  }
  if (bind (s, (struct sockaddr *) &sin, sizeof (sin)) >= 0)
    return s;
  close (s);
  return -1;
}

void
close_on_exec (int s)
{
  if (fcntl (s, F_SETFD, 1) < 0)
    fatal ("F_SETFD: %s\n", strerror (errno));
}

void
make_async (int s)
{
  int n;
  if ((n = fcntl (s, F_GETFL)) < 0
      || fcntl (s, F_SETFL, n | O_NONBLOCK) < 0)
    fatal ("O_NONBLOCK: %s\n", strerror (errno));
#if defined (SO_RCVBUF) && defined (SO_SNDBUF)
#ifdef SMALL_LIMITS
  {
    int sn = sizeof (n);
    if (getsockopt (s, SOL_SOCKET, SO_TYPE, (char *)&n, &sn) < 0
	|| n != SOCK_STREAM)
      return;
  }
  n = 32;
#else /* !SMALL_LIMITS */
/* Some Unix variants can't handle a 64k+ socket buffer.  
 * Typical values are closer to 512 bytes, after all.
 */ 
#  ifdef SFS_ALLOW_LARGE_BUFFER
  n = 0x11000;			/* 64K + header */
#  else /* SFS_ALLOW_LARGE_BUFFER */
  n = 0x05000;                  /* 16K + header */
#  endif /* SFS_ALLOW_LARGE_BUFFER */
#endif /* !SMALL_LIMITS */
  if (setsockopt (s, SOL_SOCKET, SO_RCVBUF, (char *)&n, sizeof (n)) < 0) {
#ifdef ENOTSOCK
    if (errno == ENOTSOCK)
#endif /* ENOTSOCK */
      return;
    fatal ("SO_RCVBUF: %s\n", strerror (errno));
  }
  if (setsockopt (s, SOL_SOCKET, SO_SNDBUF, (char *)&n, sizeof (n)) < 0)
    fatal ("SO_SNDBUF: %s\n", strerror (errno));
#endif /* SO_RCVBUF && SO_SNDBUF */
  /* Enable keepalives to avoid buffering write data for infinite time */
  n = 1;
  if (setsockopt (s, SOL_SOCKET, SO_KEEPALIVE, &n, sizeof (n)) < 0)
    fatal ("SO_KEEPALIVE: %s\n", strerror (errno));
}

void
make_sync (int s)
{
  int n;
  if ((n = fcntl (s, F_GETFL)) >= 0)
    fcntl (s, F_SETFL, n & ~O_NONBLOCK);
}
