/* $Id: tcpserv.c,v 1.1 1998/12/07 20:43:06 fubobb Exp $ */

#include "arpc.h"
#include "parseopt.h"

struct tcpst {
  int fd;
  void (*cb) (void *, struct axh *, struct sockaddr_in *, int fd);
  void *arg;
};

static void
tcp_accept (void *_st)
{
  struct tcpst *st = _st;
  int fd;

  for (;;) {
    struct axh *xh;
    struct sockaddr_in sin;
    int len = sizeof (sin);
    fd = accept (st->fd, (struct sockaddr *) &sin, &len);
    if (fd < 0)
      break;
    if (len != sizeof (sin))
      fatal ("tcp_accept: bad sin len\n");
    make_async (fd);
    close_on_exec (fd);
    xh = ps_alloc (fd);
    st->cb (st->arg, xh, &sin, fd);
  }
  if (errno != EAGAIN)
    warn ("tcp_accept: accept: %s\n", strerror (errno));
}

void
tcp_server (int fd,
	    void (*cb) (void *, struct axh *, struct sockaddr_in *, int fd),
	    void *arg)
{
  struct tcpst *st;
  if (listen (fd, 256) < 0)
    fatal ("tcp_server: listen: %s\n", strerror (errno));
  make_async (fd);
  close_on_exec (fd);
  st = xmalloc (sizeof (*st));
  st->fd = fd;
  st->cb = cb;
  st->arg = arg;
  cb_add (fd, 0, tcp_accept, st);
}

#ifdef HAVE_FDPASS

struct clonest {
  int fd;
  struct axh *xh;
  void (*cb) (void *, struct axh *, struct sockaddr_in *, int fd);
  void *arg;
};

static void
clone_accept (void *_st, char *pkt, int len, void *_sa, int _sal)
{
  struct clonest *st = _st;
  struct sockaddr_in sa;
  int sal = sizeof (sa);
  int fd;
  struct axh *xh;
  void *npkt;

  if (!pkt) {
    warn ("clone_accept: EOF\n");
    xh_free (st->xh);
    st->cb (st->arg, NULL, NULL, -1);
    xfree (st);
    return;
  }

  warn ("clone_accept: got one\n");
  fd = us_recvfd (st->xh);
  if (fd < 0 || getpeername (fd, (struct sockaddr *) &sa, &sal) < 0) {
    warn ("clone_accept: us_recvfd failed\n");
    if (fd >= 0)
      close (fd);
    xh_free (st->xh);
    st->cb (st->arg, NULL, NULL, -1);
    xfree (st);
    return;
  }

  make_async (fd);
  close_on_exec (fd);
  xh = ps_alloc (fd);
  npkt = xmalloc (len);
  memcpy (npkt, pkt, len);
  ps_unrecv (xh, npkt, len);
  st->cb (st->arg, xh, &sa, fd);
}

void
clone_server (int fd,
	      void (*cb) (void *, struct axh *, struct sockaddr_in *, int fd),
	      void *arg)
{
  struct clonest *st;
  st = xmalloc (sizeof (*st));
  st->fd = fd;
  st->cb = cb;
  st->arg = arg;
  make_async (fd);
  close_on_exec (fd);
  st->xh = us_alloc (fd);
  xh_setcb (st->xh, clone_accept, st);
}

#endif /* HAVE_FDPASS */

void
start_server (int port,
	     void (*cb) (void *, struct axh *, struct sockaddr_in *, int),
	     void *arg)
{
  int s;
#ifdef HAVE_FDPASS
  char *p;
  if ((p = getenv ("CLONEFD"))) {
    if (convertint (&s, p) < 0)
      warn ("CLONEFD=%s ignored--must be a number\n", p);
    else {
      warn ("Starting clone server (fd %d)\n", s);
      clone_server (s, cb, arg);
      return;
    }
  }
#endif /* HAVE_FDPASS */
  s = inetsocket (SOCK_STREAM, INADDR_ANY, port);
  if (s < 0)
    fatal ("can't create server socket: %s\n", strerror (errno));
  tcp_server (s, cb, arg);
}
