/**
** $Id: cache.c,v 1.10 1999/05/06 15:24:16 fubob Exp $
**
** $Log: cache.c,v $
** Revision 1.10  1999/05/06 15:24:16  fubob
** slowing improving cache
**
** Revision 1.9  1999/05/05 22:17:45  fubob
** stuff
**
** Revision 1.8  1999/05/05 03:49:13  fubob
**
** coded a little more of cache.c
**
**
*/

#include <string.h>

#include "cache.h"
#include "ASSERT.h"
#include "DEBUG.h"


/* Internal procedures */

int cache_flush(cache_t *cache)
{
  return 0;
}

/* Release expired attributes, flushing any dirty attrs to disk */
int attr_cache_flush(cache_t *cache)
{
  attr_cache_t *attr;

  DEBUGP(DEBUG_VERBOSE, 
	 ("attr_cache_flush(%p): Entering\n", cache));

  if (cache == NULL)
    return 0;

  attr = cache->head;

  /* Check if expired */
  if ( ((u32)time(NULL) - attr->timer) > CSFSCD_ATTR_REG_EXP) {
    return;
  }


  return 0;
}

/* Release the attr and decrement count in cache.  Do not flush */
int attr_cache_relse(cache_t *cache, attr_cache_t *attr)
{
  return 0;
}

/* Release expired buffers, flushing any dirty bufs to disk */
int buf_cache_flush(cache_t *cache)
{
  return 0;
}

/* Release buf without flushing, decrement count in cache */
int buf_cache_relse(cache_t* cache, buf_t *buf) {
  return 0;
}


/* dp_block_id has same format as in csfssd.h */
int hash_cache_read(attr_cache_t *attr, u32 dp_block_id, u8 *dp_block)
{
  return 0;
}

int hash_cache_write(attr_cache_t *attr, u32 dp_block_id, u8 *dp_block)
{
  return 0;
}


/* Public procedures */

cache_t *cache_init()
{
  cache_t *cache = malloc(sizeof(cache_t));

  cache->head = cache->tail = NULL;

  return cache;

}


int attr_cache_read(cache_t *cache, nfs_fh *fh, CLIENT *sd, 
		    csfssd_fattr **fattr)
{
  attr_cache_t *attr;
  int found = FALSE;

  DEBUGP(DEBUG_CONCISE, 
	 ("attr_cache_read(0x%p,0x%p,0x%p,0x%p): entering\n",
	  cache, fh, sd, fattr));


  if (attr_cache_flush(cache) < 0) {
    DEBUGP(DEBUG_CONCISE, 
	   ("attr_cache_read: attr flush failed\n"));
    return -1;
  }

  /* Search for the inode */
  attr = cache->head;
  while (attr != NULL) {

    if (((csfssd_fh_t *) fh)->inode ==
	attr->fattr->inode_idx) {
      found = TRUE;
      break;
    }

    attr = attr->next;
  }
  

  if (found) {

    DEBUGP(DEBUG_CONCISE, ("attr_cache_read: cached attr\n"));
    *fattr = attr->fattr;

  } else {

    /* Not cached, get from SD */
    csfssd_attrstat *attrstat;

    DEBUGP(DEBUG_CONCISE, ("attr_cache_read: no attr cached contacting SD\n"));

    attrstat = csfssdproc_getattr_1((csfssd_fh *)fh, sd);

    if (attrstat->status == NFS_OK) {

      /* Now add to cache, allocate space */
      attr = malloc(sizeof(attr_cache_t));
      memset(attr, 0, sizeof(attr_cache_t));

      attr->fattr = malloc(sizeof(csfssd_fattr));
      memcpy(attr->fattr, 
	     &(attrstat->csfssd_attrstat_u.attributes),
	     sizeof(csfssd_fattr) );

      attr->dirty = FALSE;
      attr->valid = FALSE;
      attr->buf_count = 0;
      attr->timer = (u32)time(NULL);

      if (crypto_hash_data((u8 *)attr->fattr, 
			   sizeof(csfssd_fattr), 
			   &(attr->metahash))
	  != CSFSSD_INTEGRITY_FIELD_LENGTH) {
	DEBUGP(DEBUG_CONCISE, 
	       ("attr_cache_read: metahash problem\n"));
	return -1;
      }

      attr->next = NULL;  /* Pointer to next attr of linked list */

      if (cache->tail == NULL) {
	cache->head = cache->tail = attr;
      } else {
	cache->tail->next = attr;
	cache->tail = attr;
      }
      
      *fattr = attr->fattr;
       
    } else {
      /* Error */
      DEBUGP(DEBUG_CONCISE, ("attr_cache_read: error %d\n",
			     attrstat->status));
      return -1;
    }
  }

  /* Check integrity of inode */
  if (attr->valid == FALSE) {
    u8 crypto_kmac_data(u8 *data, u32 len, u8 *key, u16 keylen, u8 **kmac);

  }
  

  DEBUGP(DEBUG_CONCISE, ("attr_cache_read: exiting\n"));
  
  return 0;
}

int attr_cache_write(cache_t *cache, nfs_fh *fh, CLIENT *sd,
		     csfssd_fattr *fattr)
{
  attr_cache_t *attr;
  int found = FALSE;
 
  DEBUGP(DEBUG_CONCISE, 
	 ("attr_cache_write(0x%p,0x%p,0x%p,0x%p): entering\n",
	  cache, fh, sd, fattr));


  /* Search for the inode */
  while ((attr = cache->head) != NULL) {
    if (((csfssd_fh_t *) fh)->inode ==
	attr->fattr->inode_idx) {
      found = TRUE;
      break;
    }
  }
  

  if (found) {

    DEBUGP(DEBUG_CONCISE, ("attr_cache_write: replacing cached attr\n"));
    memcpy(attr->fattr, fattr, sizeof(csfssd_fattr));

  } else {

    DEBUGP(DEBUG_CONCISE, ("attr_cache_write: adding new attr\n"));

    /* Now add to cache */
    attr = malloc(sizeof(attr_cache_t));
    memset(attr, 0, sizeof(attr_cache_t));
    
    attr->fattr = malloc(sizeof(csfssd_fattr));
    memcpy(attr->fattr, fattr, sizeof(csfssd_fattr));

    attr->dirty = TRUE;    /* Let flush write it to the SD */
    attr->valid = FALSE;
    attr->buf_count = 0;
    attr->next = NULL;  /* Pointer to next attr of linked list */

    if (cache->tail == NULL) {
      cache->head = cache->tail = attr;
    } else {
      cache->tail->next = attr;
      cache->tail = attr;
    }
    
  }


  DEBUGP(DEBUG_CONCISE, ("attr_cache_write: exiting\n"));
  return 0;
}




int buf_cache_read(cache_t *cache, nfs_fh *fh, u32 block_id, 
		   CLIENT *sd, u8 **buf)
{

#ifdef 0
  buf_t *cbuf;
  u32 *cbuf_id, i;

  attr_cache_t *attr;
  csfssd_fattr fattr;

  int found = FALSE;

  DEBUGP(DEBUG_CONCISE, 
	 ("buf_cache_read(0x%p,0x%p,%d,0x%p,0x%p): entering\n",
	  cache, fh, block_id, sd, buf));


  /* Make sure the attr is cached */
  if (attr_cache_read(cache, fh, sd, &fattr) < 0) {
    DEBUGP(DEBUG_CONCISE, ("buf_cache_read: file not exist\n"));
    return -1;
  }

  /* Search for the buffer's inode */
  while ((attr = cache->head) != NULL) {
    if (((csfssd_fh_t *) fh)->inode ==
	attr->fattr->inode_idx) {
      found = TRUE;
      break;
    }
  }
  

  if (!found) {
    DEBUGP(DEBUG_CONCISE, ("buf_cache_read: file not exist\n"));
    return -1;
  }

  
  /* We now have the attributes for our buffer.  Search
     the attr->buf_id array for our block.  If not there, fetch
     from SD */
  found = FALSE;

  for (i=0; i < attr->buf_count; i++) {
    if (attr->buf_id[i] == block_id) {
      found = TRUE;
      break;
    }
  }


  /* Get the buffer in memory */
  if (found) {
    DEBUGP(DEBUG_CONCISE, ("buf_cache_read: buffer in cache\n"));
    *buf = attr->buf[i];

  } else {
    struct csfssd_readargs args;
    csfssd_readres *res;

    memcpy (args.file.data, fh->data, CSFSSD_FHSIZE);
    args.offset = block_id;
    args.size = 1;

    DEBUGP(DEBUG_CONCISE, ("buf_cache_read: fetching buffer from SD\n"));

    res = csfssdproc_read_1(&args, sd);

    if (res->status == NFS_OK) {
      ASSERT (res->csfssd_readres_u.reply.data.data_len == CSFSSD_BLOCK_SIZE);
      *buf = malloc(CSFSSD_BLOCK_SIZE);
      memcpy (*buf, res->csfssd_readres_u.reply.data.data_val, CSFSSD_BLOCK_SIZE);
      free (res);

    } else {
      /* Error */
      DEBUGP(DEBUG_CONCISE, ("buf_cache_read: error reading from SD\n"));
      return -1;
    }

    cbuf = malloc(sizeof(buf_t));
    memset(cbuf, 0, sizeof(buf_t));

    
    attr->buf_id[attr->buf_count] = block_id;
    (attr->buf[attr->buf_count])->data = *buf;
    attr->buf_count++;

  }

  /* Check if decrypted */
  if (!(attr->buf[i]->flags & CSFSCD_BUF_DEC)) {
    
    if (attr->key_schedule == NULL) {
      
      DEBUGP(DEBUG_CONCISE, ("buf_cache_read: computing schedule\n"));
      attr->key_schedule = malloc (sizeof(key_schedule_t));
      crypto_key2sched(attr->fattr->owner_lockbox, 
		       CSFSSD_KEY_LENGTH,
		       attr->key_schedule);
      
    }
    
    if (crypto_decrypt_datablock(attr->buf[i]->data, 
				 CSFSSD_BLOCK_SIZE, 
				 attr->buf[i]->IV,
				 attr->key_schedule) < 0) {
      DEBUGP(DEBUG_CONCISE, ("buf_cache_read: decrypt failed\n"));
      return -1;
    }
    
    /* Decrypted buffer */
    attr->buf[i]->flags ^= CSFSCD_BUF_DEC;
    
  }
  
  DEBUGP(DEBUG_CONCISE, ("buf_cache_read: exiting\n"));
#endif  
  return 0;

}

int buf_cache_write(cache_t *cache, nfs_fh *fh, u32 block_id, 
		    CLIENT *sd, u8 *buf)
{
  return 0;
}


