/**
** $Id: cache.h,v 1.11 1999/05/06 15:24:17 fubob Exp $
**
** $Log: cache.h,v $
** Revision 1.11  1999/05/06 15:24:17  fubob
** slowing improving cache
**
** Revision 1.10  1999/05/05 22:17:45  fubob
** stuff
**
** Revision 1.9  1999/05/05 03:49:13  fubob
**
** coded a little more of cache.c
**
**
*/

#ifndef __CACHE_H__
#define __CACHE_H__ 1

#include <rpc/rpc.h>

#include "csfssd_prot.h"
#include "nfs_prot.h"
#include "sd/csfssd.h"
#include "misc/types.h"
#include "crypto.h"

#define CSFSCD_BUF_INVAL  1    /* Dirty? */
#define CSFSCD_BUF_VALID  2    /* Did we personally verify 
				  integrity of this block? */
#define CSFSCD_BUF_ENC    4    /* Is the block encrypted? */

#define CSFSCD_INT 1 /* perform integrity check */

/* Max in-memory cache */
#define CSFSCD_MAX_BUFFERS   2048 /* Max 8MB */
#define CSFSCD_MAX_ATTRS     512  /* Max 512KB */


/* Expiration times in seconds */
#define CSFSCD_ATTR_REG_EXP 60
#define CSFSCD_ATTR_DIR_EXP 30

/* encrypted or decrypted blocks */
typedef struct buf_s {
  u8      flags;      /* Dirty, Validated, Encrypted? */
  u8      *IV;
  u8      *data;
} buf_t;

/* fake 16-byte grp keys for demo */
#define GRPKEY 0x12345678901234567890123456789012

/* For dindirect block pointers */
typedef struct dindirect_hashtree_s {
  u8 valid;             /* Is this hash validated? */
  u8 *hash;             /* Hash of the children */
  csfssd_dp_block_t *dp_block;
  struct sindirect_hashtree_t *children;
  u8 num_children;                
} dindirect_hashtree_t;


/* For sindirect block pointers, kinda */
typedef struct sindirect_hashtree_s {
  u8 valid;                       /* Is this hash validated? */
  u8 *hash;                       /* Hash of the children */
  csfssd_dp_block_t *dp_block;    /* An 8KB block of DPs */
} sindirect_hashtree_t;


typedef struct attr_cache_s {
  csfssd_fattr *fattr;        /* This contains the inode_idx */
  u8 dirty;                   /* If inode attrs need to be written to SD */
  u8 *metahash; /* Hash of the inode */
  sindirect_hashtree_t sindirect[CSFSSD_NUM_SINDIRECT];
  dindirect_hashtree_t dindirect[CSFSSD_NUM_DINDIRECT];
                              /* Hashes for the sindirect and dindirect */
  u8 valid;                   /* whether aicf in inode validated */
  u32 buf_count;              /* How many buffers are cached, size of array */
  buf_t **buf;                /* Array of buffers */
  u32 *buf_id;            /* Array of buffer IDs (which buffer of the file) */
  key_schedule_t *key_schedule;  /* Eventually move to UA */
                        /* Cache the key schedule.  NULL if none computed */
  struct attr_cache_s *next;  /* Pointer to next attr of linked list */
  u32 timer;                  /* seconds since jan 1, 1970 for flushing */
} attr_cache_t;


typedef struct cache_s {
  u32 attr_cnt;
  u32 buf_cnt;  /* Total numbers of buffers in memory */
  attr_cache_t *head;
  attr_cache_t *tail;
} cache_t;


/* 8KB dir block, 255byte entries-> 2^16 = 2^8 * 2^8 */

typedef struct dirblock_s {
  u32 inode[256];
  char filename[256][251];
} dirblock_t;

/* Intialize an in-memory buffer cache */
cache_t *cache_init();

/* int cache_flush(cache_t *cache);*/

/* Requires: all inputs allocated.
 *
 * Effects: Sets fattr to memory containing the fh's attributes.
 *        Checks cache, but will contact SD if necessary.
 *        Flushes expired attrs from cache first.
 *
 * Returns: 0 if ok, else -1.
 */
int attr_cache_read(cache_t *cache, nfs_fh *fh, CLIENT *sd, 
		    csfssd_fattr **fattr);

int attr_cache_write(cache_t *cache, nfs_fh *fh, CLIENT *sd, 
		     csfssd_fattr *fattr);

int buf_cache_read(cache_t *cache, nfs_fh *fh, u32 block_id, 
		   CLIENT *sd, u8 **buf);

int buf_cache_write(cache_t *cache, nfs_fh *fh, u32 block_id, 
		    CLIENT *sd, u8 *buf);



#endif
