
/**
** Server procs based on nfs_prot.x 
**
** $Id: nfs_server.c,v 1.9 1999/05/08 17:57:14 drew42 Exp $
**
** $Log: nfs_server.c,v $
** Revision 1.9  1999/05/08 17:57:14  drew42
** nfs_server compiles
**
** Revision 1.8  1999/05/05 03:49:13  fubob
**
** coded a little more of cache.c
**
**
*/

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>

#include "nfs_prot.h"
#include "csfssd_prot.h"
#include "sd/csfssd.h"
#include "DEBUG.h"
#include "assert.h"
#include "upt/queue.h"
#include "cache.h"


CLIENT *sd;
cache_t *cache;

/* kludge for RPC */
static void
nfs_program_2(struct svc_req *rqstp, register SVCXPRT *transp);


typedef struct authunix_parms *ap_p;


/* If not connected to server, connect
 * Returns: < 0 on error.
            A newly allocated cliconn
 */
int newconn (CLIENT **sd) {

  /* Should not be pointing to anything */
  if (*sd != NULL) {
    return (-1);
  }

  /* name of the sd server */
  *sd = clnt_create("localhost", CSFSSD_PROGRAM, CSFSSD_VERSION, "udp");
  if (*sd == NULL) {
    return (-2);
  }

  return 0;
}

void freeconn (CLIENT *sd) {
  clnt_destroy (sd);
  return;
}

static int isinvalidfh(nfs_fh *fh)
{
  /* lookup "." directory wrt fh on server.  If match, ok */
  
  return (0);
} 

void fake_attr_dir(fattr *attr) {
  attr->type = NFDIR;
  attr->mode = S_IROTH | S_IWOTH | S_IXOTH | S_IFDIR;
  attr->nlink =1;
  attr->uid =0;
  attr->gid =0;
  attr->size =0;
  attr->blocksize= 8192;
  attr->rdev =0;
  attr->blocks =0;
  attr->fsid =0;
  attr->fileid =0;
  attr->atime.seconds =0;
  attr->atime.useconds=0;
  attr->mtime.seconds=0;
  attr->mtime.useconds=0;
  attr->ctime.seconds=0;
  attr->ctime.useconds=0;

  return;
}

/* Begin exported functions */

attrstat * 
nfsproc_getattr_2_svc(nfs_fh *argp, struct svc_req *rqstp)
{

  static attrstat  res;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_getattr_2_svc(%p, %p): entering\n", argp, rqstp));

  if (isinvalidfh(argp->data)) {
    printf ("stale fh (%s)\n", argp->data);
    res.status = NFSERR_STALE;
    return ((attrstat*) &res);
  }

  /* Do access check here w/ svcreq */

  /* Get attr from cache */

  /* Verify integrity */

  fake_attr_dir(&res.attrstat_u.attributes);

  res.status = NFS_OK;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_getattr_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

attrstat * 
nfsproc_setattr_2_svc(sattrargs *argp, struct svc_req *rqstp)
{

  static attrstat  res;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_setattr_2_svc(%p, %p): entering\n", argp, rqstp));

  /* Sign changes, connect to server, set attr */

  /* Must authenticate */

  /* Error conditions */


  /* Fake permissions because not implemented */
  res.status = NFSERR_PERM; 

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_setattr_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

diropres * 
nfsproc_lookup_2_svc(diropargs *argp, struct svc_req *rqstp)
{
  static diropres  res;
  csfssd_lookupres *sdres;
  csfssd_lookupargs sdargs;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_lookup_2_svc(%p, %p): entering\n", argp, rqstp));
  /*
  if (sd == NULL) {
    if (newconn(&cliconn) < 0) {
      printf ("nfsproc_lookup_2_svc: newconn error\n");
    }
  }
  */
  if (argp->name[0] == '.' && argp->name[1] == '\0') {
    memcpy((nfs_fh *)&res.diropres_u.diropres.file,
	   &argp->dir,
	   sizeof(nfs_fh)); 

    fake_attr_dir(&res.diropres_u.diropres.attributes);
    res.status = NFS_OK;

    return ((diropres *) &res);
  }

  /* Otherwise contact server for FH */
  /*
  memcpy (&sdargs.inode, &argp->dir, sizeof(csfssd_fh)) ;
  strncpy (sdargs.name, argp->name, NFS_MAXNAMLEN);
  sdargs.chal.challenge_len = 0;
  sdargs.chal.challenge_val = NULL;

  sdres = csfssdproc_lookup_1(&sdargs, sd);

  if (sdres->status == CSFSSD_OK) {

  } else {
    res.status = sdres->status;
    memcpy(&res.diropres_u.diropres.file, &sdres->csfssd_diropres_u.diropres.file, sizeof(csfssd_fh));
    memcpy(&res.diropres_u.diropres.file, &sdres->csfssd_diropres_u.diropres.file, sizeof(csfssd_fh));
    */
    /* Verify signature */
  /*   
  }
*/    
  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_lookup_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

readlinkres * 
nfsproc_readlink_2_svc(nfs_fh *argp, struct svc_req *rqstp)
{
  
  static readlinkres  res;
  
  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_readlink_2_svc(%p, %p): entering\n", argp, rqstp));

  /* Fake permissions because not implemented */
  res.status = NFSERR_PERM; 
  
  /*
   * insert server code here
   */
  
  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_readlink_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

readres * 
nfsproc_read_2_svc(readargs *argp, struct svc_req *rqstp)
{

  static readres  result;
  u8 *data;
  u32 block_id, 
    start_block, end_block, 
    bytes_read,
    start_byte, end_byte;
  csfssd_fattr *inode_attr;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_read_2_svc(%p, %p): entering\n", argp, rqstp));

  
  /*
   * insert server code here
   */
  
  /* allocate result memory */
  result.readres_u.reply.data.data_len = argp->count;
  result.readres_u.reply.data.data_val = malloc(argp->count);

  /* convert to blocks */

  start_block = argp->offset/CSFSSD_BLOCK_SIZE;
  end_block = (argp->offset + argp->count - 1)/CSFSSD_BLOCK_SIZE;

  start_byte = argp->offset % CSFSSD_BLOCK_SIZE;
  end_byte  = (argp->offset + argp->count - 1) % CSFSSD_BLOCK_SIZE;

  /* call cache for blocks */
  /* takes care of calling sd_server & integrety checks */
  
  bytes_read = 0;
  for(block_id=start_block; block_id<end_block; block_id++)
    {
      if((result.status = 
	 buf_cache_read(cache, &(argp->file), block_id, sd, &data)) != NFS_OK)
	{
	  /* error stuff */
	  return(&result);
	}
      /* extract needed data */
      memcpy(result.readres_u.reply.data.data_val+bytes_read,
	     data+start_byte, CSFSSD_BLOCK_SIZE - start_byte);
      start_byte = 0;
      bytes_read += CSFSSD_BLOCK_SIZE - start_byte;
    }
  if((result.status= 
     buf_cache_read(cache, &(argp->file), end_block, sd, &data)) != NFS_OK)
    {
      /* error stuff */
      return(&result);
    }
  /* extract needed data */
  memcpy(result.readres_u.reply.data.data_val+bytes_read, data, end_byte+1);
  

  if((result.status = 
      attr_cache_read(cache, &(argp->file), sd, &inode_attr)) != NFS_OK)
    {
      return(&result);
    }

  result.readres_u.reply.attributes.type = inode_attr->type;
  result.readres_u.reply.attributes.mode = inode_attr->mode;
  result.readres_u.reply.attributes.nlink = inode_attr->link_count;
  result.readres_u.reply.attributes.uid = inode_attr->uid;
  result.readres_u.reply.attributes.gid = inode_attr->gid;
  result.readres_u.reply.attributes.size = inode_attr->size;
  result.readres_u.reply.attributes.blocksize = CSFSSD_BLOCK_SIZE;
  result.readres_u.reply.attributes.rdev = 0;
  result.readres_u.reply.attributes.blocks = 
    inode_attr->size/CSFSSD_BLOCK_SIZE + 
    ((inode_attr->size%CSFSSD_BLOCK_SIZE)!=0);
  result.readres_u.reply.attributes.fsid = 0;
  result.readres_u.reply.attributes.fileid = inode_attr->inode_idx;
  result.readres_u.reply.attributes.atime.seconds = inode_attr->atime;
  result.readres_u.reply.attributes.mtime.seconds = inode_attr->mtime;
  result.readres_u.reply.attributes.ctime.seconds = inode_attr->ctime;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_read_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&result);
}

attrstat *
nfsproc_write_2_svc(writeargs *argp, struct svc_req *rqstp)
{
  static attrstat result;
  u8 *data;
  u32 start_block, end_block,
    start_byte, end_byte,
    bytes_written,
    block_id;
  csfssd_fattr *inode_attr;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_write_2_svc(%p, %p): entering\n", argp, rqstp));

  
  /*
   * insert server code here
   */

  /* compute blocks to write */
  start_block = argp->offset / CSFSSD_BLOCK_SIZE;
  end_block = (argp->offset + argp->data.data_len) / CSFSSD_BLOCK_SIZE;
  
  start_byte = argp->offset % CSFSSD_BLOCK_SIZE;
  end_byte = (argp->offset + argp->data.data_len) % CSFSSD_BLOCK_SIZE;


  /* read first block, update and write */

  if((result.status = 
      buf_cache_read(cache, &(argp->file), start_block, sd, &data)) != NFS_OK)
    {
      return(&result);
    }

  memcpy(&data[start_byte],argp->data.data_val,CSFSSD_BLOCK_SIZE - start_byte);

  if((result.status = 
      buf_cache_write(cache, &(argp->file), start_block, sd, data)) != NFS_OK)
    {
      return(&result);
    }
  
  bytes_written = CSFSSD_BLOCK_SIZE - start_byte;

  /* write all; but last block */

  for(block_id = start_block+1; block_id < end_block; block_id++)
    {
      if((result.status = 
	  buf_cache_write(cache,
			  &(argp->file),
			  block_id, 
			  sd, 
			  argp->data.data_val + bytes_written)) != NFS_OK)
	{
	  return(&result);
	}
      bytes_written += CSFSSD_BLOCK_SIZE;
    }

  /* read last block, update and write */

  if((result.status = 
      buf_cache_read(cache, &(argp->file), end_block, sd, &data)) != NFS_OK)
    {
      return(&result);
    }

  memcpy(data, argp->data.data_val, end_byte);

  if((result.status = 
      buf_cache_write(cache, &(argp->file), end_block, sd, data)) != NFS_OK)
    {
      return(&result);
    }
  

  /* now get the file attr */

  if((result.status = 
      attr_cache_read(cache, &(argp->file), sd, &inode_attr)) != NFS_OK)
    {
      return(&result);
    }

  result.attrstat_u.attributes.type = inode_attr->type;
  result.attrstat_u.attributes.mode = inode_attr->mode;
  result.attrstat_u.attributes.nlink = inode_attr->link_count;
  result.attrstat_u.attributes.uid = inode_attr->uid;
  result.attrstat_u.attributes.gid = inode_attr->gid;
  result.attrstat_u.attributes.size = inode_attr->size;
  result.attrstat_u.attributes.blocksize = CSFSSD_BLOCK_SIZE;
  result.attrstat_u.attributes.rdev = 0;
  result.attrstat_u.attributes.blocks = 
    inode_attr->size/CSFSSD_BLOCK_SIZE + 
    ((inode_attr->size%CSFSSD_BLOCK_SIZE)!=0);
  result.attrstat_u.attributes.fsid = 0;
  result.attrstat_u.attributes.fileid = inode_attr->inode_idx;
  result.attrstat_u.attributes.atime.seconds = inode_attr->atime;
  result.attrstat_u.attributes.mtime.seconds = inode_attr->mtime;
  result.attrstat_u.attributes.ctime.seconds = inode_attr->ctime;


  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_write_2_svc(%p, %p): exiting\n", argp, rqstp));
  
  return(&result);
}

diropres * 
nfsproc_create_2_svc(createargs *argp, struct svc_req *rqstp)
{
  
  static diropres  result;
  dirblock_t *data;
  csfssd_createargs sdargs;
  csfssd_createres *sdres;
  csfssd_fh_t fh;
  csfssd_fattr inode_attr;
  int i;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_create_2_svc(%p, %p): entering\n", argp, rqstp));

  
  /*
   * insert server code here
   */

  /* read directory block from cache */

  if((result.status = 
      buf_cache_read(cache, &(argp->where.dir), 0, sd, (u8**)&data)) != NFS_OK)
    {
      return(&result);
    }
 


  /* search for filename */
  /* error if found */

  for(i=0; (i<256) && (argp->where.name != NULL); i++)
    {
      if(strncmp(data->filename[i], argp->where.name, 251) == 0)
	{
	  result.status = NFSERR_EXIST;
	  return(&result);
	}
    }

  /* create file on server; get new inode_idx */

  memcpy(&fh, &(argp->where.dir), sizeof(csfssd_fh_t));
  sdargs.dir_inode_idx = fh.inode;
  
  sdres = csfssdproc_create_1(&sdargs, sd);

  if((result.status = sdres->status) != NFS_OK)
    {
      return(&result);
    }

  
  /* add filename, inode pair to file */
  /* figure out first empty slot */

  strncpy(data->filename[i], argp->where.name, 251);
  data->inode[i] = sdres->csfssd_createres_u.inode_idx;

  /* write dir to cache */

  if((result.status = 
      buf_cache_write(cache, &(argp->where.dir), 0, sd, (u8*)data)) != NFS_OK)
    {
      return(&result);
    }
 
  /* create results */
  /* make file handle */
  fh.inode = sdres->csfssd_createres_u.inode_idx;
  fh.gen = 0;
  memcpy(&(result.diropres_u.diropres.file), &fh, sizeof(csfssd_fh_t)); 

  /* get attr */

  if((result.status = 
      attr_cache_read(cache, &fh, sd, &inode_attr)) != NFS_OK)
    {
      return(&result);
    }

  result.diropres_u.diropres.attributes.type = inode_attr.type;
  result.diropres_u.diropres.attributes.mode = inode_attr.mode;
  result.diropres_u.diropres.attributes.nlink = inode_attr.link_count;
  result.diropres_u.diropres.attributes.uid = inode_attr.uid;
  result.diropres_u.diropres.attributes.gid = inode_attr.gid;
  result.diropres_u.diropres.attributes.size = inode_attr.size;
  result.diropres_u.diropres.attributes.blocksize = CSFSSD_BLOCK_SIZE;
  result.diropres_u.diropres.attributes.rdev = 0;
  result.diropres_u.diropres.attributes.blocks = 
    inode_attr.size/CSFSSD_BLOCK_SIZE + 
    ((inode_attr.size%CSFSSD_BLOCK_SIZE)!=0);
  result.diropres_u.diropres.attributes.fsid = 0;
  result.diropres_u.diropres.attributes.fileid = inode_attr.inode_idx;
  result.diropres_u.diropres.attributes.atime.seconds = inode_attr.atime;
  result.diropres_u.diropres.attributes.mtime.seconds = inode_attr.mtime;
  result.diropres_u.diropres.attributes.ctime.seconds = inode_attr.ctime;


  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_create_2_svc(%p, %p): exiting\n", argp, rqstp));
  
  return(&result);
}

nfsstat * 
nfsproc_remove_2_svc(diropargs *argp, struct svc_req *rqstp)
{

  static nfsstat  res;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_remove_2_svc(%p, %p): entering\n", argp, rqstp));

  /* Fake permissions because not implemented */
  res = NFSERR_PERM; 
  
  /*
   * insert server code here
   */

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_remove_2_svc(%p, %p): exiting\n", argp, rqstp));
  
  return(&res);
}

nfsstat * 
nfsproc_rename_2_svc(renameargs *argp, struct svc_req *rqstp)
{

  static nfsstat  res;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_rename_2_svc(%p, %p): entering\n", argp, rqstp));

  /* Fake permissions because not implemented */
  res = NFSERR_PERM; 
  
  /*
   * insert server code here
   */
  
  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_rename_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

nfsstat * 
nfsproc_link_2_svc(linkargs *argp, struct svc_req *rqstp)
{

  static nfsstat  res;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_link_2_svc(%p, %p): entering\n", argp, rqstp));

  /* Fake permissions because not implemented */
  res = NFSERR_PERM; 
  
  /*
   * insert server code here
   */
  
  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_link_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

nfsstat * 
nfsproc_symlink_2_svc(symlinkargs *argp, struct svc_req *rqstp)
{

  static nfsstat  res;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_symlink_2_svc(%p, %p): entering\n", argp, rqstp));

  /* Fake permissions because not implemented */
  res = NFSERR_PERM; 
  
  /*
   * insert server code here
   */

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_symlink_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

diropres * 
nfsproc_mkdir_2_svc(createargs *argp, struct svc_req *rqstp)
{

  static diropres  res;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_mkdir_2_svc(%p, %p): entering\n", argp, rqstp));

  /* Fake permissions because not implemented */
  res.status = NFSERR_PERM; 
  
  /*
   * insert server code here
   */
  
  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_mkdir_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

nfsstat * 
nfsproc_rmdir_2_svc(diropargs *argp, struct svc_req *rqstp)
{

  static nfsstat  res;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_rmdir_2_svc(%p, %p): entering\n", argp, rqstp));

  /* Fake permissions because not implemented */
  res = NFSERR_PERM; 
  
  /*
   * insert server code here
   */
  
  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_rmdir_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}

readdirres * 
nfsproc_readdir_2_svc(readdirargs *argp, struct svc_req *rqstp)
{

  static readdirres  result;
  dirblock_t *data;
  struct entry *dir_entry;
  int i;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_readdir_2_svc(%p, %p): entering\n", argp, rqstp));


  /* read dir from cache */
  if((result.status = 
      buf_cache_read(cache, &(argp->dir), 0, sd, &data)) != NFS_OK)
    {
      return(&result);
    }
 



  /* loop through entries and fill results*/
  
  dir_entry = result.readdirres_u.reply.entries = malloc(sizeof(struct entry));
  bzero(dir_entry, sizeof(struct entry));
  strncpy(dir_entry->name, data->filename[0], 251);
  dir_entry->fileid = data->inode[0];

  for(i=1; i<256 ; i++)
    {
      if(data->filename[i] == NULL)
	break;
      dir_entry->nextentry = malloc(sizeof(struct entry));
      dir_entry = dir_entry->nextentry;
      bzero(dir_entry, sizeof(struct entry));
      strncpy(dir_entry->name, data->filename[i], 251);
      dir_entry->fileid = data->inode[i];
    }




  /*  nfscookie retcookie;
  entry *entries;
  */
  /* in the args:
	nfs_fh dir;
	nfscookie cookie;
	u_int count;
  */
  /*
  entries = malloc (3 * sizeof(entry));

  res.readdirres_u.reply.eof = 1;
  res.readdirres_u.reply.entries = entries;

  bzero(retcookie, sizeof(retcookie));

  retcookie[0] = 1;

  entries[0].fileid = 2;
  entries[0].name = strdup(".");
  memcpy(entries[0].cookie, retcookie, sizeof(nfscookie));
  entries[0].nextentry = &entries[1];

  retcookie[0] = 2;

  entries[1].fileid = 3;
  entries[1].name = strdup("..");
  memcpy(entries[1].cookie, retcookie, sizeof(nfscookie));
  entries[1].nextentry = NULL;

  res.status = NFS_OK; 
  */


  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_readdir_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&result);
}

statfsres * 
nfsproc_statfs_2_svc(nfs_fh *argp, struct svc_req *rqstp)
{

  static statfsres  res;
  csfssd_fh sdargs;
  csfssd_statfsres *sdres;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_statfs_2_svc(%p, %p): entering\n", argp, rqstp));


  memcpy(&(sdargs.data), &(argp->data), NFS_FHSIZE);

  sdres = csfssdproc_statfs_1(&sdargs, sd);
  
  res.statfsres_u.reply.tsize = 8192;
  res.statfsres_u.reply.bsize = sdres->csfssd_statfsres_u.reply.bsize;
  res.statfsres_u.reply.blocks = sdres->csfssd_statfsres_u.reply.blocks;
  res.statfsres_u.reply.bfree = sdres->csfssd_statfsres_u.reply.bfree;
  res.statfsres_u.reply.bavail = sdres->csfssd_statfsres_u.reply.bavail;

  DEBUGP(DEBUG_CONCISE,
	 ("nfsproc_statfs_2_svc(%p, %p): exiting\n", argp, rqstp));

  return(&res);
}


int
main(int argc, char **argv)
{
	register SVCXPRT *transp;
	debuglevel = DEBUG_VERBOSE;
	DEBUGP(DEBUG_CONCISE, ("client daemon starting.\n"));
	(void) pmap_unset(NFS_PROGRAM, NFS_VERSION);

	transp = svcudp_create(RPC_ANYSOCK);
	if (transp == NULL) {
		fprintf(stderr, "cannot create udp service.");
		exit(1);
	}
	if (!svc_register(transp, NFS_PROGRAM, NFS_VERSION, nfs_program_2, IPPROTO_UDP)) {
		fprintf(stderr, "unable to register (NFS_PROGRAM, NFS_VERSION, udp).");
		exit(1);
	}

	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
	if (transp == NULL) {
		fprintf(stderr, "cannot create tcp service.");
		exit(1);
	}
	if (!svc_register(transp, NFS_PROGRAM, NFS_VERSION, nfs_program_2, IPPROTO_TCP)) {
		fprintf(stderr, "unable to register (NFS_PROGRAM, NFS_VERSION, tcp).");
		exit(1);
	}

	newconn(&sd);

	svc_run();
	fprintf(stderr, "svc_run returned");
	exit(1);
	/* NOTREACHED */
}
