/* $Id: arc4.h,v 1.1 1999/02/16 18:37:45 fubobb Exp $ */

#ifndef _ARC4_H_
#define _ARC4_H_



/* Arcfour random stream generator.  This code is derived from section
 * 17.1 of Applied Cryptography, second edition, which describes a
 * stream cipher allegedly compatible with RSA Labs "RC4" cipher (the
 * actual description of which is a trade secret).  The same algorithm
 * is used as a stream cipher called "arcfour" in Tatu Ylonen's ssh
 * package.
 *
 * RC4 is a registered trademark of RSA Laboratories.
 *
 * Borrowed from dm's SFS 
 *
 */

#include "sysconf.h"

struct arc4_stream {
  u_int8_t i;
  u_int8_t j;
  u_int8_t s[256];
};

void arc4_init (struct arc4_stream *, u_int8_t *, int);

static inline u_int8_t
arc4_getbyte (struct arc4_stream *as)
{
  u_int8_t si, sj;

  as->i = (as->i + 1) & 0xff;
  si = as->s[as->i];
  as->j = (as->j + si) & 0xff;
  sj = as->s[as->j];
  as->s[as->i] = sj;
  as->s[as->j] = si;
  return (as->s[(si + sj) & 0xff]);
}

static inline u_int32_t
arc4_getword (struct arc4_stream *as)
{
  u_int32_t val;
  val = arc4_getbyte (as) << 24;
  val |= arc4_getbyte (as) << 16;
  val |= arc4_getbyte (as) << 8;
  val |= arc4_getbyte (as);
  return val;
}

static inline void
arc4_transform (struct arc4_stream *as, char *dest, int len)
{
  const char *const eom = dest + len;
  while (dest < eom)
    *dest++ ^= arc4_getbyte (as);
}

static inline void
arc4_fill (struct arc4_stream *as, char *dest, int len)
{
  const char *const eom = dest + len;
  while (dest < eom)
    *dest++ = arc4_getbyte (as);
}

#endif /* _ARC4_H_ */
