#include <stdlib.h>
#include <string.h>
#include "crypto.h"
#include "DEBUG.h"
#include "ASSERT.h"


u8 crypto_hash_data(u8 *data, u32 len, u8 **hash) {

#ifdef SHA1
  sha1_ctx c;
  
  sha1_init (&c);
  sha1_update (&c, data, len);

  *hash = malloc(SHA1_SIZE);

  sha1_final (*hash, &c);

  return(SHA1_SIZE);
#endif
#ifdef MD5
  MD5_CTX c;

  MD5Init (&c);  
  MD5Update (&c, data, len); 
  MD5Final (&c);

  *hash = malloc(16);

  memcpy(hash, (u8 *)c.digest, 16);

  return(16);
#endif

}

u8 crypto_kmac_data(u8 *data, u32 len, u8 *key, u16 keylen, 
		    u8 **kmac)
{
  /* HMAC_k(m) = F(k, pad1, F(k, pad2, m)) where F=hash alg (SHA1) */

#ifdef SHA1
  sha1_ctx c;
  u8 mac[SHA1_SIZE];
  u8 *pad1, *pad2;
  unsigned int pad1len, pad2len;

  /* DOUBLE CHECK THESE LENGTHS! */
  pad1len = 1024-(len+keylen);
  pad1 = malloc(pad1len);
  memset(pad1, 0x31, pad1len);

  pad2len = 1024-(keylen+SHA1_SIZE);
  pad2 = malloc(pad2len);
  memset(pad1, 0x13, pad2len);

  /* F(k, pad2, m) */
  sha1_init (&c);
  sha1_update (&c, key, keylen);
  sha1_update (&c, pad2, pad2len);
  sha1_update (&c, data, len);
  sha1_final ((u8 *)&mac, &c);

  /* F(k, pad1, F()) */
  sha1_init (&c);
  sha1_update (&c, key, keylen);
  sha1_update (&c, pad1, pad1len);
  sha1_update (&c, (u8 *)&mac, SHA1_SIZE);
  
  *kmac = malloc(SHA1_SIZE);
  sha1_final (*kmac, &c);

  return(SHA1_SIZE);
#else 
  return(-1);
#endif

}

inline void crypto_key2sched(u8 *key, u8 keylen, 
			     key_schedule_t *key_schedule) 
{

#ifdef CRYPTO_RC5
  ASSERT(keylen == rc5_b);
  RC5_SETUP(key, &(key_schedule->schedule));
#endif

}

int crypto_encrypt_datablock(char *data, int len, char *IV,
			     key_schedule_t *key_schedule) {
  
#ifdef CRYPTO_RC5
  int j;
  WORD pt[2], ct[2] = {0,0};

  DEBUGP(DEBUG_VERBOSE,
	 ("crypto_encrypt_datablock(0x%p,%d,0x%p,0x%p): Entering\n",
	  data, len, IV, key_schedule));

  RC5_ENCRYPT(&(key_schedule->schedule), (WORD *) IV, ct);  

  if (len != 8192) {
    memset(data + len, 0, 8192 - len);
  }

  for (j=0; j < 2048; j+=2) {

    pt[0] = ((unsigned long int*) data)[j];
    pt[1] = ((unsigned long int*) data)[j+1];

    /* The CBC */
    pt[0] ^= ct[0];
    pt[1] ^= ct[1];

    RC5_ENCRYPT(&(key_schedule->schedule), pt, ct);  

    ((unsigned long int*) data)[j] = ct[0];
    ((unsigned long int*) data)[j+1] = ct[1];

  }

#endif

  DEBUGP(DEBUG_VERBOSE, ("crypto_encrypt_datablock: Exiting\n"));

  return 0;
}

int crypto_decrypt_datablock(char *data, int len, char *IV,
			     key_schedule_t *key_schedule) {

#ifdef CRYPTO_RC5
  int j;
  WORD pt[2], ct[2], ctfeed[2] = {0,0};

  DEBUGP(DEBUG_VERBOSE,
	 ("crypto_decrypt_datablock(0x%p,%d,0x%p,0x%p): Entering\n",
	  data, len, IV, key_schedule));


  RC5_ENCRYPT(&(key_schedule->schedule), (WORD *) IV, ctfeed);  

  if (len != 8192) {
    DEBUGP(DEBUG_VERBOSE, ("crypto_encrypt_datablock: bad data size\n"));
    return -1;
  }


  /* Use the IV */
  RC5_ENCRYPT(&(key_schedule->schedule), (WORD *) IV, ctfeed);  

  for (j=0; j < 2048; j+=2) {

    ct[0] = ((unsigned long int*) data)[j];
    ct[1] = ((unsigned long int*) data)[j+1];

    RC5_DECRYPT(&(key_schedule->schedule), ct, pt);

    /* The CBC */
    pt[0] ^= ctfeed[0];
    pt[1] ^= ctfeed[1];

    ctfeed[0] = ct[0];
    ctfeed[1] = ct[1];

    ((unsigned long int*) data)[j] = pt[0];
    ((unsigned long int*) data)[j+1] = pt[1];

  }

#endif

  DEBUGP(DEBUG_VERBOSE, ("crypto_decrypt_datablock: Exiting\n"));
  return 0;
}

int crypto_generate_IV(char *data, char *state){
  return 0;
}
