/* Defines a high-level interface into the crypto procedures */

#ifndef _CRYPTO_H
#define _CRYPTO_H

#include "sha1.h"
#include "rc5.h"
#include "misc/types.h"

#define CRYPTO_RC5 1

/* Pick one of the following */
#define SHA1 1
/* #define MD5 1 */

#ifdef CRYPTO_RC5

#define CRYPTO_IV_LEN 8  /* 8-byte IV */

typedef struct key_schedule_s {
  WORD schedule;
} key_schedule_t;

#endif

/*
 * Given: Pointer to data and length of data in bytes
 *
 * Effects: Compute hash of data using the DEFINEd hash alg
 *
 * Returns: Returns length of hash.  Returns hash in *hash (must be
 * explicitly freed).
 */
u8 crypto_hash_data(u8 *data, u32 len, u8 **hash);


/*
 * Given: Pointer to data and length of data in bytes
 *
 * Effects: Compute HMAC (symmetric signature) of data using the
 * DEFINEd hash alg.  HMAC is the keyed MAC designed by Bellare,
 * Canetti, and Krawczyk.
 *
 * Returns: Returns length of keyed MAC.  Sets *kmac to the keyed MAC
 *           (must be explicitly freed).  */
u8 crypto_kmac_data(u8 *data, u32 len, u8 *key, u16 keylen, u8 **kmac);


/*
 * Given: key and length of key, allocated key_schedule
 *
 * Effects: Generate key schedule for enc/dec
 *
 */
inline void crypto_key2sched(u8 *key, u8 keylen,
			     key_schedule_t *key_schedule);


/*
 * Given: Plaintext in data with length len, but allocated to 8KBx
 *        for padding
 *
 * Effects: Encrypt data in-place with key and IV.  Uses defined cipher
 * in CBC mode.
 *
 * Returns: 0 if ok, < 0 on error.
 */

int crypto_encrypt_datablock(char *data, int len, char *IV,
			     key_schedule_t *key_schedule);


/*
 * Given: Ciphertext in data with length len, but allocated 8KB for
 *         padding
 *
 * Effects: Decrypt data in-place with key and IV.  Uses defined cipher
 * in CBC mode.
 * WARNING: Must discard the trailing decrypted data you do not need.
 *  We assume that you know what data to truncate based on the position
 * of the block and the file size.
 *
 * Returns: 0 if ok, < 0 on error.
 */

int crypto_decrypt_datablock(char *data, int len, char *IV,
			     key_schedule_t *key_schedule);

#endif

