#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "crypto.h"
#include "ASSERT.h"


void test_rc5() 
{ WORD i, j, pt1[2], pt2[2], ct[2] = {0,0};
  WORD S[rc5_t];                      /* expanded key table */
  unsigned char key[rc5_b];
  time_t t0, t1;
  if (sizeof(WORD)!=4) 
    printf("RC5 error: WORD has %d bytes.\n",sizeof(WORD));
  printf("RC5-32/12/16 examples:\n");
  for (i=1;i<6;i++)
    { /* Initialize pt1 and key pseudorandomly based on previous ct */
      pt1[0]=ct[0]; pt1[1]=ct[1]; 
      for (j=0;j<rc5_b;j++) key[j] = ct[0]%(255-j);
      /* Setup, encrypt, and decrypt */
      RC5_SETUP(key, S);  
      RC5_ENCRYPT(S,pt1,ct);  
      RC5_DECRYPT(S,ct,pt2);
      /* Print out results, checking for decryption failure */
      printf("\n%ld key = ",i); 
      for (j=0; j<rc5_b; j++) printf("%.2X ",key[j]);
      printf("\n   plaintext %.8lX %.8lX  --->  ciphertext %.8lX %.8lX  \n",
             pt1[0], pt1[1], ct[0], ct[1]);
      if (pt1[0]!=pt2[0] || pt1[1]!=pt2[1]) 
        printf("Decryption Error!");
    }
  time (&t0);
  for (i=1;i<100000;i++)
    RC5_ENCRYPT(S,ct,ct);
  time (&t1);
  printf ("\n   Time_t for 100000 blocks:  %ld \n", t1-t0);
}

void test_cbc_enc() {
  key_schedule_t key_schedule;  /* expanded key table */
  u8 buffer[8192];
  u8 IV[CRYPTO_IV_LEN+1];
  u32 i, j, filesize, foo;
  char filename[100];
  u8 key[17];
  FILE *fp;

  /*  printf("Enter 16-char key: ");
  scanf("%s", key);
  memset((char *)(key + strlen(key)), 0, 16 - strlen(key));
  */

  strcpy(key, "0123456789012345");
  crypto_key2sched(key, 16, &key_schedule);

  /*  printf("Enter 8-char IV: ");
  scanf("%s", IV);
  memset((char *)(IV + strlen(IV)), 0, 8 - strlen(IV));  


  printf("File to encrypt: ");
  scanf("%s", filename);
  */

  strcpy(IV, "01234567");
  strcpy(filename,"/tmp/encme");

  if ((fp = fopen(filename,"r+")) == NULL) {
    printf ("Skipping this test.\n");
    return;
  }
  
  fseek(fp, 0L, SEEK_END);
  filesize = ftell(fp);
  printf ("Opening %s, %d bytes\n", filename, filesize);

  /* Encrypt each 8KB block, overwrite to file */
  fseek(fp, 0L, SEEK_SET);

  for (j = 0; ; j++) {

    i = fread (buffer, 1, 8192, fp);
    if (i == 0) {
      printf("Processed %d blocks. flushing\n", j);
      fflush(fp);
      fclose(fp);
      break;
    }

    /*    printf("Encrypting %dth 8KB block.  Really size %i bytes\n",
	   j, i);
	   */
    crypto_encrypt_datablock(buffer, i, IV, &key_schedule);
    
    fseek(fp, j * 8192, SEEK_SET);
    if ((foo = fwrite (buffer, 1, 8192, fp)) != 8192) {
      printf("fwrite failed.  only wrote %d bytes\n",foo);
      exit(-1);
    }
    i = fread (buffer, 1, 8192, fp);
  }
}

void test_cbc_dec() {
  key_schedule_t key_schedule;  /* expanded key table */
  u8 buffer[8192];
  u8 IV[CRYPTO_IV_LEN+1];
  u32 i, j, filesize, foo;
  char filename[100];
  u8 key[17];
  FILE *fp;

  /*  printf("Decrypt test\n");

  printf("Enter 16-char key: ");
  scanf("%s", key);
  memset((char *)(key + strlen(key)), 0, 16 - strlen(key)); */

  strcpy(key, "0123456789012345");
  crypto_key2sched(key, 16, &key_schedule);

  /*  printf("Enter 8-char IV: ");
  scanf("%s", IV);
  memset((char *)(IV + strlen(IV)), 0, 8 - strlen(IV));  

  printf("File to decrypt: ");
  scanf("%s", filename);

  */

  strcpy(IV, "01234567");
  strcpy(filename,"/tmp/encme");

  if ((fp = fopen(filename,"r+")) == NULL) {
    printf ("Skipping this test.\n");
    return;
  }
  
  fseek(fp, 0L, SEEK_END);
  filesize = ftell(fp);
  printf ("Opening %s, %d bytes\n", filename, filesize);

  /* Decrypt each 8KB block, overwrite to file */
  fseek(fp, 0L, SEEK_SET);

  for (j = 0; ; j++) {

    i = fread (buffer, 1, 8192, fp);
    if (i == 0) {
      printf("Processed %d blocks. flushing\n", j);
      fflush(fp);
      fclose(fp);
      break;
    }

    /*    printf("Decrypting %dth 8KB block.  Really size %i bytes\n",
	   j, i);
	   */
    crypto_decrypt_datablock(buffer, i, IV, &key_schedule);
    
    fseek(fp, j * 8192, SEEK_SET);
    if ((foo = fwrite (buffer, 1, 8192, fp)) != 8192) {
      printf("fwrite failed.  only wrote %d bytes\n",foo);
      exit(-1);
    }
    i = fread (buffer, 1, 8192, fp);
  }
}

void test_hash()
{
  char filename[10];
  FILE *fp;
  struct stat st;
  u16 i, j, numhash;
  u8 buffer[8192];
  u8 *hash[10];
  u8 key[16], *kmac;

  /*
  printf("Enter filename of input to hash: ");
  scanf("%s", filename);
  */

  strcpy(filename, "/tmp/decme");

  stat (filename, &st);
  printf ("Opening %s, %d bytes\n", filename, (int)st.st_size);

  fp = fopen(filename,"r");
  if (fp == NULL) {
    printf ("Skipping this test.\n");
    return;
  }

  numhash = 0;
  while ((i = fread (buffer, 1, 8192, fp)) != 0) {
    if (numhash == 4)
      break;
    crypto_hash_data(buffer, i, &(hash[numhash]));
    numhash++;
  }

  /*
  printf("Hashes (%d):\n", numhash);

  for (j=0; j<numhash; j++) {
    for (i=0; i<20; i+=4) {
      printf("%.2X%.2X%.2X%.2X ",
	     hash[j][i], hash[j][i+1], hash[j][i+2], hash[j][i+3]);
    }
    printf("\n");
  }
  */

  memset(key, 0x8f, 16);
  crypto_kmac_data((u8 *)hash, 20*4, key, 16, &kmac);
  
  /*  printf("KMAC:\n");

  for (i=0; i<20; i+=4) {
    printf("%.2X%.2X%.2X%.2X ",
	   kmac[i], kmac[i+1], kmac[i+2], kmac[i+3]);
  }
  printf("\n");
  */
}

void main()
{
  /*  test_cbc_enc(); 
 test_cbc_dec(); */
 test_hash();
 /*  test_rc5();  */
}
