/* $Id: misc.h,v 1.1 1999/02/16 18:37:46 fubobb Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */


#ifndef _MISC_H_
#define _MISC_H_

#include "sysconf.h"
#include <setjmp.h>
#include <stdarg.h>

struct suio;

#ifdef DMALLOC
#define DMALLOC_FUNC_CHECK
#include <dmalloc.h>
#undef memcpy
#undef xfree
#define memcpy(to, from, len) \
  _dmalloc_memcpy((char *) to, (const char *) from, len)
#define memmove(to, from, len) \
  _dmalloc_bcopy((const char *) from, (char *) to, len)
/* Work around Dmalloc's misunderstanding of free's definition */
static inline void
_xfree_wrap (const char *file, int line, void *ptr)
{
  if (ptr)
    _xfree_leap(file, line, ptr);
}
static inline void
xfree (void *ptr)
{
  if (ptr)
    _xfree_leap("unknown file", 0, ptr);
}
#define xfree(ptr) _xfree_wrap(__FILE__, __LINE__, ptr)
#endif /* DMALLOC */

#ifndef HAVE_STRCASECMP
#ifdef /* !DMALLOC */
/* These funcitons are only implemented on systems that actually have
 * strcasecmp. */
#undef strcasecmp
#undef strncasecmp
#endif /* !DMALLOC */
int strcasecmp (const char *, const char *);
int strncasecmp (const char *, const char *, int);
#endif /* HAVE_STRCASECMP */

#ifndef __GNUC__
#define __attribute__(x)
#endif /* !__GNUC__ */

#define min(a, b) (((a) <= (b)) ? (a) : (b))
#define max(a, b) (((a) >= (b)) ? (a) : (b))

/* amain.c */
extern int maxfd;
extern int amain_jmpset;
extern jmp_buf amain_jmp;
int readytowrite (int);
int readytoread (int);
void ioready (int, int *, int *);
void cb_free (int, int);
void cb_add (int, int, void (*)(void *), void *);
struct tmo;
struct tmo *tmo_add (time_t, void (*)(void *), void *);
void tmo0_add (int, void (*) (void *), void *);
void tmo_del (struct tmo *);
time_t tmo_time (struct tmo *);
void chld_add (pid_t, void (*) (void *, int), void *);
void acheck (void);
void amain (void) __attribute__ ((noreturn));
void ainit (void);

/* xmalloc.c */
extern void (*xmalloc_handler) (int);
#ifndef __DMALLOC_H__
void *xmalloc (size_t);
void *xrealloc (void *, int);
char *xstrdup (const char *s);
#define xfree free
#endif
/* !__DMALLOC_H__ */

/* misc.c */
char *xstrsep (char **, const char *);
char *strnnsep (char **, char *);
char *mempbrk (char *, const char *, int);
int log2c (u_int32_t v);
int fls (u_int32_t v);
u_int iovsize (struct iovec *, int);

/* err.c */
extern const int errfd;
extern char *progname;
extern void (*fatalhook) (void);
void err_init (void);
void err_flush (void);
void warn (const char *, ...) __attribute__ ((format (printf, 1, 2)));
void warnx (const char *, ...) __attribute__ ((format (printf, 1, 2)));
void vwarn (const char *fmt, va_list ap);
void vwarnx (const char *fmt, va_list ap);
void fatal (const char *msg, ...)
     __attribute__ ((noreturn, format (printf, 1, 2)));
void panic (const char *msg, ...)
     __attribute__ ((noreturn, format (printf, 1, 2)));
void setprogname (char *);

/* gencb.c */
void gencb_set (void *, void (*) (void *), void *);
void gencb_make (void *);
void gencb_makeone (void *);
#ifdef NDEBUG
#define gencb_assertnone(chan) ((void)0)
#else /* !NDEBUG */
void gencb_assertnone (void *chan);
#endif /* !NDEBUG */


/* socket.c */
int inetsocket (int, u_int32_t, u_short);
int inetsocket_resvport (int, u_int32_t);
void close_on_exec (int);
void make_async (int);
void make_sync (int);

#ifdef HAVE_FDPASS
/* passfd.c */
int sendfd (int, int);
int recvfd (int);
#endif /* HAVE_FDPASS */

/* myname.c */
char *myname (void);

/* myaddrs.c */
struct ipaddrs {
  int num;
  struct in_addr addrs[1];
};
struct ipaddrs *myipaddrs (void);

/* daemonize.c */
void daemonize (void);

/* suio_vuprintf.c */
#if defined (DMALLOC) && __GNUC__ == 2
#define DSPRINTF_DEBUG 1
#endif /* DMALLOC && GCC 2 */
#ifndef DSPRINTF_DEBUG
extern char *dsprintf (const char *, ...)
     __attribute__ ((format (printf, 1, 2)));
extern char *dvsprintf (const char *, va_list);
#else /* DSPRINTF_DEBUG */
extern char *__dsprintf (const char *, int, const char *, ...)
     __attribute__ ((format (printf, 3, 4)));
extern char *__dvsprintf (const char *, int, const char *, va_list);
#define dsprintf(fmt, args...) \
     __dsprintf (__FILE__, __LINE__, fmt , ## args)
#define dvsprintf(fmt, args) \
     __dvsprintf (__FILE__, __LINE__, fmt , args)
#endif /* DSPRINTF_DEBUG */

void sha1_suio (u_char digest[20], struct suio *uio);

#define TAILQ_DECL(head, type) \
  TAILQ_HEAD (, type) head = {NULL, &head->tqh_first}

/* Maximum # bytes malloc reserves for each 2^n block.  Doubles should
 * also be alignable to addresses of this multiple (for arena.c) */
#define MALLOCRESV 16

#endif /* !_MISC_H_ */
