/* Borrowed from dm's SFS */
/* $Id: mpaux.c,v 1.1 1999/02/16 18:37:47 fubobb Exp $ */

/*
 * Some random auxiliary functions for manipulating MP_INTs.
 */

#include <rpc/rpc.h>
#include "mpaux.h"
#include "sha1.h"


/* Store mpz_t big endian */
int mpz_get_raw (u_char *data, int len, mpz_t z)
{
   mpz_t t;
   u_int32_t i;
   int bytes;
   switch (mpz_sgn (z)) {
   case -1:
      warn ("mpz_get_raw: negative number\n");
      return -1;
   case 0:
      bzero (data, len);
      return 0;
   }
   if( !len ) {
      warn ("mpz_get_raw: number too large\n");
   }

   mpz_init_set (t, z);
   bytes = 0;
   data += len - 1;             /* big endian */
   while (1) {
      i = mpz_get_ui (t);
      mpz_tdiv_q_2exp (t, t, sizeof (u_int32_t) * 8);
      *data-- = i & 0xFF;
      if (++bytes >= len) break;
      *data-- = i>>8 & 0xFF;
      if (++bytes >= len) break;
      *data-- = i>>16 & 0xFF;
      if (++bytes >= len) break;
      *data-- = i>>24 & 0xFF;
      if (++bytes >= len) break;
   }
   mpz_clear (t);
   return 0;
}

void
mpz_set_raw (mpz_t z, const u_char *data, int len)
{
   u_int32_t i;
   mpz_set_ui (z, 0);
   while (3 < len) {
      i = *data++ << 24;
      i |= *data++ << 16;
      i |= *data++ << 8;
      i |= *data++;
      mpz_add_ui (z, z, i);
      len -= 4;
      mpz_mul_2exp (z, z, min (len*8, 32));
    }
   i = 0;
   switch (len) {
   case 3:
      i |= *data++ << 16;
   case 2:
      i |= *data++ << 8;
   case 1:
      i |= *data++;
      break;
   case 0:
   }
   mpz_add_ui (z, z, i);
}


/*
 * realloc/free functions that scrub memory
 */

static void *
scrub_realloc (void *ptr, size_t old_size, size_t new_size)
{
  int s;
  void *p = xmalloc (new_size);
  s = old_size;
  if (old_size > new_size)
    s = new_size;
  memcpy (p, ptr, s);
  bzero (ptr, old_size);
  xfree (ptr);
  return p;
}
static void
scrub_free(void *ptr, size_t size)
{
  bzero (ptr, size);
  xfree (ptr);
}
void
mpz_setscrub (void)
{
#ifdef DMALLOC
  mp_set_memory_functions(malloc, scrub_realloc, scrub_free);
#else /* !DMALLOC */
  mp_set_memory_functions(xmalloc, scrub_realloc, scrub_free);
#endif /* !DMALLOC */
}

/* Generate a random number of the desired number of bits.  */
void
mpz_random_integer(mpz_t ret, RandomState *state, unsigned int bits)
{
  unsigned int bytes = (bits + 7) / 8;
  char *str = xmalloc(bytes * 2 + 1);
  unsigned int i;

  /* We first create a random hex number of the desired size, and then
     convert it to a mp-int. */
  for (i = 0; i < bytes; i++)
    sprintf(str + 2 * i, "%02x", random_get_byte(state));

  /* Convert it to the internal representation. */
  mpz_set_str(ret, str, 16);

  /* Clear extra data. */
  bzero (str, 2 * bytes);
  xfree(str);

  /* Reduce it to the desired number of bits. */
  mpz_mod_2exp(ret, ret, bits);
}



/* For debugging */
void
mpz_print (mpz_t z)
{
  fprintf (stderr, "0x");
  mpz_out_str (stderr, 16, z);
}


/* Can't tolerate aliasing */
void mpz_xor(mpz_t r, mpz_t a)
{
   mpz_t t;
   mpz_init(t);

   mpz_ior(t, r, a);
   mpz_and(r, r, a);
   mpz_com(r, r);
   mpz_and(r, r, t);

   mpz_clear(t);
}
