/* RC5REF.C -- Reference implementation of RC5-32/12/16 in C.        */
/* Copyright (C) 1995 RSA Data Security, Inc.                        */
#include <stdio.h>
#include <time.h>
#include "rc5.h"

/* WORD P = 0xb7e15163, Q = 0x9e3779b9; */  /* magic constants             */

/* Note these values may not strictly adhere to the RC5 spec.  Should
   clarify if we publish a precise AMAC specification */


void RC5_SETUP(unsigned char *K, WORD *S) /* secret input key K[0...b-1]    */
{  WORD i, j, k, u=rc5_w/8, A, B, L[rc5_c]; 
   int  i2;
   /* Initialize L, then S, then mix key into S */
   for (i2=rc5_b-1,L[rc5_c-1]=0; i2!=-1; i2--) L[i2/u] = (L[i2/u]<<8)+K[i2];
   for (S[0]=rc5_P,i=1; i<rc5_t; i++) S[i] = S[i-1]+rc5_Q;
   for (A=B=i=j=k=0; k<3*rc5_t; k++,i=(i+1)%rc5_t,j=(j+1)%rc5_c)  
     /* 3*rc5_t > 3*rc5_c */
     { A = S[i] = ROTL(S[i]+(A+B),3);  
       B = L[j] = ROTL(L[j]+(A+B),(A+B)); 
     } 
}

void RC5_ENCRYPT(WORD *S, WORD *pt, WORD *ct) /* 2 WORD input pt/output ct  */
{ WORD i, A=pt[0]+S[0], B=pt[1]+S[1];
  for (i=1; i<=rc5_r; i++) 
    { A = ROTL(A^B,B)+S[2*i]; 
      B = ROTL(B^A,A)+S[2*i+1]; 
    }
  ct[0] = A; ct[1] = B;  
} 

void RC5_DECRYPT(WORD *S, WORD *ct, WORD *pt) /* 2 WORD input ct/output pt  */
{ WORD i, B=ct[1], A=ct[0];
  for (i=rc5_r; i>0; i--) 
    { B = ROTR(B-S[2*i+1],A)^A; 
      A = ROTR(A-S[2*i],B)^B; 
    }
  pt[1] = B-S[1]; pt[0] = A-S[0];  
} 


