/* RC5REF.C -- Reference implementation of RC5-32/12/16 in C.        */
/* Copyright (C) 1995 RSA Data Security, Inc.                        */

/* Code borrowed from S. Rajagopalan <sraj@bellcore.com> */


/* Modified by Kevin Fu <fubob@mit.edu> to be RC5/32/12/16 */

#ifndef _RC5_H_
#define _RC5_H_


typedef unsigned long int WORD; /* Should be 8-bit = 1 bytes        */
#define rc5_w        32             /* word size in bits                 */
#define rc5_r        12             /* number of rounds                  */  
#define rc5_b        16             /* number of bytes in key            */
#define rc5_c         4             /* number  words in key = ceil(8*b/w)*/
#define rc5_t        26             /* size of table S = 2*(r+1) words   */

/* Rotation operators. x must be unsigned, to get logical right shift*/
#define ROTL(x,y) (((x)<<(y&(rc5_w-1))) | ((x)>>(rc5_w-(y&(rc5_w-1)))))
#define ROTR(x,y) (((x)>>(y&(rc5_w-1))) | ((x)<<(rc5_w-(y&(rc5_w-1)))))

/* magic constants */
#define rc5_P 0xb7e15163
#define rc5_Q 0x9e3779b9


/* Given: A b-byte key in K and allocated WORD for state S
 *
 * Effects: Generate a key schedule in S
 *
 * Returns: Nothing.  Sets S appropriately.
 */
void RC5_SETUP(unsigned char *K, WORD *S);

/* Given: State S, plaintext pt, and allocated memory for ct
 *
 * Effects: Encrypt the block
 *
 * Returns: The ciphertext in ct (2-member array)
 */
void RC5_ENCRYPT(WORD *S, WORD *pt, WORD *ct);


/* Given: State S, ciphertext ct, and allocated memory for pt
 *
 * Effects: Decrypt the block
 *
 * Returns: The plaintext in pt (2-member array)
 */
void RC5_DECRYPT(WORD *S, WORD *ct, WORD *pt);


#endif
