/* Borrowed from dm's SFS */
/* $Id: rw.h,v 1.1 1999/02/16 18:37:53 fubobb Exp $ */

#ifndef RW_H
#define RW_H

#include "mpaux.h"


typedef MP_INT rw_pub;                     /* Public key format */
typedef struct rw_priv rw_priv;            /* Private key format */

/*
 * Key management functions.
 *
 * Any memory returned by a function (e.g. rw_priv2ascii) will be
 * allocated by the Rabin-Williams code but must be freed by the
 * caller.
 *
 * These routines do their own memory management for rw_priv
 * structures, but you must already have called mpz_init on any rw_pub
 * arguments.
 */

int rw_set_verbose (int);                  /* 1 = be chatty about primes */
rw_priv *rw_keygen (RandomState *, int);   /* New private key */
rw_priv *rw_ascii2priv (const char *);     /* Create key from string */
rw_priv *rw_file2priv (const char *);      /* Load priv key from file */
int rw_file2pub (rw_pub *sk, const char *);/* Load pub key from file */
void rw_freepriv (rw_priv *);              /* Destroy private key */
char *rw_priv2ascii (rw_priv *sk);         /* Convert key to string */
int rw_priv2file (const char *, rw_priv *);/* Store private key in file */
int rw_pub2file (const char *, rw_pub *);  /* Store public key in file */
void rw_priv2pub (rw_pub *, rw_priv *);    /* Extract public key */

int rw_ascii2pub (rw_pub *pk, const char *str); /* Create key from string */
char *rw_pub2ascii (rw_pub *);             /* Convert pub key to string */
rw_priv * rw_ascii2priv (const char *ibuf);/* Create priv key from string */
char *rw_priv2ascii (rw_priv *sk);         /* Convert priv key to string */


/*
 * Rabin-Williams operations.
 */

int rw_encrypt (mpz_t, RandomState *, rw_pub *, mpz_t);
int rw_decrypt (mpz_t, rw_priv *, mpz_t);
/* Last int is flag.  Set to 1 to enable message recovery */
int rw_sign (mpz_t, const struct iovec *, int, rw_priv *, RandomState *, int);
int rw_verify (char*, int, mpz_t, rw_pub *, int);

#endif /* RW_H */
