/* $Id: csfsauth_prot.x,v 1.8 1999/04/19 16:10:54 drew42 Exp $ */

/* Define a new type of AUTH for <rpc/auth.h> */
#ifdef RPC_HDR

%#ifndef AUTH_CSFS
%#define AUTH_CSFS 1

#endif /* RPC_HDR */

const MAX_USERNAME_LEN = 8;
const MAX_GROUPNAME_LEN = 8;
const MAX_KEY_LEN = 25;
const COOKIE_LEN = 2;
const MAX_USERS_PER_GRP = 10;
const MAX_GRPS_PER_USER = 10;


/* Error status 
   Note, these errors only cover benign errors
   such as unknown usernames.  Authentication
   and authorization errors are listed in <rpc/auth.h>
*/
enum csfsauthstat {
  CSFSAUTH_OK = 0,			/* no error */
  CSFSAUTH_NOUSER = 1,	                /* username does not exist */
  CSFSAUTH_NOUID = 2,	                /* UID does not exist */
  CSFSAUTH_NOGRP = 3,	                /* group name does not exist */
  CSFSAUTH_NOGID = 4,	                /* GID does not exist */
  CSFSAUTH_NOTIMPL = 5,                 /* Function not implemented */
  CSFSAUTH_GRP_OWNER = 6		/* Attempt to delete user who 
						owns a group */
};


struct csfsauth_key_t {
  int type;
  opaque content<MAX_KEY_LEN>;
};

typedef int csfsauth_uid_t;
typedef int csfsauth_gid_t;
typedef int csfsauth_gkid_t;
typedef string csfsauth_user_t<MAX_USERNAME_LEN>;
typedef string csfsauth_grp_t<MAX_GROUPNAME_LEN>;

/* Bitstrings of permissions */
enum csfsauth_perms {
  CSFSAUTH_ADDUSER 	    = 1,
  CSFSAUTH_DELUSER          = 2,
  CSFSAUTH_LISTUSERS        = 4,
  CSFSAUTH_USER2UID         = 8,
  CSFSAUTH_UID2USER         = 16,
  CSFSAUTH_GETUSERPK        = 32,
  CSFSAUTH_SETUSERPK        = 64,

  CSFSAUTH_GETUSERPERMS	    = 128,
  CSFSAUTH_SETUSERPERMS     = 256,

  CSFSAUTH_ADDGRP           = 512,
  CSFSAUTH_DELGRP           = 1024,
  CSFSAUTH_GRP2GID          = 2048,
  CSFSAUTH_GID2GRP          = 4096,
  CSFSAUTH_GETGRPKEY        = 8192,

  CSFSAUTH_ADDGRPMEM 	    =16384,
  CSFSAUTH_DELGRPMEM        =32768,
  CSFSAUTH_LISTGRPS         =65536,
  CSFSAUTH_LISTGRPMEMS      =131072,
  CSFSAUTH_CHGRPKEY         =262144,
  CSFSAUTH_CHGRPOWNER       =524288,

  CSFSAUTH_GETMEMGRPS      = 1048576,

  CSFSAUTH_ADMIN_PERMS=0,
  CSFSAUTH_USER_PERMS=0  
};
typedef int csfsauth_perms_t;


struct csfsauth_adduserargs {
	csfsauth_user_t username;
	csfsauth_uid_t uid;
	csfsauth_key_t pk;
	csfsauth_perms_t perms;

};

union csfsauth_adduserres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_uid_t uid;
default:
	void;
};


struct csfsauth_deluserargs {
	csfsauth_uid_t uid;
};

union csfsauth_deluserres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	void;
default:
	void;
};


struct csfsauth_listusersargs {
        opaque cookie[COOKIE_LEN];
};

struct csfsauth_listusersokres {
	csfsauth_uid_t uid<>;
	csfsauth_user_t username<>;
        opaque cookie[COOKIE_LEN];
};

union csfsauth_listusersres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_listusersokres listusersres;
default:
	void;
};


struct csfsauth_user2uidargs {
	csfsauth_user_t username;
};

union csfsauth_user2uidres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_uid_t uid;
default:
	void;
};


struct csfsauth_uid2userargs {
	csfsauth_uid_t uid;
};

union csfsauth_uid2userres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_user_t username;
default:
	void;
};


struct csfsauth_getuserpkargs {
	csfsauth_uid_t uid;
};

union csfsauth_getuserpkres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_key_t pk;
default:
	void;
};


struct csfsauth_setuserpkargs {
	csfsauth_uid_t uid;
	csfsauth_key_t pk;
};

union csfsauth_setuserpkres switch (csfsauthstat status) {
case CSFSAUTH_OK:
        void;
default:
	void;
};


struct csfsauth_getuserpermsargs {
	csfsauth_uid_t uid;
	csfsauth_perms_t perms;
};

union csfsauth_getuserpermsres switch (csfsauthstat status) {
case CSFSAUTH_OK:
        csfsauth_perms_t perms;
default:
	void;
};


struct csfsauth_setuserpermsargs {
	csfsauth_uid_t uid;
	csfsauth_perms_t perms;
};

union csfsauth_setuserpermsres switch (csfsauthstat status) {
case CSFSAUTH_OK:
        void;
default:
	void;
};


struct csfsauth_addgrpargs {
	csfsauth_grp_t grpname;
	csfsauth_gid_t gid;
	csfsauth_key_t pk;
	csfsauth_uid_t owner;
};

union csfsauth_addgrpres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_gid_t gid;
default:
	void;
};


struct csfsauth_delgrpargs {
	csfsauth_gid_t gid;
};

union csfsauth_delgrpres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	void;
default:
	void;
};


struct csfsauth_listgrpsargs {
        opaque cookie[COOKIE_LEN];
};

struct csfsauth_listgrpsokres {
	csfsauth_gid_t gid<>;
	csfsauth_grp_t grpname<>;
        opaque cookie[COOKIE_LEN];
};

union csfsauth_listgrpsres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_listgrpsokres listgrpsres;
default:
	void;
};


struct csfsauth_listgrpmemsargs {
       csfsauth_gid_t gid;
       opaque cookie[COOKIE_LEN];
};

struct csfsauth_listgrpmemsokres {
	csfsauth_uid_t uid<>;
        opaque cookie[COOKIE_LEN];
};

union csfsauth_listgrpmemsres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_listgrpmemsokres listgrpmemsres;
default:
	void;
};


struct csfsauth_grp2gidargs {
	csfsauth_grp_t grpname;
};

union csfsauth_grp2gidres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_gid_t gid;
default:
 	void;
};


struct csfsauth_gid2grpargs {
	csfsauth_gid_t gid;
};

union csfsauth_gid2grpres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_grp_t grpname;
default:
 	void;
};


struct csfsauth_getgrpkeyargs {
	csfsauth_gid_t gid;
	csfsauth_uid_t uid;
};

union csfsauth_getgrpkeyres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	csfsauth_key_t key;
default:
	void;
};


struct csfsauth_addgrpmemargs {
	csfsauth_gid_t gid;
	csfsauth_uid_t uid;
	csfsauth_key_t pk;
};

union csfsauth_addgrpmemres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	void;
default:
	void;
};


struct csfsauth_delgrpmemargs {
	csfsauth_gid_t gid;
	csfsauth_uid_t uid;
};

union csfsauth_delgrpmemres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	void;
default:
	void;
};


struct csfsauth_chgrpkeyargs {
	csfsauth_gid_t gid;
	csfsauth_uid_t uid;
        csfsauth_key_t key;
};

union csfsauth_chgrpkeyres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	void;
default:
	void;
};


struct csfsauth_chgrpownerargs {
	csfsauth_gid_t gid;
	csfsauth_uid_t uid;
};

union csfsauth_chgrpownerres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	void;
default:
	void;
};


struct csfsauth_getmemgrpsargs {
        csfsauth_uid_t uid;
};

struct csfsauth_getmemgrpsokres {
        csfsauth_gid_t gid<MAX_GRPS_PER_USER>;
        csfsauth_key_t key<MAX_GRPS_PER_USER>;
};

union csfsauth_getmemgrpsres switch (csfsauthstat status) {
case CSFSAUTH_OK:
        csfsauth_getmemgrpsokres getmemgrpsres;
default:
	void;
};

struct csfsauth_usringrpargs {
	csfsauth_uid_t uid;
	csfsauth_gid_t gid;
};

union csfsauth_usringrpres switch (csfsauthstat status) {
case CSFSAUTH_OK:
	void;
default:
	void;
};

program CSFSAUTH_PROG {
  version CSFSAUTH_VERS {
    csfsauth_adduserres
      ADDUSER(csfsauth_adduserargs) = 1;
    csfsauth_deluserres 		
      DELUSER(csfsauth_deluserargs) = 2;
    csfsauth_listusersres 		
      LISTUSERS(csfsauth_listusersargs) = 3;
    csfsauth_user2uidres
      USER2UID(csfsauth_user2uidargs) = 4;
    csfsauth_uid2userres 
      UID2USER(csfsauth_uid2userargs) = 5;
    csfsauth_getuserpkres 		
      GETUSERPK(csfsauth_getuserpkargs) = 6;
    csfsauth_setuserpkres 		
      SETUSERPK(csfsauth_setuserpkargs) = 7;
    csfsauth_getuserpermsres
      GETUSERPERMS(csfsauth_getuserpermsargs) = 8;
    csfsauth_setuserpermsres
      SETUSERPERMS(csfsauth_setuserpermsargs) = 9;

    csfsauth_addgrpres 		        
      ADDGRP(csfsauth_addgrpargs) = 10;
    csfsauth_delgrpres 		  
      DELGRP(csfsauth_delgrpargs) = 11;
    csfsauth_listgrpsres 		
      LISTGRPS(csfsauth_listgrpsargs) = 12;
    csfsauth_grp2gidres
      GRP2GID(csfsauth_grp2gidargs) = 13;
    csfsauth_gid2grpres
      GID2GRP(csfsauth_gid2grpargs) = 14;
    csfsauth_getgrpkeyres
      GETGRPKEY(csfsauth_getgrpkeyargs) = 15; 

    csfsauth_addgrpmemres
      ADDGRPMEM(csfsauth_addgrpmemargs) = 16; 
    csfsauth_delgrpmemres 
      DELGRPMEM(csfsauth_delgrpmemargs) = 17;
    csfsauth_listgrpmemsres
      LISTGRPMEMS(csfsauth_listgrpmemsargs) = 18;
    csfsauth_chgrpkeyres 
      CHGRPKEY(csfsauth_chgrpkeyargs) = 19;
    csfsauth_chgrpownerres 
      CHGRPOWNER(csfsauth_chgrpownerargs) = 20;

    csfsauth_getmemgrpsres 
      GETMEMGRPS(csfsauth_getmemgrpsargs) = 21;
 
    csfsauth_usringrpres
      USRINGRP(csfsauth_usringrpargs) = 22;
  } = 1;
} = 300055;

#ifdef RPC_HDR
%#endif /* !_CSFSAITH_PROT_H_EXTRA_ */
#endif /* RPC_HDR */



