/* $Id: csfssd_prot.x,v 1.17 1999/04/26 16:14:05 fubob Exp $ */

/* Based on nfs_prot.x
 *
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

const CSFSSD_PORT          = 2049;
const CSFSSD_BLOCK_SIZE    = 8192;
const CSFSSD_MAXDATA       = 57344;  /* 8192*7 */
const CSFSSD_MAXPATHLEN    = 1024;
const CSFSSD_MAXNAMLEN	   = 255;
const CSFSSD_MAXCHALLENGE  = 8;
const CSFSSD_FHSIZE   	   = 32;
const CSFSSD_COOKIESIZE	   = 4;
const CSFSSD_FIFO_DEV      = -1;	/* size kludge for named pipes */

const CSFSSD_KEY_LENGTH    = 16; /* Number of bytes for the key. */
                                 /* should be multiple of 4 for word
					boundaries */

const CSFSSD_NUM_META      = 1; /* Number of hashes for metadata */
const CSFSSD_NUM_DIRECT    = 1; /* Number of pointers to blocks in an inode */
const CSFSSD_NUM_SINDIRECT = 1; /* The array member of an inode's block
				 pointer which is singly indirect */
const CSFSSD_NUM_DINDIRECT = 1;  /* The array member of an inode's block
				 pointer which is doubly indirect */
const CSFSSD_BLOCK_PTR_SIZE=32; /* size of data pointer = 4 byte addr,
				                           8 byte IV,
				            		   20 byte hash */
const CSFSSD_IV_LEN =       8; /* IVs are 8 bytes */
const CSFSSD_DPTS = 3;
const CSFSSD_HASHES = 4;
const CSFSSD_INTEGRITY_FIELD_LENGTH = 20; /* Num bytes of integ. field
					     in inode */
const CSFSSD_MAXDP_REQ = 7;  /* Max number of DP blocks one can request
				at a time */


/*
 * File types
 */
const CSFSSDMODE_FMT  = 0170000;	/* type of file */
const CSFSSDMODE_DIR  = 0040000;	/* directory */
const CSFSSDMODE_CHR  = 0020000;	/* character special */
const CSFSSDMODE_BLK  = 0060000;	/* block special */
const CSFSSDMODE_REG  = 0100000;	/* regular */
const CSFSSDMODE_LNK  = 0120000;	/* symbolic link */
const CSFSSDMODE_SOCK = 0140000;	/* socket */
const CSFSSDMODE_FIFO = 0010000;	/* fifo */

/*
 * Error status
 */
enum csfssdstat {
	CSFSSD_OK= 0,		/* no error */
	CSFSSDERR_PERM=1,	/* Not owner */
	CSFSSDERR_NOENT=2,	/* No such file or directory */
	CSFSSDERR_IO=5,		/* I/O error */
	CSFSSDERR_NXIO=6,	/* No such device or address */
	CSFSSDERR_ACCES=13,	/* Permission denied */
	CSFSSDERR_EXIST=17,	/* File exists */
	CSFSSDERR_NODEV=19,	/* No such device */
	CSFSSDERR_NOTDIR=20,	/* Not a directory*/
	CSFSSDERR_ISDIR=21,	/* Is a directory */
	CSFSSDERR_FBIG=27,	/* File too large */
	CSFSSDERR_NOSPC=28,	/* No space left on device */
	CSFSSDERR_ROFS=30,	/* Read-only file system */
	CSFSSDERR_NAMETOOLONG=63,/* File name too long */
	CSFSSDERR_NOTEMPTY=66,	/* Directory not empty */
	CSFSSDERR_DQUOT=69,	/* Disc quota exceeded */
	CSFSSDERR_STALE=70,	/* Stale CSFSSD file handle */
	CSFSSDERR_WFLUSH=99,	/* write cache flushed */
	CSFSSDERR_LOCK=100	/* file already locked */
};

/*
typedef opaque challenge<CSFSSD_MAXCHALLENGE>;

struct chalargs {
	challenge chal;
};

struct chalokres {
	challenge chal;
};

union chalres switch(csfssdstat status) {
case CSFSSD_OK:
	chalokres reply;
default:
	void;
};

*/

/*
 * File types
 */
enum csfssd_ftype {
	CSFSSD_NFNON = 0,	/* non-file */
	CSFSSD_NFREG = 1,	/* regular file */
	CSFSSD_NFDIR = 2,	/* directory */
	CSFSSD_NFBLK = 3,	/* block special */
	CSFSSD_NFCHR = 4,	/* character special */
	CSFSSD_NFLNK = 5,	/* symbolic link */
	CSFSSD_NFSOCK = 6,	/* unix domain sockets */
	CSFSSD_NFBAD = 7,	/* unused */
	CSFSSD_NFFIFO = 8 	/* named pipe */
};

/*
 * File access handle
 */
struct csfssd_fh {
  opaque data[CSFSSD_FHSIZE]; /* translates to inode and gen num */
};

typedef opaque csfssd_hash[CSFSSD_INTEGRITY_FIELD_LENGTH];
typedef opaque csfssd_dp_block[CSFSSD_BLOCK_PTR_SIZE];

/*
 * File attributes
 */
struct csfssd_fattr {
  csfssd_ftype type;	/* file type */
  unsigned mode;	/* protection mode bits */
  unsigned long atime;	/* time of last access */
  unsigned long mtime;	/* time of last modification */
  unsigned long ctime;	/* time of last change */
  unsigned long dtime;	/* time of deletion */
  unsigned uid;		/* owner user id */
  unsigned gid;		/* owner group id */
  unsigned long size;	/* num bytes of file data */
  unsigned long gkid;   /* Group key ID */
  opaque owner_lockbox[CSFSSD_KEY_LENGTH];  /* file key locked for 
					       owner's use only */
  opaque group_lockbox[CSFSSD_KEY_LENGTH];  /* file key locked for
					       group's use only */
  opaque old_owner_lockbox[CSFSSD_KEY_LENGTH]; /* old file key locked
						  for owner's use only */
  opaque old_group_lockbox[CSFSSD_KEY_LENGTH]; /* old file key locked for
						  group's use only */
  unsigned link_count;	/* # hard links */
  unsigned generation;
  unsigned long inode_idx; 
  csfssd_dp_block block_ptr[CSFSSD_DPTS];
  unsigned locked_by;   /* UID */
  unsigned locked_duration; /* Time lock expires, time past locked_at.
			       0 means unlocked */
  unsigned long locked_at;  /* Time locked, seconds since 1970 */
  unsigned long magic;      
  csfssd_hash block_hash[CSFSSD_HASHES];
  csfssd_hash aicf; /* Integrity field */
};

/*
 * Subset of file attributes which can be set
 */
struct csfssd_sattr {
  unsigned mode;	/* protection mode bits */
  unsigned uid;		/* owner user id */
  unsigned gid;		/* owner group id */
  unsigned long size;	/* file size in bytes */
  unsigned long atime;	/* time of last access */
  unsigned long mtime;	/* time of last modification */
  unsigned long dtime;	/* time of deletion */
  unsigned long gkid;	/* Group key ID */
  opaque owner_lockbox[CSFSSD_KEY_LENGTH];  /* file key locked for 
					       owner's use only */
  opaque group_lockbox[CSFSSD_KEY_LENGTH];  /* file key locked for */
  csfssd_hash aicf;	/* Integrity field */  		
};


/*
 * Reply status with file attributes
 */
union csfssd_attrstat switch (csfssdstat status) {
case CSFSSD_OK:
  csfssd_fattr attributes;
default:
  void;
};

struct csfssd_sattrargs {
  csfssd_fh file;
  csfssd_sattr attributes;
};


struct csfssd_lookupargs {
  unsigned inode;
};


union csfssd_lookupres switch (csfssdstat status) {
case CSFSSD_OK:
  csfssd_fh file;
default:
  void;
};

/*
 * Arguments to remote read
 */
struct csfssd_readargs {
  csfssd_fh file;	/* handle for file */
  unsigned offset;	/* block offset in file */
  unsigned size;	/* total read count (from this offset)
			  in blocks */
};

/*
 * Status OK portion of remote read reply
 */
struct csfssd_readokres {
  opaque data<CSFSSD_MAXDATA>;
};

union csfssd_readres switch (csfssdstat status) {
case CSFSSD_OK:
  csfssd_readokres reply;
default:
  void;
};

/*
 * Arguments to remote write 
 */
struct csfssd_writeargs {
  csfssd_fh file;	/* handle for file */
  unsigned offset;	/* beginning byte offset in file */
  unsigned size;	/* total write count (to this offset)*/
  opaque data<CSFSSD_MAXDATA>;
};


struct csfssd_lockargs {
  csfssd_fh file;
  unsigned uid; 	/* not really necessary, just easier for now */
  unsigned duration;	/* duration requested */
};


union csfssd_lockres switch (csfssdstat status) {
 case CSFSSD_OK:
   unsigned locked_duration;  /* granted lock until */
 case CSFSSDERR_LOCK:
   unsigned already_locked_duration;  /* locked for this many more secs */
 default:
   void;
};


struct csfssd_unlockargs {
  csfssd_fh file;
  unsigned uid; 	/* not really necessary, just easier for now */
};


struct csfssd_readdpargs {
  csfssd_fh file;
  unsigned dp_nums<CSFSSD_MAXDP_REQ>;   /* The names of the data pointers
					 0=singly indirect, 1=d. indirect,
					 2= left child of dindir, etc */
};


struct csfssd_readdpres {
  csfssd_dp_block dp<CSFSSD_MAXDP_REQ>;
};

struct csfssd_writedpargs {
  csfssd_fh file;
  unsigned dp_nums<CSFSSD_MAXDP_REQ>;   /* The names of the data pointers
					 0=singly indirect, 1=d. indirect,
					 2= left child of dindir, etc */
  csfssd_dp_block dp<CSFSSD_MAXDP_REQ>;
};

struct csfssd_statfsokres {
  unsigned bsize;	/* fundamental file system block size */
  unsigned blocks;	/* total blocks in file system */
  unsigned bfree;	/* free blocks in fs */
  unsigned bavail;	/* free blocks avail to non-superuser */
};

union csfssd_statfsres switch (csfssdstat status) {
case CSFSSD_OK:
  csfssd_statfsokres reply;
default:
  void;
};


struct csfssd_createargs {
  unsigned long dir_inode_idx; /* Allocate inode near dir_inode_idx */
};

union csfssd_createres switch (csfssdstat status) {
 case CSFSSD_OK:
   unsigned long inode_idx;
 default:	
   void;
};


struct csfssd_linkargs {
  unsigned long inode_idx; /* make another link to inode_idx */
};

union csfssd_linkres switch (csfssdstat status) {
 case CSFSSD_OK:
   unsigned link_count;
 default:	
   void;
};


struct csfssd_unlinkargs {
  unsigned long inode_idx; /* decrement link count of inode_idx */
};

union csfssd_unlinkres switch (csfssdstat status) {
 case CSFSSD_OK:
   unsigned link_count;
 default:	
   void;
};


/*
 * Remote file service routines
 */
program CSFSSD_PROGRAM {
	version CSFSSD_VERSION {

		csfssd_attrstat 
		CSFSSDPROC_GETATTR(csfssd_fh) =	1;

		csfssd_attrstat 
		CSFSSDPROC_SETATTR(csfssd_sattrargs) = 2;

		csfssd_lookupres 
		CSFSSDPROC_LOOKUP(csfssd_lookupargs) = 3;

		csfssd_readres 
		CSFSSDPROC_READ(csfssd_readargs) = 4;

		csfssd_statfsres
		CSFSSDPROC_WRITE(csfssd_writeargs) = 5;

	        csfssd_lockres
		CSFSSDPROC_LOCK(csfssd_lockargs) = 6;

		csfssdstat
		CSFSSDPROC_UNLOCK(csfssd_unlockargs) = 7;

		csfssd_readdpres
		CSFSSDPROC_READ_DP_BLOCK(csfssd_readdpargs) = 8;

		csfssd_statfsres
		CSFSSDPROC_WRITE_DP_BLOCK(csfssd_writedpargs) = 9;

		csfssd_statfsres
		CSFSSDPROC_STATFS(csfssd_fh) = 10;

	        csfssd_createres
	        CSFSSDPROC_CREATE(csfssd_createargs) = 11;

	        csfssd_linkres
	        CSFSSDPROC_LINK(csfssd_linkargs) = 12;

	        csfssd_unlinkres
	        CSFSSDPROC_UNLINK(csfssd_unlinkargs) = 13;

		/*
                chalres
	        CSFSSDPROC_CHALLENGE(chalargs) = 14; 
		*/

	} = 1;
} = 100004;


