/* $ Id: $ */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <unistd.h>
#include <time.h>
#include "misc/ipc.h"


/* This won't work in linux otherwise */
#ifdef SCM_RIGHTS
#undef SCM_RIGHTS
#endif

/* Borrowed from Steven's _Advanced UNIX Programming_ */

int serv_listen (const char *name) {
  int fd,len;
  struct sockaddr_un unix_addr;

  /* Create UNIX domains socket */
  if ( (fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
    return (-1);

  unlink(name); /* In case it already exists */

  /* Fill in socket addr structure */
  memset(&unix_addr, 0, sizeof(unix_addr));
  unix_addr.sun_family = AF_UNIX;

  if (snprintf(unix_addr.sun_path, UNIX_PATH_MAX, "%s", name) 
      == UNIX_PATH_MAX)
    return(-2);

#ifdef SCM_RIGHTS   /* 4.3BSD Reno and later */
  len = sizeof(unix_addr.sun_len) + sizeof(unix_addr.sun_family) +
    strlen(unix_addr.sun_path) + 1;
  unix_addr.sun_len = len;
#else
  len = strlen(unix_addr.sun_path) + sizeof(unix_addr.sun_family);
#endif

  /* Bind the name to the descriptor */
  if (bind(fd, (struct sockaddr *) &unix_addr, len) < 0)
    return(-3);
  
  if (listen(fd, 5) < 0) /* Tell the kernel we're a server */
    return(-4);

  return(fd);
}

int serv_accept (int listenfd, uid_t *uidptr) {
  int clifd, len;
  time_t staletime;
  struct sockaddr_un unix_addr;
  struct stat statbuf;

  len = sizeof(unix_addr);
  if ( (clifd = accept(listenfd, (struct sockaddr *) &unix_addr, &len)) < 0)
    return(-1); /* often errno=EINTR, if signal caught */
  
  /* Obtains the client's UID from its calling address */
#ifdef SCM_RIGHTS /* 4.3BSD Reno and later */
  len -= sizeof(unix_addr.sun_len) - sizeof(unix_addr.sun_family);
#else  /* Vanilla 4.3BSD */
  len -= sizeof(unix_addr.sun_family); /* len of full path */
#endif
  unix_addr.sun_path[len]=0;  /* Null terminate */

  if (stat(unix_addr.sun_path, &statbuf) <0)
    return(-2);
#ifdef S_ISSOCK /* not defined for SVR4 */
  if (S_ISSOCK(statbuf.st_mode) == 0)
    return(-3);  /* not a socket */
#endif
  if ((statbuf.st_mode & (S_IRWXG | S_IRWXO)) ||
      (statbuf.st_mode & S_IRWXU) != S_IRWXU)
    return(-4);  /* is not rwx______ */

  staletime = time(NULL) - STALE;
  if (statbuf.st_atime < staletime ||
      statbuf.st_ctime < staletime ||
      statbuf.st_mtime < staletime)
    return(-5);  /* inode is too old */

  if (uidptr != NULL)
    *uidptr = statbuf.st_uid;   /* Return UID of caller */

  unlink(unix_addr.sun_path);   /* we're done with pathname now */

  return(clifd);
}


int cli_conn(const char *name) {
  int fd, len;
  struct sockaddr_un unix_addr;

  /* Create a UNIX domain stream socket */
  if ( (fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
    return(-1);

  /* Fill socket address structure w/ our address */
  memset(&unix_addr, 0, sizeof(unix_addr));
  unix_addr.sun_family = AF_UNIX;
  if (snprintf(unix_addr.sun_path, UNIX_PATH_MAX, "%s%05d", CLI_PATH, getpid())
      == UNIX_PATH_MAX)
    return(-2);
  
#ifdef SCM_RIGHTS /* 4.3BSD Reno and later */
  len = sizeof(unix_addr.sun_len) + sizeof(unix_addr.sun_family) +
    strlen(unix_addr.sun_path) + 1;
  unix_addr.sun_len = len;
#else
  len = strlen(unix_addr.sun_path) + sizeof(unix_addr.sun_family);
  if (len != 16)
    err_quit("length != 16"); /* Hack? */
#endif

  unlink(unix_addr.sun_path); /* In case it already exists */
  if (bind(fd, (struct sockaddr *) &unix_addr, len) < 0)
    return(-3);
  if (chmod(unix_addr.sun_path, CLI_PERM) < 0)
    return(-4);

  /* Fill socket addr strut with server's addr */
  memset(&unix_addr, 0, sizeof(unix_addr));
  unix_addr.sun_family = AF_UNIX;
  if (snprintf(unix_addr.sun_path, UNIX_PATH_MAX, "%s", name) ==
      UNIX_PATH_MAX)
    return(-5);

#ifdef SCM_RIGHTS /* 4.3 BSD Reno and later */
  len = sizeof(unix_addr.sun_len) + sizeof(unix_addr.sun_family) +
    strlen(unix_addr.sun_path);
  unix_addr.sun_len = len;
#else
  len = strlen(unix_addr.sun_path) + sizeof(unix_addr.sun_family);
#endif
  
  if (connect(fd, (struct sockaddr *) &unix_addr, len) <0)
    return(-6);

  return(fd);
}
