/* 
** Portions borrowed from Kevin Fu's 6.033 team, 1997
** $Id: csfssd.h,v 1.26 1999/04/29 05:47:14 fubob Exp $
**
** $Log: csfssd.h,v $
** Revision 1.26  1999/04/29 05:47:14  fubob
** i hate bmap.
**
** Revision 1.25  1999/04/28 23:43:44  fubob
** almost done with blockmapping [bmap()] routine and block allocation
** routines.  Painly computation of singly/doubly indirect pointers.
** yuck.
**
*/

/* 
 * These are middle-level functions which code in sd_server.c may call.
 * 
 */

#ifndef __CSFSSD_HEADER__
#define __CSFSSD_HEADER__

#include "misc/types.h"
#include "csfssd_prot.h"
#include "disk.h"
#include <time.h>

#ifdef MIN
#undef MIN
#endif
#ifdef MAX
#undef MAX
#endif

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

#define CSFSSD_SUPER_MAGIC       0xBC
#define CSFSSD_INODE_MAGIC       0xAD
#define CSFSSD_SUPER_BLOCK       0  /* absolute location of superblock */
#define CSFSSD_ROOT_INODE        0
#define CSFSSD_MAX_FILESIZE      955377664UL      /* ~911MB */
#define CSFSSD_MAX_MOUNTS        10 /* Max num of mounts before forcing
				       consistency check */
#define CSFSSD_MAX_INTERVAL      86400 /* Max num seconds since last 
				       consistency check, 86400 sec = 1 day */
#define CSFSSD_MAX_LEASE_SECONDS 120  /* Max seconds one can lock a file */


/* On-disk (and in-memory) representation of the SB */
typedef struct sb_s {
  u32   num_blocks;           /* num blocks disk has */
  u32   num_inodes;           /* num of inodes disk can have */
  u32   first_freemap_block;  /* first block of the freemap,
				 freemap size = nblocks bits */
  u32   first_inode_block;    /* first inode block,
				 num inodes = f(nblocks) */
  u32   first_data_block;     /* first block of data */
  u16   mount_count;          /* number of mounts since consistency check */
  u16   dirty;                /* shutdown cleanly? */
  u32   lastchecked_time;     /* last time checked FS in seconds since
				 Jan 1,1970 */

  /* The following fields are correct
     _only if_ the dirty bit is FALSE. */
  u32   num_free_blocks;      /* Number of data blocks unused */
  u32   num_free_inodes;      /* Number of free inodes left */

  u32   magic;                /* CSFSSD_SUPER_MAGIC */

} sb_t;


#define CSFSSD_SUPER_PADDING (CSFSSD_BLOCK_SIZE - sizeof(sb_t))
#define CSFSSD_INODE_SIZE  512    /* Num of bytes for inode structure */

/* On disk SB data structure (just for padding) */
typedef struct sb_disk_s {
  char magic[CSFSSD_SUPER_PADDING];
  sb_t sb;
} sb_disk_t;

/* In-memory representation of the FS */

typedef struct csfssd_s {
  disk_t       *disk;         /* our virtual disk */
  sb_t         *sb;           /* our superduperblock */
  void         *freemap;      /* our vast freemap */
  u8           mount_flags;   /* R/W, RO, etc */
} csfssd_t;

typedef struct csfssd_dp_s {
  u32 dp;
  u8 hash[CSFSSD_INTEGRITY_FIELD_LENGTH];
  u8 iv[CSFSSD_IV_LEN];
} csfssd_dp_t;


typedef struct csfssd_dp_block_s {
  u32 dp[CSFSSD_BLOCK_SIZE/sizeof(csfssd_dp_t)];
  u8 hash[CSFSSD_BLOCK_SIZE/sizeof(csfssd_dp_t)][CSFSSD_INTEGRITY_FIELD_LENGTH];
  u8 iv[CSFSSD_BLOCK_SIZE/sizeof(csfssd_dp_t)][CSFSSD_IV_LEN];
} csfssd_dp_block_t;


typedef struct csfssd_fh_s {
  u32 inode;
  u16 gen;
  u8 pad[26];
} csfssd_fh_t;

/*
 * On-disk representation of the i-node. Remember that for Linux, it
 * is 4-byte word aligned!  The order of structure members makes a
 * difference in the physical size.  An inode is 256 bytes -> 32 inodes
 * fill a block.
 *
 * Portions borrowed from Derek Atkins <warlord@MIT.EDU> 94-10-20 
 */
typedef struct inode_s {
  u32 magic[51];      /* CSFSSD_INODE_MAGIC */
  u32 mode;
  u32 atime;
  u32 mtime;
  u32 ctime;
  u32 dtime;
  u16 uid;  
  u16 gid;  
  u32 size;
  u32 gkid;      
  u16 link_count; /* number of pointers to this inode, 0=free inode */
  u16 generation;
  u32 inode_idx;  /* Self-identification of this inode - its location */
  u16 locked_by;   /* UID */
  u16 locked_duration;  /* Time lock expires, time past locked_at.
			   0 means unlocked */
  u32 locked_at;  /* Time locked, seconds since 1970 */
  u8 owner_lockbox[CSFSSD_KEY_LENGTH];
  u8 group_lockbox[CSFSSD_KEY_LENGTH]; 
  u8 old_owner_lockbox[CSFSSD_KEY_LENGTH];
  u8 old_group_lockbox[CSFSSD_KEY_LENGTH];

  u8 block_hash [CSFSSD_NUM_META +
		CSFSSD_NUM_DIRECT +
		CSFSSD_NUM_SINDIRECT + 
		CSFSSD_NUM_DINDIRECT] [CSFSSD_INTEGRITY_FIELD_LENGTH];
  u8 aicf[CSFSSD_INTEGRITY_FIELD_LENGTH]; /* Keyed hash of data + metadata */

  csfssd_dp_t block_ptr[CSFSSD_NUM_DIRECT +    /* direct blocks */
	       CSFSSD_NUM_SINDIRECT +  /* Singly indirect blocks */
	       CSFSSD_NUM_DINDIRECT];  /* Doubly indirect blocks */
} inode_t;  



#define CSFSSD_INODES_PER_BLOCK (CSFSSD_BLOCK_SIZE / sizeof(inode_t))


/*
** REQUIRES: Nothing.
** RETURNS: Current time in seconds since the epoch,
**          seconds since Jan 1,1970
** SEE ALSO: time(2)
*/
#define CSFSSD_get_time() (u32)time(NULL);


/** csfssd.c **/

/*
 * REQUIRES: allocated csfssd, legal filename.
 *
 * EFFECTS: Starts up ye olde disk library and makes a fake
 * disk for our FS.  Mounts disk according to mount_flags
 *
 * RETURNS: 0 on success, -1 if something goes wrong.  Sets up
 *         csfssd appropriately
 */
int CSFSSD_mount(csfssd_t *csfssd, char *filename, u8 mount_flags);

/*
 * REQUIRES: allocated csfssd, legal numblocks, legal filename.
 *
 * EFFECTS:  Initializes a blank disk with 'numblocks' blocks
 *           and 'numinodes' inodes.
 *
 * RETURNS: 0 on success, -1 if something goes wrong.
 */
int CSFSSD_init(char *filename, u32 numblocks, u32 numinodes);


/*
 * REQUIRES: valid csfssd
 *
 * RETURNS: 0 if ok, <0 if error
 *
 * EFFECTS: Syncs the disk and closes up our fake diskfile.
 * The csfssd structure is invalid after this procedure
 * since its members are freed away.
 */
int CSFSSD_shutdown(csfssd_t *csfssd);

/*
 * REQUIRES: valid csfssd
 *
 * RETURNS: 0 if ok, <0 if error
 *
 * EFFECTS: Syncs the disk, writes out superblock and freemap.
 */
int CSFSSD_sync(csfssd_t *csfssd);

/* 
 * Requires: valid csfssd, allocated bsize, blocks, bfree, bavail
 *
 * Effects: Returns vital stats about the file system
 *
 * Returns: 0 if ok, <0 otherwise.  Sets bsize, blocks,
 *          bfree, and bavail to:
 *
 *	fundamental file system block size in bytes
 *      total blocks in file system
 * 	free blocks in fs
 *	free blocks avail to non-superuser
 *
 * respectively.
 */
int CSFSSD_statfs(csfssd_t *csfssd, u16 *bsize, u32 *blocks, 
		  u32 *bfree, u32 *bavail);


/*
 * REQUIRES: FS state gone to hell in a handbasket.
 *
 * EFFECTS: Crashes out, hard. Splashes in some 
 * gasoline and burns brightly.
 *
 * RETURNS: You're joking, right?
 */
void CSFSSD_panic(char *message);


/** inode.c **/

/* Requires: inode allocated, inode_idx has correct value
 *
 * Effects: Sets all fields of inode to defaults.  Default zero keys.
 *
 * Returns: 0 if ok, < 0 otherwise
 */
int CSFSSD_resetInode(inode_t *inode, u32 inode_idx);


/*
** REQUIRES: Valid csfssd, inode_idx, uid, duration of lease in seconds.
**
** EFFECTS: Locks the file for uid for duration given in duration.  No
** one but the user UID can modify the file in this time period.
** A user may re-lock the file if the lease does not exceed the
** MAX_LEASE_SECONDS
**
** RETURNS: 0 on success, < 0 on failure.  Places amount of time
**  file left locked in duration, whether or not the lock is acquired.
*/
int CSFSSD_lockFile(csfssd_t *csfssd, u32 inode_idx, u16 uid, 
		    u16 *duration);


/*
** REQUIRES: Valid csfssd, inode, uid, length of lease.
**
** EFFECTS: unLocks the file for uid
**
** RETURNS: 0 on success, < 0 on failure.
*/
int CSFSSD_unlockFile(csfssd_t *csfssd, u32 inode_idx, u16 uid);



/*
** REQUIRES: valid csfssd structure, allocated new_inode_idx
** EFFECTS: finds a location near dir_inode_idx
**          and records this in new_inode_idx.  Sets link_count=1, used.
**          Makes sure inode is zeroed out properly.
** RETURNS: 0 on success, errno and -1 on failure.
*/
int CSFSSD_allocInode(csfssd_t *csfssd, u32 *new_inode_idx, 
		      u32 dir_inode_idx);

/*
** REQUIRES: valid csfssd structure, valid inode at inode_idx, 
**           link count for inode=0.
** EFFECTS: Frees the inode at the given location by writing over inode lnk=0
** RETURNS: 0 on success, errno on failure.
*/
int CSFSSD_deallocInode(csfssd_t *csfssd, u32 inode_idx);

/*
** REQUIRES: valid csfssd structure, valid inode at inode_idx.
** EFFECTS: Writes out inode at the given location.  
** RETURNS: 0 on success, errno on failure.
*/
int CSFSSD_writeInode(csfssd_t *csfssd, inode_t *inode, u32 inode_idx);

/*
** REQUIRES: valid csfssd structure, valid inode (allocated) at inode_idx.
** EFFECTS: Reads in inode at the given location.
** RETURNS: 0 on success, errno on failure.
*/
int CSFSSD_readInode(csfssd_t *csfssd, inode_t *inode, u32 inode_idx);


/** file.c **/

/*
** REQUIRES: csfssd, blocks, valid pointers. valid inode_idx. 
**           first_block_id refers to which 8KB of the file to start
**           reading. numblocks refers to how many to read from that point
**
** EFFECTS: reads in from file data from first_block_id up to 
**     first_block_id + numblocks -1.  Returns data in blocks.
**          
**
** RETURNS: If first_block_id past end of file or error, returns 0 
** blocks read.  Otherwise, returns number of blocks read.  The output
** and input numblocks will not necessarily match.
*/
u16 CSFSSD_readData(csfssd_t *csfssd, u32 inode_idx,
		    u32 first_block_id, u16 numblocks, u8 *blocks);



/*
** REQUIRES: csfssd, blocks, valid pointers.  valid inode_idx.
** first_block_id refers to which block of the file to start reading.
** num_blocks is how many to read from that point.
**
** EFFECTS: writes out from data blocks to file.
** WARNING: it's the responsiblity of the CD to update the inode accordingly.
**
** RETURNS: number of blocks written
*/
u16 CSFSSD_writeData(csfssd_t *csfssd, u32 inode_idx,
		     u32 first_block_id, u16 numblocks, u8 *blocks);


/*
** REQUIRES: csfssd, dp_block, valid pointers. file valid inode.
** block_num is either sindirect or dindirect block
**
** EFFECTS: writes contents of dp_block into dp block block_num
** dp_block_id is a 4-byte quantity.  The high byte represents a tree
** level (0 or 1).  The 2nd MSB represents which first level branch.  The
** 3rd represents which second level branch.  The 4th is not used.
**
**  Examples (in decimal): 
**
**  0   2   0   0 -> read the 2nd data block pointer from at the first
**    branching level (typically a doubly indirect pointer data block)
**
**  1   2 250   0 -> read the 2nd data block pointer from at the first
**    branching level (typically a doubly indirect pointer data block)
**    Then read the 250th child of that block.  This is typically a
**    singly indirect DP block.  Return this block.
**
** RETURNS: 0 if ok, -1 if bad */
int CSFSSD_writeDPBlock(csfssd_t *csfssd, u32 inode_idx,
			u32 dp_block_id, u8 *dp_block);


/*
** REQUIRES: csfssd, file, valid pointers. file valid inode.
** block_num is either sindirect or dindirect block
**
** EFFECTS: reads contents of dp block block_num into dp_block
**
** RETURNS: 0 if ok, -1 if bad
*/
int CSFSSD_readDPBlock(csfssd_t *csfssd, u32 inode_idx,
		       u32 dp_block_id, u8 **dp_block);



/*
 * These are low-level functions which are called only by functions in
 * csfssd.c. 
 */

/** sb.c **/

/* 
** REQUIRES: allocated sb, numblocks multiple of 8, numinodes multiple
**           of CSFSSD_INODES_PER_BLOCK
** EFFECTS: returns a filled-in SB structure with no used inodes.
** RETURNS: 0 on success, < 0 on error
*/
int CSFSSD_createSB(sb_t *sb, u32 numblocks, u32 numinodes);

/*
** REQUIRES: csfssd must have valid disk field. 
**           sb must be allocated.
** EFFECTS: Writes the superblock to disk. syncs.
** RETURNS: 0 on success, < 0 on error
*/
int CSFSSD_writeSB(sb_t *sb, disk_t *disk);

/*
** REQUIRES: sb must be allocated.  disk must be opened.
** EFFECTS: Returns superblock in sb
** RETURNS: 0 on success, < 0 on error
*/
int CSFSSD_readSB(sb_t *sb, disk_t *disk);

/*
 * Requires: csfssd set, disk field set, sb field set
 *
 * Effects: Checks the file system for consistency, attempts to recover
 *
 * Returns: 0 if successful recover, <0 otherwise
 */
int CSFSSD_fsck(sb_t *sb, disk_t *disk);


/** freemap.c **/

/* 
** REQUIRES: numblocks >= 3
** EFFECTS: Calculates the freemap size in bytes
** RETURNS: Size of the freemap in bytes
*/
inline int CSFSSD_sizeofFreemap_inbytes(u32 numblocks);


/* 
** REQUIRES: numblocks >= 3
** EFFECTS: Calculates the freemap size in blocks
** RETURNS: Size of the freemap in blocks
*/
inline int CSFSSD_sizeofFreemap_inblocks(u32 numblocks);

/* 
** REQUIRES: csfssd allocated
**  csfssd->sb allocated, sb->numblocks and sb->first_data_block set
** EFFECTS: allocates space for free map and points csfssd->freemap to it.
**          marks superblock, freemap, and inode area as used in freemap.
** RETURNS: 0 if ok, < 0 if error
*/
int CSFSSD_createFreemap(csfssd_t *csfssd);

/* 
** REQUIRES: Opened disk and setup superblock
** EFFECTS: Sets in-memory freemap to the bits on disk.
**          MUST EXPLICITLY FREE FREEMAP!!!
** RETURNS: Size of freemap (in bytes) on success, < 0 on error
*/
int CSFSSD_readFreemap(sb_t *sb, disk_t *disk, void **freemap);


/* 
** REQUIRES: valid superblock, opened disk, allocated freemap of the correct
**           size
** EFFECTS: Write the freemap appropriately to disk and sync
** RETURNS: 0 on success, < 0 on error
*/
int CSFSSD_writeFreemap(sb_t *sb, disk_t *disk, void *freemap);


/* 
** REQUIRES: valid superblock, opened disk
** EFFECTS: Calculates number of free blocks according to 
**          freemap on disk.  Return result number in num_free_blocks
** RETURNS: 0 on success, < 0 on error
*/
int CSFSSD_dcountFreeBlocks(sb_t *sb, disk_t *disk, u32 *num_free_blocks);

/* 
** REQUIRES: valid superblock, allocated freemap
** EFFECTS: Calculates number of free blocks according to 
**          freemap in memory.  Return result number in num_blocks
** RETURNS: 0 on success, < 0 on error
*/
int CSFSSD_mcountFreeBlocks(sb_t *sb, void *freemap, u32 *num_free_blocks);

/*
** REQUIRES: valid csfssd, allocated array[numblocks]
** EFFECTS:  returns in array list of available block numbers
** which can be used.  Tries to locate blocks near
** nearblock. Marks appropriate blocks as used in freemap.
** Flushes freemap and sb to disk.
** RETURNS: -1 on failure. (disk full) Else 0
**/
int CSFSSD_allocBlocks(csfssd_t *csfssd, u32 nearblock, 
		       u16 numblocks, u32 *blockarray);

/*
** REQUIRES: valid csfssd, allocated array[numblocks]
** EFFECTS: marks list of blocks in array as freed.  Flushes
**          freemap and sb to disk.
** RETURNS: -1 on failure.
**/
int CSFSSD_deallocBlocks(csfssd_t *csfssd, u16 numblocks, 
			 u32 *blockarray);

#endif
