/*
**
** Portions borrowed from CFS - Chernobyl File System
**
** Patrick McCormick
** Kevin Fu
** Joy Nicholson
**
** April 9, 1997
**
** $Id: disk.c,v 1.8 1999/04/28 14:50:51 fubob Exp $
**
** $Log: disk.c,v $
** Revision 1.8  1999/04/28 14:50:51  fubob
** wow, I had a major bug in file.c.  Not done fixing, but almost.  Forgot
** entirely the data block allocation routines.
**
** Revision 1.7  1999/04/27 03:36:06  fubob
** fixed lots of little typos
**
** Revision 1.6  1999/04/26 16:14:23  fubob
** fixed silly mistakes (printf problems), fixed inode size to 512 bytes.
**
** Revision 1.5  1999/04/23 02:20:09  fubob
** cleaned up the csfssd.h interface.  using inode *numbers* instead of literal
** *inodes* as the argument.  Moved auxillary code (freemap, sb) to
** csfssd_aux.h
**
**
**/

/**
 * This is a list of stubs for the Disk class.
 **/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "misc/types.h"
#include "disk.h"
/*#include "csfssd.h"*/
#include "csfssd_prot.h"
#include "DEBUG.h"
#include "ASSERT.h"

int createDisk (char *filename, u32 nblocks) {
  FILE *fp;
  struct stat st;
  u8 buffer[CSFSSD_BLOCK_SIZE];
  u32 i;
  int blockswrite;

  ASSERT(filename != NULL);
  
  DEBUGP(DEBUG_VERBOSE, ("createDisk(%s,%lu): entering\n",
			 filename, nblocks));
  
  /* check if file exists */
  if (stat (filename, &st) == 0) { 
    
    DEBUGP(DEBUG_CONCISE,
	   ("createDisk: \"%s\" already exists.\n",
	    filename));
    return (-1);
  } 

  if ( ( fp = fopen(filename, "wb") ) == NULL ) {
    DEBUGP (DEBUG_CONCISE, 
	    ("createDisk: can't fopen \"%s\"\n", filename));
    return (-1);
  }

  memset(buffer, 0, CSFSSD_BLOCK_SIZE);

  for (i=0; i < nblocks; i++) {
    if ( (blockswrite = fwrite(buffer, CSFSSD_BLOCK_SIZE, 1, fp)) != 1) {
      DEBUGP(1, ("createDisk: fwrite returned %d, expected 1. yuck.\n",
		 blockswrite));
      return -1;
    }
  }


  fclose(fp);

  DEBUGP(DEBUG_VERBOSE, ("createDisk(%s,%lu): exiting\n"));
  return 0;
}

disk_t *openDisk (char *filename, u32 *nblocks)
{
  struct stat st;
  unsigned long bytecount;
  disk_t *disk;
  
  ASSERT(filename != NULL);
  
  DEBUGP(DEBUG_VERBOSE, ("openDisk(%s,%p): entering\n",
			 filename, nblocks));
  
  
  disk = (disk_t*)malloc(sizeof(disk_t));
  memset(disk, 0, sizeof(disk_t));
  
  /* if no diskfile */
  if (stat (filename, &st) < 0) { 
    
    DEBUGP(DEBUG_CONCISE,
	   ("openDisk: \"%s\" does not exist.\n",
	    filename));
    free(disk);
    return NULL;
  } 
  
  if ( ( disk->fp = fopen(filename, "rb+") ) == NULL ) {
    DEBUGP(1, ("openDisk: fopen rb+ failed for  \"%s\"\n", filename));
    free(disk);
    return NULL;
  }
  
  /* Check the size. */
  
  if (fseek(disk->fp, 0L, SEEK_END) < 0) {
    DEBUGP(DEBUG_CONCISE, ("openDisk: fseek returned < 0. Yuck.\n"));

    fclose(disk->fp);
    free(disk);
    return NULL;
  }
  
  bytecount = ftell(disk->fp); /* Last byte of file */
  
  if ( (bytecount % CSFSSD_BLOCK_SIZE) != 0 ) {
    DEBUGP(DEBUG_CONCISE,
	   ("openDisk: disk size, %d bytes, not multiple of 8KB\n",
	    bytecount));

    fclose(disk->fp);
    free(disk);
    return NULL;
  }
  
  *nblocks = bytecount/CSFSSD_BLOCK_SIZE;
  
  rewind(disk->fp); /* go back to beginning of file. */
  
  DEBUGP(DEBUG_VERBOSE, ("openDisk: exiting\n"));
  
  return disk;
}

int closeDisk (disk_t *disk) {
  DEBUGP(DEBUG_VERBOSE, ("closeDisk(%p): entering\n", disk));

  fclose(disk->fp);
  free(disk);

  DEBUGP(DEBUG_VERBOSE, ("closeDisk: exiting\n"));

  return 0;
}

int readBlock (disk_t *disk, u32 blocknr, void *block)
{
    size_t blocksread;

    ASSERT(block != NULL);
    ASSERT(disk != NULL);

    DEBUGP(DEBUG_VERBOSE,
	   ("readBlock(%p,%lu,%p): entering\n", disk, blocknr, block));

    if (fseek(disk->fp, blocknr * CSFSSD_BLOCK_SIZE, SEEK_SET) < 0) {
      DEBUGP(DEBUG_CONCISE,
	     ("readBlock: fseek returned < 0. Yuck.\n"));
      return -1;
    }

    if ( (blocksread = fread(block, CSFSSD_BLOCK_SIZE, 1, disk->fp)) != 1) {
      DEBUGP(DEBUG_CONCISE,
	     ("readBlock: fread returned %d, expected 1. Yuck.\n",
	      blocksread));
      return -1;
    }

    printf("%d\n", *((u8 *)block));

    DEBUGP(DEBUG_VERBOSE, ("readBlock: exiting\n"));
    
    return 0;
}

int writeBlock (disk_t *disk, u32 blocknr, void *block) 
{
    size_t blockswrite;

    ASSERT(block != NULL);
    ASSERT(disk != NULL);

    DEBUGP(DEBUG_VERBOSE,
	   ("writeBlock(%p,%lu,%p): entering\n",
	    disk, blocknr, block));

    if (fseek(disk->fp, blocknr * CSFSSD_BLOCK_SIZE, SEEK_SET) < 0) {
      DEBUGP(DEBUG_CONCISE,
	     ("writeBlock: fseek returned < 0. yuck.\n"));
      return -1;
    }

    printf("%d\n", *((u8 *)block));

    if ( (blockswrite = fwrite(block, CSFSSD_BLOCK_SIZE, 1, disk->fp)) != 1) {
      DEBUGP(1, ("writeBlock: fwrite returned %d, expected 1. yuck.\n",
		 blockswrite));
      return -1;
    }

    DEBUGP(DEBUG_VERBOSE,
	   ("writeBlock: exiting.\n"));
    
    return 0;
}

int syncDisk (disk_t *disk) 
{
    DEBUGP(DEBUG_VERBOSE, ("syncDisk: entering.\n"));
    
    ASSERT(disk != NULL);

    if ( fflush(disk->fp) != 0 ) {
      return (-1);
    }

    DEBUGP(DEBUG_VERBOSE, ("syncDisk: exiting\n"));
    
    return(0);
}

void set_Sync(disk_t *disk, int sync) {

  if (sync == 1) {
    /* Must flush buffers before setting synchronous */
    fflush(disk->fp);
    
    setvbuf(disk->fp, (char *)NULL, _IONBF, 0);
  } else {
   
    setvbuf(disk->fp, disk_buffer, _IOFBF, CSFSSD_BLOCK_SIZE*2);
  }
}



