/**
** Portions from CFS - Chernobyl File System
**
** Patrick McCormick
** Kevin Fu
** Joy Nicholson
**
** April 9, 1997
**
** Low level disk routines
**
** $Id: disk.h,v 1.2 1999/04/21 15:47:10 fubob Exp $
**
** $Log: disk.h,v $
** Revision 1.2  1999/04/21 15:47:10  fubob
** wow, my odl code was crap.  The interface is much cleaner now.  Automated
** testing program in sdtest.c.  I brought the inode down to 256 bytes. Yeah!
**
**
**/

/**
 * This is a list of stubs for the Disk class.
 **/

#ifndef __DISK_H__
#define __DISK_H__

#include "misc/types.h"
#include "csfssd_prot.h"

#ifdef __cplusplus
extern "C" {
#endif
  
typedef struct disk_s {
  FILE *fp;
  /* For future expansion, caching */
} disk_t;

char disk_buffer[CSFSSD_BLOCK_SIZE*2];

/*
 * Effects: creates a disk of nblocks.  Closes the file again
 * Returns: 0 if ok, if file already exists or error, -1
 */
int createDisk (char *filename, u32 nblocks);

/**
 * EFFECTS: Opens a file (filename) for reading and writing.  The file
 * size must be fixed at multiple of CSFSSD_BLOCK_SIZE bytes.  
 *
 * RETURNS: If successful, return a descriptor used to access the disk.
 * and the number of CSFSSD_BLOCK_SIZE blocks in blocknum.
 * Otherwise return a negative number if an underlying UNIX sys call fails,
 * if nbytes is not a multiple of CSFSSD_BLOCK_SIZE.
 *  
 */
disk_t* openDisk (char *filename, u32 *nblocks);

/* Effects: closes and deallocates disk in memory. flush 
 * Returns: 0 if ok, -1 otherwise
 */
int closeDisk (disk_t *disk);

/*
 * REQUIRES: disk be a valid file ptr.
 *
 * EFFECTS: Reads disk block blocknr from the disk pointed to by disk.
 *
 * RETURNS: Returns 0 if successful, a negative number otherwise.
 */
int  readBlock (disk_t *disk, u32 blocknr, void *block);

/*
 * REQUIRES: disk a valid descriptor
 *
 * EFFECTS: Writes the data in block to disk block blocknr pointed to by
 *          disk.  blocknr ranges from 0 to size of disk -1
 *
 * RETURNS: 0 if successful, otherwise a negative number
 */
int  writeBlock (disk_t *disk, u32 blocknr, void *block);

/*
 * EFFECTS: Forces all outstanding writes to disk
 */
int  syncDisk (disk_t *disk);

/* 
 * Effects: If sync=1, sets the disk to have synchronous writes (no
 * buffering, slower). Otherwise sets to asynchronous.
 */
void set_Sync(disk_t *disk, int sync);

#ifdef __cplusplus
}
#endif

#endif
