#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <errno.h>

#include "csfssd.h"
#include "csfssd_prot.h"
#include "DEBUG.h"
#include "ASSERT.h"

extern int errno;

int main (int argc, char *argv[]) {
  int  numblocks, numinodes;
  char *filename = "csfsdisk";

  debuglevel = DEBUG_VERBOSE;

  if ( (argc > 4) || (argc < 3) ) {
    fprintf (stderr, "Usage: mkfs <num blocks> <num inodes> [filename]\n");
    return (-1);
  }

  numblocks = atoi(argv[1]);
  if ((numblocks % 8) != 0) {
    fprintf (stderr, "num blocks must be multiple of 8\n");
    return (-1);
  } else if (numblocks < 2) {
    fprintf (stderr, "num blocks must be > 2.\n");
    return (-1);
  }

  numinodes = atoi(argv[2]);
  if ((numinodes < 1) || 
      (((sizeof(inode_t) * numinodes) % CSFSSD_BLOCK_SIZE)) != 0) {
    fprintf (stderr, "num inodes must be > 0 and multiple of %d.\n",
	     (CSFSSD_BLOCK_SIZE / sizeof(inode_t)));
    return (-1);
  }
  
  if (argc==4) {
    filename = argv[3];
  }
  
  printf ("Creating disk in %s.  %d %dKB blocks, %d inodes, disk size %dKB\n",
	  filename, numblocks, CSFSSD_BLOCK_SIZE/1024, numinodes, 
	  numblocks*CSFSSD_BLOCK_SIZE/1024);
  
  printf("CSFSSD_INODE_SIZE: %d bytes\n",CSFSSD_INODE_SIZE);
  printf("inode_t size: %d bytes\n",sizeof(inode_t));

  /* Create disk */
  if (CSFSSD_init(filename, numblocks, numinodes) < 0 ) {
    fprintf (stderr, "mkfs.main:Could not create %s: %s\n", filename,
	     strerror(errno));
    return (-1);
  }

  return (0);
}
