/* $Id: sdtest.c,v 1.6 1999/04/28 14:50:51 fubob Exp $
 * $Log: sdtest.c,v $
 * Revision 1.6  1999/04/28 14:50:51  fubob
 * wow, I had a major bug in file.c.  Not done fixing, but almost.  Forgot
 * entirely the data block allocation routines.
 *
 * Revision 1.5  1999/04/27 03:36:06  fubob
 * fixed lots of little typos
 *
 * Revision 1.4  1999/04/26 16:14:23  fubob
 * fixed silly mistakes (printf problems), fixed inode size to 512 bytes.
 *
 * Revision 1.3  1999/04/23 02:20:09  fubob
 * cleaned up the csfssd.h interface.  using inode *numbers* instead of literal
 * *inodes* as the argument.  Moved auxillary code (freemap, sb) to
 * csfssd_aux.h
 *
 * Revision 1.2  1999/04/21 15:47:10  fubob
 * wow, my odl code was crap.  The interface is much cleaner now.  Automated
 * testing program in sdtest.c.  I brought the inode down to 256 bytes. Yeah!
 *
 * Revision 1.1  1998/11/20 03:47:13  fubobb
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "types.h"
#include "csfssd.h"
#include "csfssd_prot.h"
#include "disk.h"
#include "DEBUG.h"
#include "ASSERT.h"

#define TEST_MAX_LEN 30

void test_disk() {
  disk_t *disk;
  char filename[TEST_MAX_LEN];
  char block[CSFSSD_BLOCK_SIZE];
  u32 blocks,i,j,num,checksum,checksum2;
  sb_t sb;

  printf ("Create disk filename: ");
  scanf("%s",filename);

  printf ("\nNumber of 8KB blocks: ");
  scanf("%d",&blocks);

  printf ("\nOpening disk %s, blocks %u, size %lu kb\n",
	  filename,blocks,
	  (unsigned long)(blocks*CSFSSD_BLOCK_SIZE/1024));

  if (createDisk(filename, blocks) < 0) {
    printf ("createDisk failed\n");
    exit(1);
  }

  if ((disk = openDisk(filename, (u32 *)&blocks)) == NULL) {
    printf ("openDisk failed\n");
    exit(1);
  }

  printf ("Writing over all blocks.\n");

  memset((sb_t *)&sb, 0, sizeof(sb_t));

  if (CSFSSD_writeSB((sb_t *)&sb, disk) < 0) {
      printf ("WriteSB failed\n");
      exit(-1);
  }

  memset(block,0,CSFSSD_BLOCK_SIZE);
  memset(block, CSFSSD_SUPER_MAGIC, 16);
  memset(block+(CSFSSD_BLOCK_SIZE-16), CSFSSD_INODE_MAGIC, 16);

  for (i = 1; i < blocks; i++) {
    if (writeBlock(disk,i,(void *)block) < 0) {
      printf ("WriteBlock on block %d failed\n", i);
      exit(-1);
    }
  }

  closeDisk(disk);

  exit(0);
  /* Put random data on disk.  Verify checksum */
#ifdef 0
  checksum=0;
  srand((int) time((long *) 0));
  for (i=0; i<blocks; i++) {
    memset(block,0,CSFSSD_BLOCK_SIZE);
    for (j=0; j<=CSFSSD_BLOCK_SIZE/sizeof(u32)-1; j++) {
      num=rand();
      checksum^=num;
      memcpy((char *) (block+(j*sizeof(u32))),
	     &num,sizeof(u32));
    }
    ASSERT (writeBlock(disk,i,(void *)block)==0);
    /*    printf("Wrote block %lu\n",i);*/
  }
  
  closeDisk(disk);

  printf ("Writing checksum: %d\n",checksum);
  printf ("Reading disk, verifying checksum\n");

  if ((disk = openDisk(filename, (u32 *)&blocks)) == NULL) {
    printf ("openDisk failed2\n");
    exit(1);
  }

  checksum2=0;
  for (i=0; i<blocks; i++) {
    ASSERT (readBlock(disk,i,(void *)block)==0);
    /*    printf("Read block %lu\n",i);*/

    for (j=0; j<=CSFSSD_BLOCK_SIZE/sizeof(u32)-1; j++) {
      memcpy(&num,(char *)(block+(j*sizeof(u32))),
	     sizeof(u32));
      checksum2^=num;
    }
  }
  printf ("Write checksum %d  Read checksum: %d\n",checksum,checksum2);
  ASSERT(checksum==checksum2);
  syncDisk(disk); 

  /*  printf ("Attempting to reopen disk.\n");
  disk=openDisk(filename,blocks);
  ASSERT(disk>=0);

  memset(block,0,CSFSSD_BLOCK_SIZE);
  printf ("Attempting to write bad block nums.\n");
  ASSERT(writeBlock(disk,blocks,block)<0);
  ASSERT(writeBlock(disk,blocks+1,block)<0);

  printf ("Attempting to read bad block nums.\n");
  ASSERT(readBlock(disk,blocks,block)<0);
  ASSERT(readBlock(disk,blocks+1,block)<0); */

#endif

}

void test_sb()
{
  csfssd_t csfssd;

  printf("SB size = %d bytes\n",sizeof(sb_t));
  printf("Ondisk SB size = %d bytes\n",sizeof(sb_disk_t));
  printf("SB padding size = %d bytes\n",CSFSSD_SUPER_PADDING);

  printf("reading superblock of /tmp/disk\n");

  if (CSFSSD_mount(&csfssd, "/tmp/disk", 0) < 0) {
    printf("mount error\n");
  }

  
  printf("sb->magic = %d\n", csfssd.sb->magic);
  printf("sb->num_blocks = %d\n", csfssd.sb->num_blocks);
  printf("sb->num_inodes = %d\n", csfssd.sb->num_inodes);
  printf("sb->first_freemap_block = %d\n", csfssd.sb->first_freemap_block);
  printf("sb->first_inode_block = %d\n", csfssd.sb->first_inode_block);
  printf("sb->first_data_block = %d\n", csfssd.sb->first_data_block);
  printf("sb->mount_count = %d\n", csfssd.sb->mount_count);
  printf("sb->dirty = %d\n", csfssd.sb->dirty);
  printf("sb->lastchecked_time = %d\n", csfssd.sb->lastchecked_time);
  printf("sb->num_free_blocks = %d\n", csfssd.sb->num_free_blocks);
  printf("sb->num_free_inodes = %d\n", csfssd.sb->num_free_inodes);

  
}

void test_inode()
{
  printf("dp_t size = %d bytes\n",sizeof(csfssd_dp_t));
  printf("Inode size = %d bytes\n",sizeof(inode_t));
}



void main(int argc, char *argv[])
{
  debuglevel = DEBUG_VERBOSE;

  test_disk(); 
  test_sb();
  test_inode();

}
