#include "arpc/arpc.h"
#include "arpc/dns.h"
#include "csfssvc/nfs_prot.h"


#include "sfsclnt.h"
#include "csfscd.h"
#include "sfsvnode.h"
#include "sfsbc.h"

#include "upt/upt.h"

/* Feel free to edit, improve this code. It's here if you find it 
   useful - change it if you don't */

static int numbufs;
static int maxbufs = 100;
static int  fd;
static u_int disksize;

static HASH_TABLE(, buf) bc;
static TAILQ_HEAD(, buf) buf_free;

void bc_init(char *name)

{
  struct stat sb;
  fd = open(name, O_RDWR);

  if (fd < 0) {
    fatal ("Buffer cache initialization failed: %s\n", strerror(errno));
  }

  if (fstat(fd, &sb) < 0) {
    fatal ("stat failed: %s\n", strerror(errno));
  }

  disksize = sb.st_size / 512;

  HASH_INIT(&bc, maxbufs);
  TAILQ_INIT(&buf_free);
}

u_int buf_hash(struct buf *bp)

{
  return (bp->offset);
}

u_int buf_cmp(struct buf *bp1, u_int offset)

{
  return ((bp1->offset != offset) || (bp1->flags & BUF_INVAL));
}

static struct buf *getnewbuf(void)

{
  struct buf *bp = buf_free.tqh_first;
  char  *olddata = NULL;

  if (numbufs < maxbufs) {
    numbufs++;
    bp = xmalloc(sizeof(*bp));
  } else if (bp) {
    TAILQ_REMOVE(&buf_free, bp, free);
    HASH_REMOVE(&bc, bp, hash);
    olddata = bp->data;
  } else
    fatal ("Out of buffers\n");

  bzero(bp, sizeof(*bp));
  bp->data = olddata;
  mutex_init(bp->lock);
  return (bp);
}

struct buf *getblk(u_int secno, u_int size)

{
  struct buf *bp;

  HASH_LOOKUP (&bc, hash, nargs1, (secno),
	       , buf_cmp, bp);

  if (!bp) {
    bp = getnewbuf();
    bp->offset = secno;
    HASH_INSERT2 (&bc, hash, bp, buf_hash, (bp));
  } else {
    warn ("Found one in buffer cache\n");
    assert (bp->offset == secno);
    TAILQ_REMOVE(&buf_free, bp, free);
  }

  mutex_lock (bp->lock);
  bp->data = xrealloc(bp->data, size);
  bp->size = size;
  return (bp);
}

struct buf *bread(u_int secno, u_int size)

{
  struct buf *bp = getblk(secno, size);

  if (!(bp->flags & BUF_DONE)) {
    lseek(fd, secno * SECTOR_SIZE, SEEK_SET);

    warn ("Reading from disk\n");
    if (secno > disksize)
      warn ("Attempt to read invalid sector\n");

    read(fd, bp->data, size);

    bp->flags |= BUF_DONE;
  }

  return (bp);
}

struct buf *breadn(u_int offset, u_int size, u_int nblocks)

{
  return (bread(offset, size));
}

int bwrite(struct buf *bp)

{
  int ret;

  lseek (fd, bp->offset * SECTOR_SIZE, SEEK_SET);
  ret = write (fd, bp->data, bp->size);
  if (ret < 0)
    ret = errno;
  else
    ret = 0;

  bp->flags |= BUF_DONE;
  bp->flags &= ~BUF_DELWRI;
  brelse(bp);

  return (ret);
}

void binval(struct buf *bp)

{
  bp->flags |= BUF_INVAL;
}

void brelse(struct buf *bp)

{
  if (bp->flags & BUF_INVAL) {
    TAILQ_INSERT_HEAD(&buf_free, bp, free);
  } else {
    TAILQ_INSERT_TAIL(&buf_free, bp, free);
  }

  mutex_unlock(bp->lock);
}
