#include <rpc/rpc.h>
#include <string.h>
#include "libdebug/DEBUG.h"
#include "libdebug/ASSERT.h"
#include "csfsauth_prot.h"
#include <getopt.h>


#define UA_ADD_USER 1
#define UA_DEL_USER 2
#define UA_USER2UID 3
#define UA_UID2USER 4
#define UA_GETUSERPK 5
#define UA_SETUSERPK 6
#define UA_GETUSERPERMS 7
#define UA_SETUSERPERMS 8
#define UA_ADDGROUP 9
#define UA_DELGROUP 10
#define UA_GRP2GID 11
#define UA_GID2GRP 12
#define UA_GETGRPKEY 13
#define UA_CHGRPKEY 14
#define UA_ADDGRPMEM 15
#define UA_DELGRPMEM 16
#define UA_CHGRPOWNER 17

int main(int argc, char *argv[]) {
  CLIENT *authcl, *sdcl; /* Client handles */
  static char username[MAX_USERNAME_LEN+1];
  int i, proc=0;
  extern char *optargs;
  extern int optind;
  int uid_flag=0, 
      name_flag=0,
      key_flag=0,
      perms_flag=0,
      gid_flag=0,
      owner_flag=0;
  char *name, *key;
  int uid, perms, gid, owner;

  /* Handle a SIGINT gracefully -- shutdown the connection */

  /* Read arguments */

  /* establish setup connection with client daemon */

  if (!(authcl =
	clnt_create("127.0.0.1",CSFSAUTH_PROG,CSFSAUTH_VERS, "tcp"))) {
    clnt_pcreateerror("127.0.0.1");
    exit(1);
  }

  authcl->cl_auth = authunix_create_default();  /* UID, GID, etc */


  /* which procedure are we calling? */
  if(strncmp(argv[1],"auth_addUser", 12) == 0)
    proc = UA_ADD_USER;
  else if(strncmp(argv[1],"auth_delUser", 12) == 0)
    proc = UA_DEL_USER;
  else if(strncmp(argv[1],"auth_user2uid", 13) == 0)
    proc = UA_USER2UID;
  else if(strncmp(argv[1],"auth_uid2user", 13) == 0)
    proc = UA_UID2USER;
  else if(strncmp(argv[1],"auth_getUserPK", 14) == 0)
    proc = UA_GETUSERPK;
  else if(strncmp(argv[1],"auth_setUserPK", 14) == 0)
    proc = UA_SETUSERPK;
  else if(strncmp(argv[1],"auth_getUserPerms", 17) == 0)
    proc = UA_GETUSERPERMS;
  else if(strncmp(argv[1],"auth_setUserPerms", 17) == 0)
    proc = UA_SETUSERPERMS;
  else if(strncmp(argv[1],"auth_addGroup", 17) == 0)
    proc = UA_ADDGROUP;
  else if(strncmp(argv[1],"auth_delGroup", 17) == 0)
    proc = UA_DELGROUP;
  else if(strncmp(argv[1],"auth_group2gid", 17) == 0)
    proc = UA_GRP2GID;
  else if(strncmp(argv[1],"auth_gid2group", 17) == 0)
    proc = UA_GID2GRP;
  else if(strncmp(argv[1],"auth_getGroupKey", 17) == 0)
    proc = UA_GETGRPKEY;
  else if(strncmp(argv[1],"auth_changeGroupKey", 19) == 0)
    proc = UA_CHGRPKEY;
  else if(strncmp(argv[1],"auth_addGroupMem", 19) == 0)
    proc = UA_ADDGRPMEM;
  else if(strncmp(argv[1],"auth_delGroupMem", 19) == 0)
    proc = UA_DELGRPMEM;
  else if(strncmp(argv[1],"auth_changeGroupOwner", 21) == 0)
    proc = UA_CHGRPOWNER;
  else
    {
      printf("Usage: %s <procedure> <args>\n", argv[0]);
      printf("Usage: %s <procedure> ?\n", argv[0]);
      exit(1);
    }

  while((i=getopt(argc, argv, "u:n:k:p:g:o:?")) != -1)
    {
      switch(i) {
      case 'u':
	uid_flag = 1;
	uid = atoi(optarg);
	break;
      case 'n':
	name_flag = 1;
	name = malloc(sizeof(char)*MAX_USERNAME_LEN);
	strncpy(name,optarg,MAX_USERNAME_LEN);
	printf("Name = %s\n",name);
	break;
      case 'k':
	key_flag = 1;
	key = malloc(sizeof(char)*MAX_KEY_LEN);
	strncpy(key, optarg,MAX_KEY_LEN);
	printf("Key = %s\n",key);
	break;
      case 'p':
	perms_flag = 1;
	perms = atoi(optarg);
	break;
      case 'g':
	gid_flag = 1;
	gid = atoi(optarg);
	break;
      case 'o':
	owner_flag = 1;
	owner = atoi(optarg);
	break;
      }
    }
       

  switch(proc) 
    {
    case UA_ADD_USER:
      if(uid_flag & name_flag & key_flag) 
	{
	  csfsauth_adduserargs adduserargs;
	  csfsauth_adduserres *results;
	  bzero(&adduserargs, sizeof(csfsauth_adduserargs));
	  adduserargs.username= malloc(sizeof(char)*MAX_USERNAME_LEN);
	  strncpy(adduserargs.username,(csfsauth_user_t)name,MAX_USERNAME_LEN);
       	  adduserargs.perms = (csfsauth_perms_t)78;/*CSFSAUTH_USER_PERMS*/;
	  adduserargs.uid = (csfsauth_uid_t)uid;
	  adduserargs.pk.content.content_len = strlen(key)+1;
	  adduserargs.pk.content.content_val = malloc(sizeof(char)*MAX_KEY_LEN);
	  strncpy(adduserargs.pk.content.content_val, key, MAX_KEY_LEN);
	  results = adduser_1(&adduserargs, authcl);
	  if(results == NULL)
	    {
	      DEBUGP(DEBUG_CONCISE,("NULL results\n"));
	      exit(0);
	    }
	  if(results->status == CSFSAUTH_OK)
	    printf("ok\n");
	  else
	    printf("bad bad bad \n");
	}
      else
	{
	}
      break;
    case UA_DEL_USER:
      if(uid_flag)
	{
	  csfsauth_deluserargs args;
	  csfsauth_deluserres *result;
	  bzero(&args, sizeof(csfsauth_deluserargs));
	  args.uid = uid;
	  result = deluser_1(&args, authcl);
	  if(result == NULL)
	    {
	      DEBUGP(DEBUG_CONCISE, ("deluser_1 returned NULL\n"));
	    }
	  else if(result->status == CSFSAUTH_NOUSER)
	    printf("No such uid: %d\n", uid);
	}
      break;
    case UA_USER2UID:
      if(name_flag)
	{
	  csfsauth_user2uidres *result;
	  csfsauth_user2uidargs args;
	  bzero(&args, sizeof(csfsauth_user2uidargs));
	  args.username = malloc(sizeof(char) * MAX_USERNAME_LEN);
	  strncpy(args.username,name, MAX_USERNAME_LEN);
	  result = user2uid_1(&args, authcl);
	  if(result == NULL)
	    {
	      DEBUGP(DEBUG_CONCISE,("user2uid_1 returned NULL\n"));
	      exit(0);
	    }
	  if(result->status == CSFSAUTH_OK)
	    printf("UID = %d, USERNAME = %s\n",
		   result->csfsauth_user2uidres_u.uid,
		   name);
	  else if(result->status == CSFSAUTH_NOUSER)
	    printf("No such user: %s\n",name);
	}
      break;
    case UA_UID2USER:
      if(uid_flag)
	{
	  csfsauth_uid2userres *result;
	  csfsauth_uid2userargs args;
	  bzero(&args, sizeof(csfsauth_uid2userargs));
	  args.uid = uid;
	  result = uid2user_1(&args, authcl);
	  if(result == NULL)
	    {
	      DEBUGP(DEBUG_CONCISE,("uid2user_1 returned NULL\n"));
	      exit(0);
	    }
	  if(result->status == CSFSAUTH_OK)
	    printf("UID = %d, USERNAME = %s\n",
		   uid,
		   result->csfsauth_uid2userres_u.username);
	  else if(result->status == CSFSAUTH_NOUID)
	    printf("No such uid: %d\n",uid);
	}
      break;
    case UA_GETUSERPK:
      if(uid_flag)
	{
	  csfsauth_getuserpkres *result;
	  csfsauth_getuserpkargs args;
	  bzero(&args, sizeof(csfsauth_getuserpkargs));
	  args.uid = uid;
	  result = getuserpk_1(&args, authcl);
	  if(result == NULL)
	    {
	      DEBUGP(DEBUG_CONCISE,("getuserpk_1 returned NULL\n"));
	      exit(0);
	    }
	  if(result->status == CSFSAUTH_OK)
	    printf("UID = %d, KEY = %s\n",
		   uid,
		   result->csfsauth_getuserpkres_u.pk.content.content_val);
	  else if(result->status == CSFSAUTH_NOUID)
	    printf("No such uid: %d\n",uid);
	}
      break;
    case UA_SETUSERPK:
      if(uid_flag && key_flag)
	{
	  csfsauth_setuserpkres *result;
	  csfsauth_setuserpkargs args;
	  bzero(&args, sizeof(csfsauth_setuserpkargs));
	  args.uid = uid;
	  args.pk.content.content_len = MAX_KEY_LEN;
	  args.pk.content.content_val = malloc(sizeof(char)*MAX_KEY_LEN);
	  strncpy(args.pk.content.content_val, key, MAX_KEY_LEN);
	  result = setuserpk_1(&args, authcl);
	  if(result == NULL)
	    {
	      DEBUGP(DEBUG_CONCISE,("setuserpk_1 returned NULL\n"));
	      exit(0);
	    }
	  if(result->status == CSFSAUTH_OK)
	    printf("key set: UID = %d, KEY = %s\n",
		   uid,
		   key);
	  else if(result->status == CSFSAUTH_NOUID)
	    printf("No such uid: %d\n",uid);
	}
      break;
    case UA_GETUSERPERMS:
      if(uid_flag)
	{
	  csfsauth_getuserpermsargs args;
	  csfsauth_getuserpermsres *result;
	  bzero(&args, sizeof(csfsauth_getuserpermsargs));
	  args.uid = uid;
	  result = getuserperms_1(&args, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("perms = %d\n", result->csfsauth_getuserpermsres_u.perms);
	  else
	    printf("No such uid: %d\n",uid);
	}
      break;
    case UA_SETUSERPERMS:
      if(uid_flag && perms_flag)
	{
	  csfsauth_setuserpermsargs args;
	  csfsauth_setuserpermsres *result;
	  bzero(&args, sizeof(csfsauth_setuserpermsargs));
	  args.uid = uid;
	  args.perms = perms;
	  result = setuserperms_1(&args, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("perms set\n");
	  else
	    printf("No such uid: %d\n",uid);
	}
      break;
    case UA_ADDGROUP:
      if(gid_flag && name_flag && owner_flag)
	{
	  csfsauth_addgrpargs args;
	  csfsauth_addgrpres *result;
	  bzero(&args, sizeof(csfsauth_addgrpargs));
	  args.gid = gid;
	  args.owner = owner;
	  args.grpname = malloc(sizeof(char)*MAX_GROUPNAME_LEN);
	  strncpy(args.grpname, name, MAX_GROUPNAME_LEN);
	  result = addgrp_1(&args, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("group added\n");
	  else
	    printf("Duplicate gid or groupname\n");
	}
      break;
    case UA_DELGROUP:
      if(gid_flag)
	{
	  csfsauth_delgrpargs args;
	  csfsauth_delgrpres *result;
	  bzero(&args, sizeof(csfsauth_delgrpargs));
	  args.gid = gid;
	  result = delgrp_1(&args, authcl);
	  if(result->status == CSFSAUTH_NOGID)
	    printf("No such gid: %d\n", gid);
	  else if(result->status == CSFSAUTH_OK)
	    printf("Group deleted\n");

	}
      break;
    case UA_GRP2GID:
      if(name_flag)
	{
	  csfsauth_grp2gidargs args;
	  csfsauth_grp2gidres *result;
	  bzero(&args, sizeof(csfsauth_grp2gidargs));
	  args.grpname = malloc(sizeof(char)*MAX_GROUPNAME_LEN);
	  strncpy(args.grpname, name, MAX_GROUPNAME_LEN);
	  result = grp2gid_1(&args, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("Groupname = %s, gid = %d\n",
		   name, 
		   result->csfsauth_grp2gidres_u.gid);
	  else
	    printf("No such group: %s\n", name);
	}
      break;
    case UA_GID2GRP:
      if(gid_flag)
	{
	  csfsauth_gid2grpargs args;
	  csfsauth_gid2grpres *result;
	  bzero(&args, sizeof(csfsauth_gid2grpargs));
	  args.gid = gid;
	  result = gid2grp_1(&args, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("Groupname = %s, gid = %d\n",
		   result->csfsauth_gid2grpres_u.grpname,
		   gid);
	  else
	    printf("No such gid: %d\n", gid);
	}
      break;
    case UA_GETGRPKEY:
      if(gid_flag && uid_flag)
	{
	  csfsauth_getgrpkeyargs args;
	  csfsauth_getgrpkeyres *result;
	  bzero(&args, sizeof(csfsauth_getgrpkeyargs));
	  args.gid = gid;
	  args.uid = uid;
	  result = getgrpkey_1(&args, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("key = %s\n",
		   result->csfsauth_getgrpkeyres_u.key.content.content_val);
	  else 
	    printf("No such gid: %d, or uid: %d\n", gid, uid);
	}
      break;
    case UA_CHGRPKEY:
      if(gid_flag && uid_flag && key_flag)
	{
	  csfsauth_chgrpkeyargs arg;
	  csfsauth_chgrpkeyres *result;
	  bzero(&arg, sizeof(csfsauth_chgrpkeyargs));
	  arg.gid = gid;
	  arg.uid = uid;
	  arg.key.content.content_val = malloc(sizeof(char) * MAX_KEY_LEN);
	  arg.key.content.content_len = MAX_KEY_LEN;
	  strncpy(arg.key.content.content_val, key, MAX_KEY_LEN);
	  result = chgrpkey_1(&arg, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("Key set\n");
	  else
	    printf("Unable to set key.\nCheck uid and gid.\n");
	}
      break;
    case UA_ADDGRPMEM:
      if(gid_flag && uid_flag && key_flag)
	{
	  csfsauth_addgrpmemargs arg;
	  csfsauth_addgrpmemres *result;
	  bzero(&arg, sizeof(csfsauth_addgrpmemargs));
	  arg.gid = gid;
	  arg.uid = uid;
	  arg.pk.content.content_val = malloc(sizeof(char) * MAX_KEY_LEN);
	  arg.pk.content.content_len = MAX_KEY_LEN;
	  strncpy(arg.pk.content.content_val, key, MAX_KEY_LEN);
	  result = addgrpmem_1(&arg, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("User %d added to group %d.\n", uid, gid);
	  else
	    printf("Unable to add user.\nCheck uid and gid.\n");
	}
      break;
    case UA_DELGRPMEM:
      if(gid_flag && uid_flag)
	{
	  csfsauth_delgrpmemargs arg;
	  csfsauth_delgrpmemres *result;
	  bzero(&arg, sizeof(csfsauth_delgrpmemargs));
	  arg.gid = gid;
	  arg.uid = uid;
	  result = delgrpmem_1(&arg, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("User %d deleted from group %d.\n", uid, gid);
	  else
	    printf("Unable to delete user.\nCheck uid and gid.\n");
	}
      break;
    case UA_CHGRPOWNER:
      if(gid_flag && owner_flag)
	{
	  csfsauth_chgrpownerargs arg;
	  csfsauth_chgrpownerres *result;
	  bzero(&arg, sizeof(csfsauth_chgrpownerargs));
	  arg.uid = owner;
	  arg.gid = gid;
	  result = chgrpowner_1(&arg, authcl);
	  if(result->status == CSFSAUTH_OK)
	    printf("Owner changed\n");
	  else
	    printf("Owner not changed\n");
	}
      break;
    }
  clnt_destroy(authcl);

  return 0;
}





