/* $Id: fdpass.c,v 1.1 1998/10/21 18:33:02 fubobb Exp $ */

#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>

#include "private.h"

#ifdef HAVE_FDPASS

static char magicmsg[] = {'\175'};

int
sendfd (int dstfd, int fd)
{
  int res;
  struct msghdr mh;
  struct iovec iov[1];
#ifdef HAVE_CMSGHDR
  char cmhbuf[sizeof (struct cmsghdr) + sizeof (int)];
  struct cmsghdr *const cmh = (void *) cmhbuf;
  int * const fdp = (int *) (cmhbuf + sizeof (struct cmsghdr));
#else /* !HAVE_CMSGHDR */
  int fdpbuf;
  int *fdp = &fdpbuf;
#endif /* !HAVE_CMSGHDR */
  int flags;

  *fdp = fd;
  bzero (&mh, sizeof mh);
  mh.msg_iov = iov;
  mh.msg_iovlen = 1;
  iov[0].iov_base = magicmsg;
  iov[0].iov_len = sizeof (magicmsg);

#ifdef HAVE_CMSGHDR
  mh.msg_control = (void *) cmh;
  mh.msg_controllen = cmh->cmsg_len = sizeof (cmhbuf);
  cmh->cmsg_level = SOL_SOCKET;
  cmh->cmsg_type = SCM_RIGHTS;
#else /* !HAVE_CMSGHDR */
  mh.msg_accrights = (void *) fdp;
  mh.msg_accrightslen = sizeof (fdpbuf);
#endif /* !HAVE_CMSGHDR */

  flags = fcntl (dstfd, F_GETFL, 0);
  if (flags < 0)
    thread_panic ("sendfd: F_GETFL\n");
  if ((flags & O_NONBLOCK)
      && fcntl (dstfd, F_SETFL, (flags & ~O_NONBLOCK)) < 0)
    thread_panic ("sendfd: F_SETFD (~O_NONBLOCK)\n");

  res = sendmsg (dstfd, &mh, 0);

  if ((flags & O_NONBLOCK) && fcntl (dstfd, F_SETFL, flags) < 0)
    thread_panic ("sendfd: F_SETFD (restore)\n");

  return res < 0 ? -1 : 0;
}

int
recvfd (int srcfd)
{
  int res;
  struct msghdr mh;
  struct iovec iov[1];
  char msg[sizeof (magicmsg)];
#ifdef HAVE_CMSGHDR
  char cmhbuf[sizeof (struct cmsghdr) + sizeof (int)];
  struct cmsghdr *const cmh = (void *) cmhbuf;
  int * const fdp = (int *) (cmhbuf + sizeof (struct cmsghdr));
#else /* !HAVE_CMSGHDR */
  int fdpbuf;
  int *fdp = &fdpbuf;
#endif /* !HAVE_CMSGHDR */
  int flags;

  *fdp = -1;
  bzero (&mh, sizeof mh);
  mh.msg_iov = iov;
  mh.msg_iovlen = 1;
  iov[0].iov_base = msg;
  iov[0].iov_len = sizeof (msg);

#ifdef HAVE_CMSGHDR
  mh.msg_control = (void *) cmh;
  mh.msg_controllen = cmh->cmsg_len = sizeof (cmhbuf);
  cmh->cmsg_level = SOL_SOCKET;
  cmh->cmsg_type = SCM_RIGHTS;
#else /* !HAVE_CMSGHDR */
  mh.msg_accrights = (void *) fdp;
  mh.msg_accrightslen = sizeof (fdpbuf);
#endif /* !HAVE_CMSGHDR */

  flags = fcntl (srcfd, F_GETFL, 0);
  if (flags < 0)
    thread_panic ("sendfd: F_GETFL\n");
  if ((flags & O_NONBLOCK)
      && fcntl (srcfd, F_SETFL, (flags & ~O_NONBLOCK)) < 0)
    thread_panic ("sendfd: F_SETFD (~O_NONBLOCK)\n");

  errno = 0;
  res = recvmsg (srcfd, &mh, 0);

  if ((flags & O_NONBLOCK) && fcntl (srcfd, F_SETFL, flags) < 0)
    thread_panic ("sendfd: F_SETFD (restore)\n");

  if (res > 0 && (res != sizeof (msg)
		  || memcmp (msg, magicmsg, sizeof (magicmsg))))
    thread_warn ("recvfd: bad magic\n");

  return res == -1 ? -1 : *fdp;
}

#endif /* HAVE_FDPASS */
