/* $Id: hashtab.h,v 1.1 1998/10/21 18:33:05 fubobb Exp $ */

#ifndef _HASHTAB_H_
#define _HASHTAB_H_ 1

#include <stddef.h>
#include <sys/types.h>

struct hashtab_entry {
  void *hte_next;
  void **hte_prev;
  u_int hte_hval;
};

struct _hashtab {
  u_int ht_buckets;
  u_int ht_entries;
  void **ht_tab;
};

extern void _hashtab_grow (struct _hashtab *, u_int);

#define hashtab_decl(name, type, field)					\
struct name {								\
  struct _hashtab ht;							\
};									\
static inline void							\
name ## _init (struct name *_htp)					\
{									\
  struct _hashtab *htp = &_htp->ht;					\
  htp->ht_entries = 0;							\
  htp->ht_buckets = 0;							\
  htp->ht_tab = NULL;							\
}									\
static inline void							\
name ## _clear (struct name *_htp)					\
{									\
  struct _hashtab *htp = &_htp->ht;					\
  free (htp->ht_tab);							\
}									\
static inline void							\
name ## _insert (struct name *_htp, struct type *elm, u_int hval)	\
{									\
  struct _hashtab *htp = &_htp->ht;					\
  u_int bn;								\
  struct type *np;							\
									\
  if (++htp->ht_entries >= htp->ht_buckets)				\
    _hashtab_grow (htp, offsetof (struct type, field));			\
  elm->field.hte_hval = hval;						\
									\
  bn = hval % htp->ht_buckets;						\
  if ((np = htp->ht_tab[bn]))						\
    np->field.hte_prev = &elm->field.hte_next;				\
  elm->field.hte_next = np;						\
  elm->field.hte_prev = &htp->ht_tab[bn];				\
  htp->ht_tab[bn] = elm;						\
}									\
static inline void							\
name ## _delete (struct name *_htp, struct type *elm)			\
{									\
  struct _hashtab *htp = &_htp->ht;					\
  htp->ht_entries--;							\
  if (elm->field.hte_next)						\
    ((struct type *) elm->field.hte_next)->field.hte_prev		\
      = elm->field.hte_prev;						\
  *elm->field.hte_prev = elm->field.hte_next;				\
}									\
static inline struct type *						\
name ## _chain (struct name *_htp, u_int hval)				\
{									\
  struct _hashtab *htp = &_htp->ht;					\
  return htp->ht_buckets ? htp->ht_tab[hval % htp->ht_buckets] : 0;	\
}									\
static inline struct type *						\
name ## _next (struct type *elm)					\
{									\
  return elm->field.hte_next;						\
}									\
struct name


#endif /* _HASHTAB_H_ */
