/* $Id: io.hh,v 1.1 1998/10/21 18:33:08 fubobb Exp $ */

#ifndef _UPT_IO_H_
#define _UPT_IO_H_ 1

#include "private.h"

extern "C" {
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/uio.h>
}

#include "selmask.hh"

class iodesc {
  static mutex fdt_lock;
  static int fdt_size;
  static iodesc **fdt;

  static selmask wrm;     /* read/write masks for readwait/writewait */
  static selmask wwm;
  static selmask srm;     /* masks for select */
  static selmask swm;
  static selmask sem;
  static selmask orm;     /* output masks for select */
  static selmask owm;
  static selmask oem;
  static condition seldone;

  condition readok;
  condition writeok;

  int closed;
  int refcnt;

  iodesc () {}  // No default constructor, so hide it

protected:
  const int fdorig;
  int fd;

  iodesc (int fdn) : fdorig (fdn) { fd = fdn; closed = 0; refcnt = 0; }
  virtual ~iodesc () {}

  terr readwait (timespec * =0);
  terr writewait (timespec * =0);

public:
  static iodesc *lookup (int);
  void release ();
  int freefdno ();

  static void checkio (timeval *);

  virtual ssize_t readv (const iovec *iov, int iovcnt)
    { return ::readv (fd, iov, iovcnt); }
  virtual ssize_t writev (const iovec *iov, int iovcnt)
    { return ::writev (fd, iov, iovcnt); }

  virtual ssize_t recvmsg (msghdr *msg, int flags)
    { return ::recvmsg (fd, msg, flags); }
  virtual ssize_t sendmsg (const msghdr *msg, int flags)
    { return ::sendmsg (fd, msg, flags); }

  virtual int accept (sockaddr *addr, int *addrlen)
    { return ::accept (fd, addr, addrlen); }
  virtual int connect (const sockaddr *name, int namelen)
    { return ::connect (fd, name, namelen); }

  virtual off_t lseek (off_t offset, int whence)
    { return ::lseek (fd, offset, whence); }

  static int select (int, fd_set *, fd_set *, fd_set *, timeval *);
};

class iononblock : public iodesc {
  mutex wm;
  struct suio *uio;
public:
  iononblock (int nfd);
  virtual ssize_t readv (const iovec *, int);
  virtual ssize_t writev (const iovec *, int);
  virtual ssize_t recvmsg (msghdr *, int);
  virtual int accept (sockaddr *, int *);
  virtual int connect (const sockaddr *, int);
};

#endif /* !_UPT_IO_H_ */
