/* $Id: iononblock.cc,v 1.1 1998/10/21 18:33:09 fubobb Exp $ */

#include "io.hh"
extern "C" {
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "suio.h"
}

iononblock::iononblock (int nfd) : iodesc (nfd)
{
  uio = suio_alloc ();

  int flags;
  if ((flags = fcntl (fd, F_GETFL, 0)) >= 0)
    fcntl (fd, F_SETFL, flags|O_NONBLOCK);
}

#define fdwait(rw) do {				\
  switch (rw ## wait ()) {			\
  case TALERTED:				\
    errno = EINTR;				\
    return -1;					\
  case TFAILED:					\
    errno = EINVAL;				\
    return -1;					\
  default:					\
  break;					\
  }						\
} while (0)

ssize_t
iononblock::readv (const iovec *iov, int iovcnt)
{
  ssize_t ret;
  while ((ret = ::readv (fd, iov, iovcnt)) < 0
	 && errno == EAGAIN)
    fdwait (read);
  return ret;
}

ssize_t
iononblock::recvmsg (msghdr *msg, int flags)
{
  ssize_t ret;
  while ((ret = ::recvmsg (fd, msg, flags)) < 0
	 && errno == EAGAIN)
    fdwait (read);
  return ret;
}

ssize_t
iononblock::writev (const iovec *iov, int iovcnt)
{
  autolock l(wm);
  ssize_t ret = 0;
  for (int i = 0; i < iovcnt; i++)
    suio_print (uio, iov[i].iov_base, iov[i].iov_len);
  ret = uio->uio_resid;
  while (uio->uio_resid) {
    int n = ::writev (fd, uio->uio_iov, uio->uio_iovcnt);
    if (n > 0) {
      suio_rembytes (uio, n);
      ret += n;
    }
    else if (n == 0 || errno != EAGAIN) {
      suio_rembytes (uio, uio->uio_resid);
      return n;
    }
    else
      switch (writewait ()) {
      case TALERTED:
	if (ret == 0) {
	  errno = EINTR;
	  return -1;
	}
	return ret;
      case TFAILED:
	errno = EINVAL;
	return -1;
      default:
	break;
      }
  }
  return ret;
}

int
iononblock::connect (const sockaddr *addr, int addrlen)
{
  autolock l(wm);
  int ret;
  ret = ::connect (fd, addr, addrlen);
  if (ret < 0 && errno == EINPROGRESS) {
    writewait ();
    int esize = sizeof (errno);
    if (getsockopt (fd, SOL_SOCKET, SO_ERROR, (char *) &errno,
		    &esize) >= 0 || errno)
      ret = 0;
  }
  return ret;
}

int
iononblock::accept (sockaddr *addr, int *addrlen)
{
  int ret;

  while ((ret = ::accept (fd, addr, addrlen)) < 0
	 && errno == EWOULDBLOCK)
    readwait ();
  return ret;
}
