/* $Id: private.h,v 1.1 1998/10/21 18:33:19 fubobb Exp $ */

#ifndef _UPT_PRIVATE_H_
#define _UPT_PRIVATE_H_ 1

#define UPT_INTERNAL 1
#include "compat.h"
#include "upt.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "machdep.h"
#include "hashtab.h"

struct callout;

typedef struct tcb {
  struct tcb_machdep t_md;      /* Machine dependent state */
  char *t_stack;		/* Bottom of thread's stack */
  tid t_id;                     /* Thread ID */
  int t_errno;                  /* Saved errno */
  int t_ipl;                    /* Interrupts masked */
  char t_wantalert;		/* Thread should be awoken on alert */
  char t_alerted;		/* Alerted flag */
  struct callout *t_alarm;	/* Alarm clock for this thread */
  struct callout *t_wake;	/* Callout for nanosleep */
  struct tcbq *t_q;             /* Head of run/wait queue */
  TAILQ_ENTRY(tcb) t_ql;        /* Run/wait queue link */
  struct hashtab_entry t_idhash; /* Thread ID hash table entry */
  struct tcbq t_joiners;	/* List of users joining this thread */
} tcb;

extern u_int thread_stack_size;
extern int thread_want_clock;
extern tcb *thread_current;
extern tcb *thread_idle;
extern struct tcbq thread_runq;
extern struct tcbq thread_freeq;
extern int thread_initialized;
extern struct tcbq thread_sleepq;

struct timeval;
void thread_select (struct timeval *);

/* Core routines (see also upt.h) */
tcb *thread_id_lookup (tid);
void thread_run (tcb *);
tcb *thread_internal_fork (void (*) (void *), void (*));
void thread_clock (int);
void thread_awakenq (struct tcbq *);
terr thread_alertyield (void);

/* Misc routines */
void thread_waketime (struct timespec *);
void thread_spl_xmalloc_handler (void);
void thread_xmalloc_begin (void *);
#define warn thread_warn

/* Callout routines */
struct callout *callout_add (struct timespec *, void (*) (void *), void *);
void callout_free (struct callout *);
void callout_reserve (void);
int callout_check (struct timespec *);
int callout_next (struct timespec *);

/* For use by thread_md_init only: */
void thread_begin (tcb *, void (*) (void *), void *);

/* Machine dependent routines */
mdreg_t thread_md_stackcheck (void);
void thread_md_init (tcb *, void (*) (void *), void *);
void thread_md_switch (tcb *, tcb *);

/* Put a thread on a particular run/wait queue */
static inline void
thread_enq (struct tcbq *q, tcb *t)
{
  TAILQ_REMOVE (t->t_q, t, t_ql);
  t->t_q = q;
  TAILQ_INSERT_TAIL (t->t_q, t, t_ql);
}

/* Make a thread immediately runnable */
static inline void
thread_awaken (tcb *t)
{
  TAILQ_REMOVE (t->t_q, t, t_ql);
  t->t_q = &thread_runq;
  TAILQ_INSERT_HEAD (t->t_q, t, t_ql);
}

#define SPL_SIG 0x1		/* No signals, but xmalloc_handler */
#define SPL_MEM 0x2		/* No malloc handler */
#define SPL_HIGH 0x3		/* No nothing */

static inline int
splhigh (void) 
{
  int s = thread_current->t_ipl;
  thread_current->t_ipl |= SPL_HIGH;
  return s;
}
static inline int
splsig (void) 
{
  int s = thread_current->t_ipl;
  thread_current->t_ipl |= SPL_SIG;
  return s;
}
static inline void
splx (int s)
{
  int os = thread_current->t_ipl;
  thread_current->t_ipl = s;
  if (!(os & SPL_SIG) && thread_want_clock)
    thread_clock (0);
}

static inline void
wantalert (void)
{
  thread_current->t_wantalert = 1;
}

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* !_UPT_PRIVATE_H_ */
