/* $Id: redblack.h,v 1.1 1998/10/21 18:33:23 fubobb Exp $ */

#ifndef _REDBLACK_H_
#define _REDBLACK_H_ 1

#include <stddef.h>

#ifndef OPAQUECONTAINER_TYPEDEFED
#define OPAQUECONTAINER_TYPEDEFED 1
typedef struct __opaquecontainer *oc;
#endif /* OPAQUECONTAINER_TYPEDEFED */

#ifndef REDBLACK_DEBUG
#define REDBLACK_DEBUG 1
#endif /* REDBLACK_DEBUG */

enum redblack_color {INVALID, BLACK, RED};

struct redblack_entry {
  oc rbe_up;
  oc rbe_left;
  oc rbe_right;
  enum redblack_color rbe_color;
};

void redblack_insert (oc *, oc, const int, int (*) (oc, oc));
void redblack_delete (oc *, oc, const int);
oc redblack_minimum (oc, const int);
oc redblack_successor (oc, const int);
#if REDBLACK_DEBUG
void __redblack_check(oc *, const int, int (*) (oc, oc));
#define redblack_check(r, os, cmpfn) __redblack_check(r, os, cmpfn)
#else /* !REDBLACK_DEBUG */
#define redblack_check(r, os, cmpfn)
#endif /* !REDBLACK_DEBUG */

#define redblack_decl(name, type, field, cmpfn)				\
struct name {								\
  oc rb_root;								\
};									\
extern int cmpfn (struct type *, struct type *);			\
static inline int							\
_ ## name ## _cmp_oc (oc a, oc b)					\
{ /* This function never gets called, but warns of bad cmpfn type */	\
  return cmpfn ((struct type *) a, (struct type *) b);			\
}									\
static inline struct type *						\
name ## _first (struct name *t)						\
{									\
  return (struct type *)						\
    (t->rb_root								\
     ? redblack_minimum (t->rb_root, offsetof (struct type, field))	\
     : NULL);								\
}									\
static inline struct type *						\
name ## _next (struct type *n)						\
{									\
  return (struct type *)						\
    redblack_successor ((oc) n, offsetof (struct type, field));		\
}									\
static inline struct type *						\
name ## _root (struct name *t)						\
{									\
  return (struct type *) t->rb_root;					\
}									\
static inline struct type *						\
name ## _up (struct type *n)						\
{									\
  return (struct type *) n->field.rbe_up;				\
}									\
static inline struct type *						\
name ## _left (struct type *n)						\
{									\
  return (struct type *) n->field.rbe_left;				\
}									\
static inline struct type *						\
name ## _right (struct type *n)						\
{									\
  return (struct type *) n->field.rbe_right;				\
}									\
static inline struct type *						\
name ## _search (struct name *t,					\
		      int (*srchfn) (void *, struct type *),		\
		      void *srcharg)					\
{									\
  struct type *n, *l;							\
  int srchres;								\
  for (n = (struct type *) t->rb_root;					\
       n && (srchres = srchfn ((void *) srcharg, n));			\
       n = srchres < 0 ? name ## _left (n) : name ## _right (n))	\
    ;									\
  /* If there are duplicate keys, find the first one */			\
  for (; n && (l = name ## _left (n))					\
	 && !srchfn ((void *) srcharg, l); n = l)			\
    ;									\
  return n;								\
}									\
static inline void							\
name ## _insert (struct name *t, struct type *n)			\
{									\
  redblack_insert (&t->rb_root, (oc) n,					\
		   offsetof (struct type, field),			\
		   (int (*) (oc, oc)) cmpfn);				\
  redblack_check (&t->rb_root, offsetof (struct type, field),		\
		  (int (*) (oc, oc)) cmpfn);				\
}									\
static inline void							\
name ## _delete (struct name *t, struct type *n)			\
{									\
  redblack_delete (&t->rb_root, (oc) n,					\
		   offsetof (struct type, field));			\
  redblack_check (&t->rb_root, offsetof (struct type, field),		\
		  (int (*) (oc, oc)) cmpfn);				\
}									\
static inline void							\
name ## _init (struct name *t)						\
{									\
  t->rb_root = NULL;							\
}									\
struct name

#endif /* _REDBLACK_H_ */
