/* $Id: selmask.cc,v 1.1 1998/10/21 18:33:24 fubobb Exp $ */

#include "selmask.hh"

/* Population count (number of bits set) */
static const char bytepop[0x100] = {
  0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 3, 3, 4,
  2, 3, 3, 4, 3, 4, 4, 5, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
  2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 1, 2, 2, 3, 2, 3, 3, 4,
  2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
  2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6,
  4, 5, 5, 6, 5, 6, 6, 7, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
  2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5,
  3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
  2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6,
  4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
  4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8,
};

static inline int
popcnt (selmask::maskelm me)
{
  int r = 0;
  for (u_int i = 0; i < sizeof (me); i++) {
    r += bytepop[me&0xff];
    me >>= 8;
  }
  return r;
}

void
selmask::_extend (u_int fd)
{
  if (fd < nbits)
    return;

  int nb = 32 * ((fd + 32)/32);
  int nm = nb / nfdbits;
  maskelm *m = new maskelm[nm];
  memset (m, 0, nm * sizeof (*m));
  memcpy (m, mask, nmasks * sizeof (*m));

  delete[] mask;
  mask = m;
  nbits = nb;
  nmasks = nm;
  nbytes = nm * sizeof (*m);
}

void
selmask::set (const selmask &m)
{
  extend (m.nbits);
  for (u_int i = 0; i < m.nmasks; i++)
    mask[i] |= m.mask[i];
}

void
selmask::clr (const selmask &m)
{
  u_int n = nmasks < m.nmasks ? nmasks : m.nmasks;
  for (u_int i = 0; i < n; i++)
    mask[i] &= ~m.mask[i];
}

int
selmask::isset (const selmask &m) const
{
  u_int n = nmasks < m.nmasks ? nmasks : m.nmasks;
  for (u_int i = 0; i < n; i++)
    if (mask[i] & m.mask[i])
      return 1;
  return 0;
}

int
selmask::nset () const
{
  int r = 0;
  for (u_int i = 0; i < nmasks; i++)
    r += popcnt (mask[i]);
  return r;
}

void selmask::clrnot (const selmask &m)
{
  u_int i;
  u_int n = nmasks < m.nmasks ? nmasks : m.nmasks;
  for (i = 0; i < n; i++)
    mask[i] &= m.mask[i];
  for (; i < nmasks; i++)
    mask[i] = 0;
}

selmask::selmask (const selmask &m)
{
  nbits = m.nbits;
  nbytes = m.nbytes;
  nmasks = m.nmasks;
  mask = new maskelm[nmasks];
  memcpy (mask, m.mask, nbytes);
}

selmask &
selmask::operator= (const selmask &m)

{
  if (this != &m) {
    nbits = m.nbits;
    nbytes = m.nbytes;
    nmasks = m.nmasks;

    delete [] mask;

    mask = new maskelm[nmasks];
    memcpy (mask, m.mask, nbytes);
  }

  return (*this);
}

selmask::selmask (const fd_set *fds, u_int nfds)
{
  nbits = nbytes = nmasks = 0; mask = 0;

  if (fds && nfds) {
    extend (nfds - 1);
    memcpy (mask, fds, (nfds + 7)/8);
  }
}
