/* $Id: selmask.hh,v 1.1 1998/10/21 18:33:25 fubobb Exp $ */

#ifndef _SELMASK_HH_
#define _SELMASK_HH_ 1

extern "C" {
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
}

class selmask {
  typedef unsigned long maskelm;

  u_int nbits;
  u_int nbytes;
  u_int nmasks;

  maskelm *mask;

  static const int nfdbits = 8 * sizeof (maskelm);

  void _extend (u_int fd);
  void extend (u_int fd) {
    assert ((signed) fd >= 0);
    if (fd >= nbits) _extend (fd);
  }

public:
  selmask () { nbits = nbytes = nmasks = 0; mask = 0; }
  selmask (const fd_set *, u_int);
  selmask (const selmask &);
  ~selmask () { delete[] mask; }

  selmask &operator= (const selmask &);

  void zero () { memset (mask, 0, nbytes); }
  void set (u_int fd)
    { extend (fd); mask[fd/nfdbits] |= 1 << fd % nfdbits; }
  void set (const selmask &m);

  void clr (u_int fd)
    { extend (fd); mask[fd/nfdbits] &= ~(1 << fd % nfdbits); }
  void clr (const selmask &m);

  int isset (u_int fd) const
    { return fd < nbits ? (mask[fd/nfdbits] & (1 << fd % nfdbits)) : 0; }
  int isset (const selmask &m) const;

  int nset () const;
  void clrnot (const selmask &m);

  int size () const { return (nbits); }

  fd_set *fdset (u_int nfds)
    { if (!nbits) return 0; extend (nfds); return (fd_set *) mask; }
};

#endif /* _SELMASK_HH_ */
