/* $Id: sync.c,v 1.1 1998/10/21 18:33:33 fubobb Exp $ */

#include <assert.h>

#include "private.h"

void
mutex_init (mutex_t m)
{
  m->m_locked = 0;
  m->m_lockerid = 0;
  TAILQ_INIT (&m->m_waiters);
}

void
mutex_lock (mutex_t m)
{
  int s = splhigh ();

  /* Don't let a thread lock a mutex twice */
  assert (!(m->m_locked && m->m_lockerid == thread_current->t_id));

  while (m->m_locked && m->m_lockerid != thread_current->t_id) {
    thread_enq (&m->m_waiters, thread_current);
    thread_yield ();
  }
  m->m_locked = 1;
  m->m_lockerid = thread_current->t_id;
  splx (s);
}

terr
mutex_trylock (mutex_t m)
{
  terr ret;
  int s = splhigh ();

  if (m->m_locked)
    ret = TFAILED;
  else {
    m->m_locked = 1;
    m->m_lockerid = thread_current->t_id;
    ret = TOK;
  }

  splx (s);
  return ret;
}

void
mutex_unlock (mutex_t m)
{
  tcb *t;
  int s = splhigh ();

  /* Don't let a thread release a lock it doesn't own */
  assert (m->m_locked);
  assert (m->m_lockerid == thread_current->t_id);

  if ((t = m->m_waiters.tqh_first)) {
    /* For fairness, we immediately grant the lock to the thread
     * waiting the longest. */
    m->m_lockerid = t->t_id;
    thread_awaken (t);
  }
  else
    m->m_locked = 0;
  splx (s);
}

void
condition_init (condition_t cv)
{
  TAILQ_INIT (&cv->cv_waiters);
}

static inline terr
condition_genwait (mutex_t m, condition_t cv, struct timespec *tvp)
{
  terr ret;
  int s = splsig ();

  /* thread_waketime could sleep in xmalloc, so it must be called
   * before releasing the mutex. */
  if (tvp)
    thread_waketime (tvp);
  mutex_unlock (m);
  thread_enq (&cv->cv_waiters, thread_current);
  ret = thread_yield ();
  mutex_lock (m);
  splx (s);
  return ret;
}

terr
condition_timedwait (mutex_t m, condition_t cv, struct timespec *tvp)
{
  return condition_genwait (m, cv, tvp);
}

terr
condition_timedalertwait (mutex_t m, condition_t cv, struct timespec *tvp)
{
  wantalert ();
  return condition_genwait (m, cv, tvp);
}

void
condition_alert (condition_t cv)

{
  tcb *cur;

  int s = splhigh ();
  cur = cv->cv_waiters.tqh_first;

  while (cur) {
    tcb *next = cur->t_ql.tqe_next;
    thread_alert(cur->t_id);
    cur = next;
  }
  splx(s);
}

void
condition_signal (condition_t cv)
{
  int s = splhigh ();
  if (cv->cv_waiters.tqh_first)
    thread_awaken (cv->cv_waiters.tqh_first);
  splx (s);
}

void
condition_broadcast (condition_t cv)
{
  int s = splhigh ();
  if (cv->cv_waiters.tqh_first)
    thread_awakenq (&cv->cv_waiters);
  splx (s);
}

void
_once (once_t o, void (*fn) (void))
{
  int s;

  s = splhigh ();
  if (o->o_done) {
    splx (s);
    return;
  }
  o->o_done = 1;
  splx (s);
  fn ();
}
