/* $Id: tcore.c,v 1.1 1998/10/21 18:33:36 fubobb Exp $ */

#include <stdio.h>  /* XXX */

#include <errno.h>
#include <assert.h>
#include <signal.h>

#include <config.h>
#include "private.h"

tcb *thread_current;
tcb *thread_idle;
struct tcbq thread_runq;
struct tcbq thread_freeq;
struct tcbq thread_sleepq;


/*
 *  Allocate, free, and lookup threads.
 */

hashtab_decl (thread_idhash, tcb, t_idhash);
extern struct thread_idhash thread_idtable;

struct thread_idhash thread_idtable;
static tid thread_lastid;


/*
 *  Useful debugging fuctions
 */

int
tq_count (struct tcbq *q)
{
  tcb *t;
  int n = 0;

  for (t = q->tqh_first; t; t = t->t_ql.tqe_next)
    n++;
  return n;
}

void
tq_dump (struct tcbq *q)
{
  tcb *t;

  warn ("qdump: ");
  for (t = q->tqh_first; t; t = t->t_ql.tqe_next) {
    warn (" %03lu", (u_long) t->t_id);
    assert (t->t_q == q);
  }
  warn ("\n");
}

void
tq_verify (tcb *t)
{
  tcb *ti;

  for (ti = t->t_q->tqh_first; ti; ti = ti->t_ql.tqe_next)
    if (ti == t)
      return;
  assert (0);
}


/* Get the thread corresponding to a thread id */
tcb *
thread_id_lookup (tid id)
{
  tcb *t;
  if (!id)
    return NULL;
  t = thread_idhash_chain (&thread_idtable, id);
  while (t && t->t_id != id)
    t = thread_idhash_next (t);
  return t;
}

static tcb *
thread_alloc (void)
{
  tcb *t;

  if ((t = thread_freeq.tqh_first)) {
    TAILQ_REMOVE (&thread_freeq, t, t_ql);
  }
  else {
    if (!(t = xmalloc (sizeof (*t))))
      return NULL;
    if (! (t->t_stack = xmalloc (thread_stack_size))) {
      xfree (t);
      return NULL;
    }
  }

  while (!++thread_lastid || thread_id_lookup (thread_lastid))
    ;
  t->t_id = thread_lastid;
  t->t_errno = 0;
  t->t_ipl = SPL_HIGH;
  t->t_wantalert = 0;
  t->t_alerted = 0;
  t->t_alarm = NULL;
  t->t_wake = NULL;
  t->t_q = &thread_runq;
  TAILQ_INSERT_TAIL (&thread_runq, t, t_ql);
  thread_idhash_insert (&thread_idtable, t, t->t_id);
  TAILQ_INIT (&t->t_joiners);
  return t;
}

static void
thread_free (tcb *t)
{
  assert (!t->t_wake);
  if (t->t_alarm)
    callout_free (t->t_alarm);
  thread_idhash_delete (&thread_idtable, t);
  thread_enq (&thread_freeq, t);
}


/*
 *  Run threads.
 */

/* Run a particular thread */
inline void
thread_run (tcb *t)
{
  tcb *self = thread_current;
  mdreg_t sp = 0;

  thread_current->t_errno = errno;

  /* When first starting up, there won't be a stack */
  if (thread_current->t_stack)
    sp = thread_md_stackcheck ();

#if 0
  warn ("T%03lu:  S %p / %p\n", (u_long) self->t_id,
	(void *) sp, self->t_stack);
#endif

  thread_md_switch (thread_current, t);
  thread_current = self;

#if 0
  warn ("T%03lu:  R %p / %p\n", (u_long) self->t_id,
	(void *) sp, self->t_stack);

#endif

#if 0
  if (sp != thread_md_stackcheck ())
    thread_panic ("invalid stack pointer\n");
#else
  thread_md_stackcheck();
#endif

  errno = thread_current->t_errno;
}

/* Run the first available thread on the run queue */
terr
thread_yield (void)
{
  int ret = TOK;
  struct callout *co = thread_current->t_wake;
  tcb *t;
  int s;

  assert (thread_initialized);
  s = splhigh ();

  if (!thread_current->t_wantalert || !thread_current->t_alerted) {
    if (thread_runq.tqh_first == thread_current)
      thread_enq (&thread_runq, thread_current);
    if (!(t = thread_runq.tqh_first))
      t = thread_idle;
    if (t != thread_current)
      thread_run (t);
  }

  if (co && !thread_current->t_wake)
    ret = TTIMEDOUT;
  else if (co && thread_current->t_wake) {
    callout_free (thread_current->t_wake);
    thread_current->t_wake = NULL;
  }
  assert (!thread_current->t_wake);

  if (thread_current->t_wantalert && thread_current->t_alerted) {
    ret = TALERTED;
    thread_current->t_alerted = 0;
  }
  thread_current->t_wantalert = 0;

  splx (s);
  return ret;
}

void
thread_awakenq (struct tcbq *q)
{
  tcb *pt, *t, *nt;

  if ((pt = q->tqh_first)) {
    nt = pt->t_ql.tqe_next;
    pt->t_q = &thread_runq;
    TAILQ_INSERT_HEAD (&thread_runq, pt, t_ql);

    while ((t = nt)) {
      nt = t->t_ql.tqe_next;
      t->t_q = &thread_runq;
      TAILQ_INSERT_AFTER (q, pt, t, t_ql);
      pt = t;
    }

    TAILQ_INIT (q);
  }
}


/*
 * Timer interrupt.
 */

int thread_want_clock;

void
thread_clock (int sig)
{
  sigset_t mask;

  if (thread_current->t_ipl & SPL_SIG) {
    thread_want_clock = 1;
    return;
  }

  splhigh ();
  sigemptyset (&mask);
  sigaddset (&mask, SIGALRM);

  callout_check (NULL);

  sigprocmask (SIG_UNBLOCK, &mask, NULL);
  thread_want_clock = 0;
  thread_yield ();
  splx (0);
}


/*
 *  Delete threads
 */

/* Delete the current thread.  Wakeup anyone who has joined it. */
void
thread_exit (void)
{
  assert (thread_initialized);
  splhigh ();
  thread_awakenq (&thread_current->t_joiners);
  thread_free (thread_current);
  thread_yield ();
  thread_panic ("thread_exit\n");
}

/* Wait for a thread to exit. */
void
thread_join (tid id)
{
  tcb *t;
  int s = splhigh ();

  if ((t = thread_id_lookup (id))) {
    thread_enq (&t->t_joiners, thread_current);
    thread_yield ();
  }
  splx (s);
}


/*
 *  Create threads
 */

/* Begin execution of a new thread.  This is the first function to get
 * run on the new stack created by thread_fork.  fn and arg are the
 * arguments thread fork received. */
void
thread_begin (tcb *t, void (*fn) (void *), void (*arg))
{
  thread_current = t;

#if 0
  warn ("T%03lu:  B %p / %p\n", (u_long) t->t_id,
	(void *) thread_md_stackcheck, t->t_stack);
#endif

  errno = 0;
  splx (0);
  fn (arg);
  thread_exit ();
}

tcb *
thread_internal_fork (void (*fn) (void *), void (*arg))
{
  tcb *t;

  if (!(t = thread_alloc ()))
    return NULL;
  thread_md_init (t, fn, arg);
  return t;
}

tid
thread_fork (void (*fn) (void *), void (*arg))
{
  tcb *t;
  int id;
  int s = splhigh ();

  assert (thread_initialized);
  t = thread_internal_fork (fn, arg);
  if (!t)
    thread_panic ("thread_internal_fork failed\n");
  id = t->t_id;
  splx (s);
  return id;
}

/*
 * Misc.
 */

tid
thread_self (void)
{
  assert (thread_initialized);
  return thread_current->t_id;
}

terr
thread_testalert (void)
{
  int s = splhigh ();
  terr ret = thread_current->t_alerted ? TOK : TALERTED;
  thread_current->t_alerted = 0;
  splx (s);
  return ret;
}

void
thread_alert (tid id)
{
  tcb *t;
  int s = splhigh ();

  if ((t = thread_id_lookup (id))) {
    t->t_alerted = 1;
    if (t->t_wantalert && t->t_q != &thread_runq)
      thread_awaken (t);
  }
  splx (s);
}
