/* $Id: tmisc.c,v 1.1 1998/10/21 18:33:37 fubobb Exp $ */

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include <stdarg.h>
#include <signal.h>
#include <sys/time.h>
#ifndef HAVE_SYSCALL_H
#include <sys/syscall.h>
#else /* HAVE_SYSCALL_H */
#include <syscall.h>
#endif /* HAVE_SYSCALL_H */
#include <sys/wait.h>

#include "private.h"
#include "suio.h"
#include "hashtab.h"

const int errfd = 2;

static union {
  double dummy_alignment_hack;
  tcb dummy_thread;
} dummy;
tcb *thread_current = &dummy.dummy_thread;

int thread_initialized;
int thread_forked;
u_int thread_stack_size;


#if 0
/*
 * Unix Fork/waitpid
 */

struct cproc {
  struct hashtab_entry he;
  TAILQ_ENTRY (qe);
  pid_t pid;
  int status;
  int exited;
};

hashtab_decl(cpt, child, he) cptab;
static TAILQ_HEAD (, cproc) cpt_doneq;
static CONDITION_DECL (cpt_done);

static struct cproc *
cpt_lookup (pid_t pid)
{
  struct cproc *c;
  for (c = cpt_chain (&cptab, pid); c && c->pid != pid;c = cpt_next (c))
    ;
  return c;
}

pid_t
fork (void)
{
  struct cproc *c = xmalloc (sizeof (*c));
  int s = splhigh ();
  pid_t pid;

  mutex_lock (cptab_lock);
  pid = syscall (SYS_fork);
  switch (pid) {
  case -1:
    xfree (c);
    break;
  case 0:
    thread_forked = 1;
    thread_initialized = 0;
    break;
  default:
    c->pid = pid;
    c->exited = 0;
    cpt_insert (&cptab, c, pid);
    return pid;
  }
  mutex_unlock (cptab_lock);
  if (pid)
    splx (s);
  return pid;
}

pid_t
waitpid (pid_t wpid, int *status, int options)
{
  int s = splhigh ();
  struct cproc *c;

  while ((!(c = (wpid <= 0 ? cpt_doneq.tqh_first : cpt_lookup (cpt)))
	  || !c->exited) && !(options & WNOHANG)) {
    if (condition_wait (cpt_done) == TALERTED) {
      errno = EINTR;
      splx (s);
      return -1;
    }
  }

  if (c && c->exited) {
    wpid = c->pid;
    if (status)
      *status = c->status;
    cpt_delete (&cptab, c);
    TAILQ_DELETE (&cpt_doneq, c);
  }
  
  splx (s);
}
#endif

/* strerror
 *
 * Often there is non-reentrant locale crap in strerror, so provide a
 * dirt simple, reentrant one. */
char *
strerror (int err)
{
  extern char *sys_errlist[];
  extern int sys_nerr;

  if (err < 0 || err >= sys_nerr)
    return "unknown errno";
  else
    return sys_errlist[err];
}

static inline int
blockerr ()
{
  int fl = syscall (SYS_fcntl, errfd, F_GETFL, 0);
  if (fl >= 0)
    syscall (SYS_fcntl, errfd, F_SETFL, fl & ~O_NONBLOCK);
  return fl;
}
static inline void
unblockerr (int fl)
{
  if (fl >= 0)
    syscall (SYS_fcntl, errfd, F_SETFL, fl);
}

void
thread_panic (char *s)
{
  static const char pp[] = "Thread PANIC:  ";

  blockerr ();
  syscall (SYS_write, errfd, pp, sizeof (pp) - 1);
  syscall (SYS_write, errfd, s, strlen (s));
  abort ();
  exit (1);
}

void
thread_warn (const char *fmt, ...)
{
  static struct suio *uio;
  va_list ap;
  int s, fl;
  
  s = splhigh ();
  if (!uio)
    uio = suio_alloc ();
  va_start (ap, fmt);
  suio_vuprintf (uio, fmt, ap);
  va_end (ap);

  fl = blockerr ();
  syscall (SYS_writev, errfd, uio->uio_iov, uio->uio_iovcnt);
  unblockerr (fl);
  
  suio_rembytes (uio, uio->uio_resid);
  splx (s);
}

void
thread_spl_xmalloc_handler (void)
{
  thread_panic ("out of memory at splhigh\n");
}

static void
idleloop (void *_null)
{
  struct timespec next;

  for (;;) {
    if (callout_check(NULL))
      thread_yield();

    if (callout_next(&next)) {
      struct timeval tv;
      tv.tv_sec  = next.tv_sec;
      tv.tv_usec = next.tv_nsec / 1000;

      thread_select (&tv);
    } else {
      thread_select (NULL);
    }
    thread_yield ();
  }
}

#if 0
static void
init_clock (void)
{
  struct sigaction sa;
  struct itimerval itv;

  bzero (&sa, sizeof (sa));
  sa.sa_handler = thread_clock;
  sa.sa_flags = 0;
#ifdef SA_RESTART
  sa.sa_flags |= SA_RESTART;
#endif /* SA_RESTART */
  sigaction (SIGALRM, &sa, NULL);

  bzero (&itv, sizeof (itv));
  itv.it_value.tv_usec = itv.it_interval.tv_usec = 10000;
  setitimer (ITIMER_REAL, &itv, 0);
}
#endif

void
thread_initialize (void (*fn) (void *), void (*arg))
{
  tcb *t;
  
  if (thread_initialized)
    thread_panic ("thread_initialize called twice!");
  if (thread_forked)
    thread_panic ("thread initialize in child process");
  thread_initialized = 1;
  if (!thread_stack_size)
    thread_stack_size = 0xffe0;

  TAILQ_INIT (&thread_runq);
  TAILQ_INIT (&thread_freeq);
  TAILQ_INIT (&thread_sleepq);

  splhigh ();

  /* Create a special idle thread */
  t = thread_internal_fork (idleloop, NULL);
  assert (t);
  thread_idle = t;
  thread_enq (&thread_sleepq, t);

  /* Create emergency malloc thread */
  t = thread_internal_fork (thread_xmalloc_begin, NULL);
  assert (t);

  /* And launch the actual application */
  t = thread_internal_fork (fn, arg);
  assert (t);

#if 0
  init_clock ();
#endif
  thread_run (t);

  thread_panic ("thread_md_run returned.\n");
}
