/* $Id: tt.c,v 1.1 1998/10/21 18:33:38 fubobb Exp $ */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <errno.h>

#include "upt.h"
/* Create a TCP socket, bind it to a particular port, and call listen
 * for connections on it.  These are the three steps necessary before
 * clients can connect to a server.  */
int
tcpserv (int port)
{
  int s, n;
  struct sockaddr_in sin;

  /* The address of this server */
  bzero (&sin, sizeof (sin));
  sin.sin_family = AF_INET;
  sin.sin_port = htons (port);
  /* We are interested in listening on any and all IP addresses this
   * machine has, so use the magic IP address INADDR_ANY. */
  sin.sin_addr.s_addr = htonl (INADDR_ANY);

  s = socket (AF_INET, SOCK_STREAM, 0);
  if (s < 0) {
    perror ("socket");
    return -1;
  }
  /* Allow the program to run again even if there are old connections
   * in TIME_WAIT.  This is the magic you need to do to avoid seeing
   * "Address already in use" errors when you are killing and
   * restarting the daemon frequently.  */
  n = 1;
  if (setsockopt (s, SOL_SOCKET, SO_REUSEADDR, (char *)&n, sizeof (n)) < 0) {
    perror ("SO_REUSEADDR");
    close (s);
    return -1;
  }
  /* This function sets the close-on-exec bit of a file descriptor.
   * That way no programs we execute will inherit the TCP server file
   * descriptor. */
  fcntl (s, F_SETFD, 1);

  if (bind (s, (struct sockaddr *) &sin, sizeof (sin)) < 0) {
    fprintf (stderr, "TCP port %d: %s\n", port, strerror (errno));
    close (s);
    return -1;
  }
  if (listen (s, 5) < 0) {
    perror ("listen");
    close (s);
    return -1;
  }

  return s;
}

void
tt2 (void *_fd)
{
  int fd = (int) _fd;
  char buf[1024];
  int n;

  thread_alarm (5);
  while ((n = read (fd, buf, sizeof (buf))) > 0)
    write (1, buf, n);
  if (n < 0)
    perror ("read");
  else
    fprintf (stderr, "EOF\n");
  close (fd);
  thread_exit ();
}

void
tt (void *_null)
{
  int s = tcpserv (7979);
  if (s < 0) 
    exit (1);
  for (;;) {
    struct sockaddr_in sin;
    int sinlen = sizeof (sin);
    int c = accept (s, (struct sockaddr *) &sin, &sinlen);
    if (c < 0) {
      perror ("accept");
      exit (1);
    }
    thread_fork (tt2, (void *) c);
  }
}

int
main (int argc, char **argv)
{
  thread_initialize (tt, NULL);

  return 0;
}
