/* $Id: upt.h,v 1.1 1998/10/21 18:33:39 fubobb Exp $ */

#ifndef _UPT_H_
#define _UPT_H_ 1

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "malloc.h"
#include "syscall.h"
#include "compat.h"
#include "queue.h"

/* Error values returned by various thread routines */
typedef enum {
  TOK = 0,		/* No error */
  TFAILED = -1,		/* Generic failure */
  TALERTED = -2,	/* Interrupted by alert */
  TTIMEDOUT = -3,	/* Time limit exceeded */
} terr;

typedef unsigned long tid;

#undef private
#ifdef UPT_INTERNAL
#define private(x) x
#else /* !UPT_INTERNAL */
#define private(x) __private__##x##__
#endif /* !UPT_INTERNAL */

struct private(tcb);
TAILQ_HEAD (private(tcbq), private(tcb));

typedef struct _mutex {
  int private(m_locked);
  tid private(m_lockerid);
  struct private(tcbq) private(m_waiters);
} mutex_t[1];

typedef struct _condition {
  struct private(tcbq) private(cv_waiters);
} condition_t[1];

/* For declaring an initialized mutex/condition variable, you can use
 * these macros.  For example:
 *    static MUTEX_DECL (mylock); */
#ifdef UPT_INTERNAL
#define MUTEX_DECL(m) mutex_t m = {{0,0,{0,&m->m_waiters.tqh_first}}}
#define CONDITION_DECL(cv) condition_t cv = {{{0,&cv->cv_waiters.tqh_first}}}
#else /* !UPT_INTERNAL */
#define MUTEX_DECL(m) mutex_t m = \
  {{0,0,{0,&m->__private__m_waiters__.tqh_first}}}
#define CONDITION_DECL(cv) condition_t cv = \
  {{{0,&cv->__private__cv_waiters__.tqh_first}}}
#endif /* !UPT_INTERNAL */

typedef struct _once {
  int private (o_done);
} once_t[1];

#define ONCE_INIT {{0}}

void thread_initialize (void (*) (void *), void (*))
     __attribute__ ((noreturn));

tid thread_fork (void (*) (void *), void *);
void thread_exit (void) __attribute__ ((noreturn));
void thread_join (tid);
terr thread_yield (void);
tid thread_self (void);
terr thread_testalert (void);
void thread_alert (tid);
terr thread_sleeptil (struct timespec *);
terr thread_nanosleep (struct timespec *, struct timespec *);
int thread_alarm (int);

void thread_panic (char *) __attribute__ ((noreturn));
void thread_warn (const char *, ...) __attribute ((format (printf, 1, 2)));

void mutex_init (mutex_t);
void mutex_lock (mutex_t);
int mutex_trylock (mutex_t);
void mutex_unlock (mutex_t);

void condition_init (condition_t);
terr condition_timedwait (mutex_t, condition_t, struct timespec *);
terr condition_timedalertwait (mutex_t, condition_t, struct timespec *);
void condition_signal (condition_t);
void condition_broadcast (condition_t);
void condition_alert(condition_t);

static inline void
condition_wait (mutex_t m, condition_t cv)
{
  condition_timedwait (m, cv, NULL);
}
static inline terr
condition_alertwait (mutex_t m, condition_t cv)
{
  return condition_timedalertwait (m, cv, NULL);
}

void _once (once_t, void (*) (void));
static inline void
once (once_t o, void (*fn) (void))
{
  if (!o->private(o_done))
    _once (o, fn);
}

#undef private

#ifdef __cplusplus
}

class mutex {
  mutex_t m;
public:
  mutex () {mutex_init (m);}
  int trylock () {return mutex_trylock (m);}
  void lock () {mutex_lock (m);}
  void unlock () {mutex_unlock (m);}
  operator _mutex* () {return m;}
};

class autolock {
  _mutex *m;
  autolock ();			/* Hide default initializer */
 public:
  autolock (_mutex *mm) { m = mm; mutex_lock (m); }
  ~autolock () { mutex_unlock (m); }
};

// An idea:  LOCK (m) { ... }
// #define LOCK(m) if (autolock __lock_holder__(m), 1)

class condition {
  condition_t cv;
public:
  condition () {condition_init (cv);}
  terr wait (mutex &m, timespec *ts = 0)
    {return condition_timedwait (m, cv, ts);}
  terr alertwait (mutex &m, timespec *ts = 0)
    {return condition_timedalertwait (m, cv, ts);}
  void signal () {condition_signal (cv);}
  void broadcast () {condition_broadcast (cv);}
  void alert () {condition_alert (cv);}
  operator _condition* () {return cv;}
};

#endif /* __cplusplus */

#endif /* !UPT_H_ */
